
###################################################################################################
###############
###############                 Gene Fusion Ananlysis
###############
###############
###############                   Mingdong Liu
###############                      2016-9-22
###################################################################################################


#function to load a package

load.packages <- function(pkg){
  if(pkg %in% rownames(installed.packages())) {
    require(pkg, character.only = TRUE)
  }else if(!pkg %in% available.packages()) {
    cat("There is NOT such a package...")
  }else {
    install.packages(pkgs = pkg, repos = "http://cran.r-project.org")
    require(pkg, character.only = TRUE)
  }
}


############################################################################
###
###   GESD
###
############################################################################
##This is one-sided test
##Sample mean and sample SD are used

GESD <- function(obs, pvalue.junc.test = fusionarg.pvalue.junction, value.zscore = "NO", r=NA) {
  #### Define and Declare Variables ####
  n <- length(obs)
  if(is.na(r)){r <- floor(n/2)} # by default, set upper bound on number of outliers 'r' to 1/2 sample size
  R <- numeric(length=r) # test statistics for 'r' outliers
  lambda <- numeric(length=r) # critical values for 'r' outliers
  outlier_ind <- numeric(length=r) # removed outlier observation values
  outlier_val <- numeric(length=r) # removed outlier observation values
  m <- 0 # number of outliers
  obs_new <- obs # temporary observation values
  
  #### Find outliers ####
  for(i in 1:r){
    
    #### Compute test statistic ####
    if((value.zscore == "YES") | (value.zscore == "Y")){
      z <- obs_new # If Z-score is alrealy computed
    }else if((value.zscore == "NO") | (value.zscore == "N")){
      z <- (obs_new - mean(obs_new))/sd(obs_new) # Z-scores
    } else{
      print("ERROR! Inappropriate value for value.score=[YES|NO]")
    }
    
    max_ind <- which(z==max(z),arr.ind=T)[1] # in case of ties, return first one
    R[i] <- z[max_ind] # max Z-score
    outlier_val[i] <- obs_new[max_ind] # removed outlier observation values
    outlier_ind[i] <- which(obs_new[max_ind] == obs, arr.ind=T)[1] # index of removed outlier observation values
    obs_new <- obs_new[-max_ind] # remove observation that maximizes |x_i - x_mean|
    
    #### Compute critical values ####
    ##n_obs <- length(obs_new) # subset sample size
    #p <- 1 - pvalue.junc.test/(2*(n-i+1)) # probability
    p <- 1 - pvalue.junc.test/(n-i+1) # one-sided test
    t_pv <- qt(p,df=(n-i-1)) # Critical value from Student's t distribution
    lambda[i] <- ((n-i)*t_pv) / (sqrt((n-i-1+t_pv^2)*(n-i+1)))
    
    #### Find exact number of outliers: largest 'i' such that R_i > lambda_i ####
    # print(c(i, R[i], lambda[i]))
    # try ((R[i] > lambda[i]), finally <- print(c(i, R[i], lambda[i]))
    # )
    if(!is.na(R[i]) & !is.na(lambda[i])) { # qt can produce NaNs
      if (R[i] > lambda[i]) {
        m <- i
      }
      # else {
      # break
      # }
      
    }
  }
  
  vals <- data.frame(NumOutliers=1:r, TestStatistic=R, CriticalValue=lambda)
  # print(vals)
  
  #### Rank outlier observations ####
  outlier_rank <- numeric(length=n)
  if (m > 0) {
    for (i in 1:m) {
      # outlier_rank[outlier_ind[i]] <- i
      outlier_rank[which(obs==outlier_val[i])] <- i
    }
  }
  num_outliers <- sum(outlier_rank != 0) #14 and 25 missing
  res <- c(num_outliers, outlier_rank)
  names(res) <- c("Total", names(obs))
  
  return(res)	
}

############################################################################
###
###   robustGESD
###
############################################################################
##This is one-sided test
##Sample median and sample SD are used
robustGESD <- function(obs, pvalue.junc.test = fusionarg.pvalue.junction, value.zscore= "NO", r=NA) {
  #### Define and Declare Variables ####
  n <- length(obs)
  
  if(is.na(r)){r <- floor(n/2)} # by default, set upper bound on number of outliers 'r' to 1/2 sample size
  R <- numeric(length=r) # test statistics for 'r' outliers
  lambda <- numeric(length=r) # critical values for 'r' outliers
  outlier_ind <- numeric(length=r) # removed outlier observation values
  outlier_val <- numeric(length=r) # removed outlier observation values
  m <- 0 # number of outliers
  obs_new <- obs # temporary observation values
  
  #### Find outliers ####
  for(i in 1:r){
    
    mymedian <- median(obs_new)
    
    #### Compute test statistic ####
    if((value.zscore == "YES") | (value.zscore == "Y")){
      z <- obs_new # If Z-score is alrealy computed, select real z score
    }else if((value.zscore == "NO") | (value.zscore == "N")){
      z <- (obs_new - mymedian)/sd(obs_new) # Z-scores, select real z score for one-sided test
    } else{
      print("ERROR! Inappropriate value for value.score=[YES|NO]")
    }
    
    max_ind <- which(z==max(z),arr.ind=T)[1] # in case of ties, return first one
    R[i] <- z[max_ind] # max Z-score
    outlier_val[i] <- obs_new[max_ind] # removed outlier observation values
    outlier_ind[i] <- which(obs_new[max_ind] == obs, arr.ind=T)[1] # index of removed outlier observation values
    obs_new <- obs_new[-max_ind] # remove observation that maximizes |x_i - x_mean|
    
    #### Compute critical values ####
    ##n_obs <- length(obs_new) # subset sample size
    #p <- 1 - pvalue.junc.test/(2*(n-i+1)) # probability
    p <- 1 - pvalue.junc.test/(n-i+1) # one-sided test
    t_pv <- qt(p,df=(n-i-1)) # Critical value from Student's t distribution
    lambda[i] <- ((n-i)*t_pv) / (sqrt((n-i-1+t_pv^2)*(n-i+1)))
    
    #### Find exact number of outliers: largest 'i' such that R_i > lambda_i ####
    # print(c(i, R[i], lambda[i]))
    # try ((R[i] > lambda[i]), finally <- print(c(i, R[i], lambda[i]))
    # )
    if(!is.na(R[i]) & !is.na(lambda[i])) { # qt can produce NaNs
      if (R[i] > lambda[i]) {
        m <- i
      }
      else {
        break
      }
      
    }
  }
  
  vals <- data.frame(NumOutliers=1:r, TestStatistic=R, CriticalValue=lambda)
  # print(vals)
  
  #### Rank outlier observations ####
  outlier_rank <- numeric(length=n)
  if (m > 0) {
    for (i in 1:m) {
      # outlier_rank[outlier_ind[i]] <- i
      outlier_rank[which(obs==outlier_val[i])] <- i
    }
  }
  num_outliers <- sum(outlier_rank != 0) #14 and 25 missing
  res <- c(num_outliers, outlier_rank)
  names(res) <- c("Total", names(obs))
  
  return(res)	
}


############################################################################
###
###   GeneralizedRobustGESD
###
############################################################################
##This is one-sided test
##Sample median and sample SD are used
GeneralizedRobustGESD <- function(obs, pvalue.junc.test = fusionarg.pvalue.junction, value.zscore = "NO",coef = c(1.2172288, -0.1250655), r= NA, floor = 0.5095818) {
  #### Define and Declare Variables ####
  n <- length(obs)
  
  if(is.na(r)){r <- floor(n/2)} # by default, set upper bound on number of outliers 'r' to 1/2 sample size
  R <- numeric(length=r) # test statistics for 'r' outliers
  lambda <- numeric(length=r) # critical values for 'r' outliers
  outlier_ind <- numeric(length=r) # removed outlier observation values
  outlier_val <- numeric(length=r) # removed outlier observation values
  m <- 0 # number of outliers
  obs_new <- obs # temporary observation values
  
  #### Find outliers ####
  for(i in 1:r){
    
    mymedian <- median(obs_new)
    thissd <- coef[1] + coef[2] * mymedian
    
    thissd <- pmax(floor, thissd)#set floor for predicted SD
    
    #### Compute test statistic ####
    if((value.zscore == "YES") | (value.zscore == "Y")){
      z <- obs_new # If Z-score is alrealy computed, select real z score
    }else if((value.zscore == "NO") | (value.zscore == "N")){
      z <- (obs_new - mymedian)/thissd # Z-scores, select real z score for one-sided test
    } else{
      print("ERROR! Inappropriate value for value.score=[YES|NO]")
    }
    
    max_ind <- which(z==max(z),arr.ind=T)[1] # in case of ties, return first one
    R[i] <- z[max_ind] # max Z-score
    outlier_val[i] <- obs_new[max_ind] # removed outlier observation values
    outlier_ind[i] <- which(obs_new[max_ind] == obs, arr.ind=T)[1] # index of removed outlier observation values
    obs_new <- obs_new[-max_ind] # remove observation that maximizes |x_i - x_mean|
    
    #### Compute critical values ####
    ##n_obs <- length(obs_new) # subset sample size
    #p <- 1 - pvalue.junc.test/(2*(n-i+1)) # probability
    p <- 1 - pvalue.junc.test/(n-i+1) # one-sided test 
    t_pv <- qt(p,df=(n-i-1)) # Critical value from Student's t distribution
    lambda[i] <- ((n-i)*t_pv) / (sqrt((n-i-1+t_pv^2)*(n-i+1)))
    
    #### Find exact number of outliers: largest 'i' such that R_i > lambda_i ####
    # print(c(i, R[i], lambda[i]))
    # try ((R[i] > lambda[i]), finally <- print(c(i, R[i], lambda[i]))
    # )
    if(!is.na(R[i]) & !is.na(lambda[i])) { # qt can produce NaNs
      if (R[i] > lambda[i]) {
        m <- i
      }else{
        break
      }
    }
  }
  
  vals <- data.frame(NumOutliers=1:r, TestStatistic=R, CriticalValue=lambda)
  # print(vals)
  
  #### Rank outlier observations ####
  outlier_rank <- numeric(length=n)
  if (m > 0) {
    for (i in 1:m) {
      # outlier_rank[outlier_ind[i]] <- i
      outlier_rank[which(obs==outlier_val[i])] <- i
    }
  }
  num_outliers <- sum(outlier_rank != 0) #14 and 25 missing
  res <- outlier_rank
  names(res) <- names(obs)
  
  return(res)	
}

############################################################################
###
###   OutlierByNB
###
############################################################################
##  Directly compare the log2 count to the empirical CDF obtained from negative binomial distribution with size =7 and mu = pmax(3, median(data)).

outlierByNB <- function(obs, alpha = 0.001,r = NA) {
  if (is.na(r)) r = floor(length(obs)/2)
  outlier.idx <- c()
  obs_new <- obs
  res <- rep(0, length(obs))
  names(res) <- names(obs)
  for (i in 1:r){
    mymedian <- pmax(3,median(obs_new))
    
    #simulate NB distribution
    x <- rnbinom(n = 1000, size = 7, mu = 2^mymedian)
    x <- log2(0.5 + x)
    x.ecdf <- ecdf(x) 
    critical.value <- quantile(x, 1-alpha/r)
    
    if (max(obs_new) > critical.value){
      outlier.idx <- c(outlier.idx, which(obs ==  max(obs_new)))
      res[which(obs ==  max(obs_new))] <- i
      obs_new <- obs_new[-which(obs_new == max(obs_new))]
    }else break
  }
  return(res)	
}



##################################################
###   readFusionData
##################################################

## The funciton of readFusionData takes 3 argments: 1) the raw dataset output from RCC files, with the probe names 
##    on the columns and the sample IDs on the row. 2) probe annotation: the column "Fusion.Probe.Type" specifies 
##    the type of probe (either end probe (5p or 3p) or the junction probes; the column "Fusion.Base" specifies 
##    the gene to be investigated in the fusion analysis; The column of "ProbeID" specifies the names of probes.
## The function of readFusionData output a list which include: 
##        1) raw: the log-transformed raw dataset; 
##        2) endProbes: the list of end probes under the gene; 
##        3) junctionProbes: the list of junction probes under the gene;
##        4) pvalue: a list with genes as the names, a data frame under gene names include:
##                i) statistic.t from t test; 
##                ii) p.value from t test; 
##                iii) classification from clustering with end probe data; 
##                iv) lm.pvalue: p values from linear regression; 
##                v) classification based on p.value of t test.
##        5) junction.ttest: p values from outlier test;
##        6) junction.cluster: if there is a reference dataset, classification is present;
##        7) mysample: sampleID in the input dataset.

readFusionData <- function(f.data, 
                           f.pannot, 
                           junctionRef = NULL,
                           hk.QC.threshold = 30, #this is the minimum count for any HK probes, otherwise, QC failure
                           pvalue.ep.ttest = fusionarg.pvalue.endprobe, #this is the p value for t test of end probes
                           pvalue.ep.lmtest = 0.0001, #this is the p value for lm test of end probes
                           pvalue.junc.test = fusionarg.pvalue.junction, #pvalue threshold for GESD test
                           junc.pvalue.threshold = 0.0001, #this is the pvalue threshold for fusion calling from outlier test
                           fusion.min.ref.cluster = 10, #this is the minimum sample size for reference dataset to run cluster analysis
                           fusion.min.sample.size = 20, #this is the minimum sample size of final dataset to do cluster analysis
                           fusion.pvalue.threshold.less.probes = 0.005,#this is  the p value threshold for genes with less than 3 junction probes, such as NTRK1
                           QC.excluded = FALSE, #this is the control whether or not to exclude QC failed sample from analysis
                           ref.sample.imbalanced = NA, #this specifies the reference sample which will be used for linear regression
                           sd.from.water.lane = 1.174,
                           end.cluster.arg = FALSE,#turn off clustering on imbalance probes
                           myhk,
                           logfile = log,
                           ...){
  
  sampleID <- rownames(f.data)
  
  cat("----------------------------", sep = "\n",file = logfile, append = TRUE)
  cat("Fusion Analysis", sep = "\n",file = logfile, append = TRUE)
  cat("----------------------------", sep = "\n",file = logfile, append = TRUE)
  
  
  # Verify f.pannot
  #----------------
  required.f.pannot.cols <- c("ProbeID","Analyte.Type", "Fusion.Probe.Type", "Fusion.Base")
  if(any(!required.f.pannot.cols %in% colnames(f.pannot)))
    stop(paste("required",paste(required.f.pannot.cols[!required.f.pannot.cols %in% colnames(f.pannot)],collapse = ","),"column(s) missing from fusion probe annotations. Fusion analysis can't be completed. Make sure the proper probe annotation for fusion analysis is uploaded\n"))
  
  
  # if(any(!c("3p","5p") %in% f.pannot[,"Fusion.Probe.Type"]))
  #   stop("Required 3p/5p information is missing from probe annotation. Fusion analysis can not be completed. Make sure the proper probe annotation for fusion analysis is uploaded\n")
  # 
  
  if(!"fusion" %in% tolower(f.pannot[,"Analyte.Type"]))
    stop("No fusion probe is identified in the assay. Fusion analysis can not be completed. Make sure the proper probe annotation for fusion analysis is uploaded\n")
  
  
  if (!is.null(junctionRef)) {
    cat("References for junction probes will be added.",  file = logfile, sep = "\n",append = TRUE)
    
    end.cluster.arg = TRUE
    
    junction.ref <- read.csv(junctionRef, header = T, stringsAsFactors = F, na.strings = "", check.names = FALSE)
    junction.ref1 <- as.data.frame(t(junction.ref))
    colnames(junction.ref1) <- as.character(unlist(junction.ref[,1]))
    junction.ref1$sampleID <- rownames(junction.ref1)
    junction.ref1 <- junction.ref1[-1,]
    mydata.merge <- merge(rawdata, junction.ref1, by = intersect(names(mydata), names(junction.ref1)), all = T, sort = FALSE)
    rownames(mydata.merge) <- mydata.merge$sampleID
    mydata.merge <- mydata.merge[,-which(names(mydata.merge) == "sampleID")]
    mydata <- as.data.frame(mydata.merge)
  }else{
    cat("There is no reference samples for junction probes." , file = logfile, sep = "\n", append = TRUE)
  }
  
  #get the gene and probe information from annotation file  
  
  #imbalance probes
  geneid.end <- unique(f.pannot$Fusion.Base[toupper(f.pannot$Analyte.Type) == "FUSION" & f.pannot$Fusion.Probe.Type %in% c("3p","5p")])
  endProbes <- NULL
  
  if (length(geneid.end) == 0) {
    end.probe.arg <- FALSE # turn on test on imbalance probes
    cat("No imbalance Probe found!", file = logfile, sep = '\n', append = TRUE)
  } else {
    end.probe.arg <- TRUE # This controls if there is any end probes, such as leukemia fusion panel
    end.probes <- vector(mode = "list", length = length(geneid.end))
    names(end.probes) <- geneid.end
    endProbes = end.probes
    for (gene in toupper(geneid.end)){
      end.probes[[gene]] <- f.pannot$ProbeID[f.pannot$Fusion.Probe.Type %in% c("3p","5p") & toupper(f.pannot$Fusion.Base) == gene]
      endProbes[[gene]] <- list(p3 = f.pannot$ProbeID[f.pannot$Fusion.Probe.Type == "3p" & toupper(f.pannot$Fusion.Base) == gene],p5 = f.pannot$ProbeID[f.pannot$Fusion.Probe.Type == "5p" & toupper(f.pannot$Fusion.Base) == gene])
    }
  }
  
  #junction probes
  geneid.fusion <- unique(f.pannot$Fusion.Base[toupper(f.pannot$Analyte.Type) == "FUSION" & toupper(f.pannot$Fusion.Probe.Type) == "FUSION"])
  junction.probes <- NULL
  
  if (length(geneid.fusion) == 0){
    
    cat("No junction probe found!", file = logfile, sep ="\n", append = TRUE)
  } else {
    junc.probe.arg = TRUE #turn on tests on junction probes
    
    #junction.probes is the list of junction probes under each gene
    junction.gene <- unique(f.pannot$Fusion.Base[toupper(f.pannot$Analyte.Type) == "FUSION" & toupper(f.pannot$Fusion.Probe.Type) == "FUSION"])
    junction.probes <- vector(mode = "list", length = length(junction.gene))
    names(junction.probes) <- junction.gene
    
    for (gene in junction.gene){
      junction.probes[[gene]] <- as.character(as.character(f.pannot$ProbeID[f.pannot$Fusion.Base == gene & toupper(f.pannot$Fusion.Probe.Type) == "FUSION"]))
    }
    
    #myfusion is the list of junction probes detected
    myfusion <- vector(mode = "list", length = length(geneid.fusion))
    names(myfusion) <- geneid.fusion
  }
  
  mygene <- union(geneid.end, geneid.fusion)
  
  mydata <- f.data
  
  #log-transformation of raw data
  suppressWarnings(mydata.log.all <- mydata * NA)
  mydata.log.all.unadjusted <- mydata.log.all
  
  #mydata <- as.matrix(mydata)
  for (i in 1:ncol(mydata)){
    mydata.log.all[,i] <- pmax(log2(as.numeric(mydata[,i]) + 0.5),3)
    mydata.log.all.unadjusted[,i] <- log2(as.numeric(mydata[,i]) + 0.5)
  }
  
  cat("Counts are sucessfully log(2) transformed!", file = logfile, sep = "\n", append = TRUE)
  
  #get the probes for housekeepers
  if (length(myhk) != 0){
    cat("Probes for housekeepers found.", file = logfile, sep = "\n", append = TRUE)
  }
  
  #########################################################
  ##
  ##      quality control and normalization
  ##
  #########################################################
  
  # Set the default QC parameters
  mysample.QC.failed <- NA
  mysample.QC.pass <- rownames(mydata.log.all)
  mydata.log <- mydata.log.all
  mydata.log.unadjusted <- mydata.log.all.unadjusted
  
  # If housekeeping genes exist perform HK qc
  if(length(myhk)>0){
    rawdata.hk.t <- mydata.log.all[, myhk]
    
    tmp <- which(as.logical(apply(rawdata.hk.t,1,function(x) all(x < log2(hk.QC.threshold))))) #all HK counts of a sample less than threshold
    if (length(tmp) !=0){
      mysample.QC.failed <- rownames(mydata.log.all)[tmp]
      mysample.QC.pass <- rownames(mydata.log.all)[-tmp]
      if(QC.excluded){
        mydata.log <- mydata.log.all[-tmp,]
        mydata.log.unadjusted <- mydata.log.all.unadjusted[-tmp,]
      }
    }
    
  }
  
  # Initialized the subset of data correspondign to end and junction probes
  mydata.junction <- mydata.end <- NULL
  
  #################################################
  #####
  #####    endprobes
  #####
  #################################################
  mypvalue <- NULL
  if (end.probe.arg){
    mydata.end <- mydata.log.all[,as.character(unlist(end.probes)), drop = F]
    #create a list to store results
    allgene <- as.character(unlist(geneid.end))
    allgene.end <- vector(mode = "list", length = length(allgene))
    names(allgene.end) <- allgene
    mypvalue <- allgene.end
    
    for (gene in geneid.end){
      
      #subsetting by gene
      mydata.subset <- mydata.end[,as.character(unlist(end.probes[[gene]]))]
      
      #for end probes,3 tests will be used: t test, liner regression and clustering
      p3 <- mydata.subset[,as.character(unlist(f.pannot$ProbeID[toupper(toupper(f.pannot$Analyte.Type)) == "FUSION" & f.pannot$Fusion.Base == gene & toupper(f.pannot$Fusion.Probe.Type) == "3P"]))]
      p5 <- mydata.subset[,as.character(unlist(f.pannot$ProbeID[toupper(f.pannot$Analyte.Type) == "FUSION" & f.pannot$Fusion.Base == gene & toupper(f.pannot$Fusion.Probe.Type) == "5P"]))]
      
      #create a list to store fusion result from an individual gene
      myresult <- as.data.frame(matrix(NA,nrow = length(sampleID), ncol = 2))
      rownames(myresult) <- sampleID
      colnames(myresult) <- c("statistic", "p.value")
      
      ##########################################
      ####
      ####      t test
      ####
      ##########################################
      for(i in 1:length(sampleID)){
        if (!sampleID[i] %in% rownames(mydata.subset)) next 
        if (length(which(is.na(p3[i,]))) == ncol(p3) | length(which(is.na(p5[i,]))) == ncol(p5))  next # if all are missing
        
        #if the mean of p5 probes is greater the mean of p3 probes, set the t statistic to be -100 and p value to be 1.
        if(mean(as.numeric(p3[sampleID[i],])) <= mean(as.numeric(p5[sampleID[i],])) | mean(as.numeric(p3[sampleID[i],])) < 5) #if the mean of p3 probes less than p5 probes or the mean of p3 probes is less than 5 (32 counts),set p = 1
        {myresult[sampleID[i],] <- c(-100,1)
        }else {
          
          myfit <- t.test(p5[sampleID[i],], p3[sampleID[i],], var.equal = FALSE, paired = FALSE)# t.test for two group means with unequal variance
          #cat(myttest$statistic, myttest$p.value)
          myresult[sampleID[i],] <- c(myfit$statistic, myfit$p.value)
        }
      }
      
      mypvalue[[gene]] <- myresult
      
      
      ############################################
      ####
      ####        clustering on imbalance probes
      ####
      ############################################
      
      
      if(end.cluster.arg) {
        #endcluster is the classification from clustering
        mypvalue[[gene]]$endcluster <- NA
        
        #first, test if some probes are only listed on sample dataset but not in reference dataset, if the probe appears
        #     in bothe dataset, we want the minimal number of nonmissing data points of each probe of each sample from 
        #     the reference should be greater than min.ref.cluster. In this case, samples with missing values are removed.
        
        #  RefData.p3 and RefData.p5 are the datasets from the reference
        # This is to test if there are sufficient data points in the reference data.
        RefData.p3 <- p3[-which(rownames(p3) %in% sampleID),]
        RefData.p5 <- p5[-which(rownames(p5) %in% sampleID),]
        
        #if there is no data in RefData and the sample size in mydata.subset less than the minimum requirements
        if (nrow(RefData.p3) == 0 && nrow(mydata.subset) < fusion.min.sample.size) {
          cat("The number of samples from the reference is not sufficient to run clustering analysis.", file = log, sep = "\n",append = TRUE)
        } else {
          if (length(which(apply(RefData.p3, 1, function(x) length(which(is.na(x)))) < ncol(RefData.p3))) > fusion.min.ref.cluster && length(which(apply(RefData.p5, 1, function(x) length(which(is.na(x)))) < ncol(RefData.p5))) > fusion.min.ref.cluster){
            RefData.p3 <- RefData.p3[which(apply(RefData.p3, 1, function(x) length(which(!is.na(x)))) >= ncol(RefData.p3)),]
            RefData.p5 <- RefData.p5[which(apply(RefData.p5, 1, function(x) length(which(!is.na(x)))) >= ncol(RefData.p5)),]#only choose rows without missing values
          }
          if (nrow(RefData.p3) == 0 && nrow(mydata.subset) > fusion.min.sample.size){ #even if RefData.p3 is empty, as long as sample size greater than min.sample.size
            RefData.p3 <- p3
            RefData.p5 <- p5
          }
          
          #keep the rownames (sampleID) in RefData.p3 and RefData.p5 same and in same order
          #if there is no common sample name between p3 and p5 dataset, issue warning message
          com_samp <- intersect(rownames(RefData.p3), rownames(RefData.p5)) #avoid unmatched samples in RefData.p3 and RefData.p5
          if (length(com_samp) != 0){
            mydata.cluster <- cbind(RefData.p3[com_samp,], RefData.p5[com_samp,])
            
            #clustering using Mclust
            mymod <- Mclust(mydata.subset, G = 2)
            
            #save classification to mypvalue
            for (sp in sampleID){
              mypvalue[[gene]]$endcluster[grep(make.names(sp), make.names(rownames(mypvalue[[gene]])))] <- as.numeric(mymod$classification[sp])
            }
          }
        }  
      }#end for end.cluster.arg
      
      ######################################################
      #####
      ##### linear regression on imbalance probes
      #####
      ######################################################
      mypvalue[[gene]]$lm.pvalue <- NA
      
      #convert mydata.subset into long table
      mydata.long <- data.frame(p3out = numeric(),
                                p5mean = numeric(),
                                sample = character(),
                                probe = character(),
                                stringsAsFactors = FALSE)
      
      #mean of 3'end probes
      p5.mean <- apply(p5, 1, mean)
      num.p3probes <- ncol(p3)
      
      for (sp in rownames(mydata.subset)){
        thisrow <- nrow(mydata.long)
        mydata.long[(thisrow + 1) : (thisrow + num.p3probes),] <- NA
        mydata.long[(thisrow + 1) : (thisrow + num.p3probes),"probe"] <- colnames(p3)
        mydata.long[(thisrow + 1) : (thisrow + num.p3probes),"p5mean"] <- p5.mean[sp]
        mydata.long[(thisrow + 1) : (thisrow + num.p3probes),"sample"] <- names(p5.mean[sp])
        mydata.long[(thisrow + 1) : (thisrow + num.p3probes),"p3out"] <-  as.numeric(p3[sp,])
      }
      mydata.long$enddiff <- mydata.long$p3out - mydata.long$p5mean
      
      
      #linear regression
      
      #first,  a reference sample is chosen based on the minimal absolute value between 5p and 3p, 
      # and which is not a test samples
      if (is.na(ref.sample.imbalanced)){
        p3mean <- apply(p3, 1, mean)
        p5mean <- apply(p5, 1, mean)
        meandiff <- as.data.frame(abs(p3mean - p5mean))
        myref <- rownames(meandiff)[which(meandiff == min(meandiff))[1] ]  
      }else{
        myref <- ref.sample
      }
      
      mydata.long$sample <- relevel(as.factor(mydata.long$sample), ref = myref)
      
      myfit <- as.data.frame(summary(lm(enddiff ~ sample + probe, data = mydata.long))$coefficients)
      
      for (sp in sampleID){
        myfit.row <- grep(sp, rownames(myfit))
        if (length(myfit.row) == 0 ) next
        
        #p value is recorded only if t value is greater than 0
        if (myfit[myfit.row,"t value"] > 0) {
          mypvalue[[gene]]$lm.pvalue[which(rownames(mypvalue[[gene]]) == sp)] <- myfit[myfit.row, "Pr(>|t|)"]
        }    
      }
    }    
  }#end of end.probe.arg
  
  #################################################
  #####
  #####    junction probes
  #####
  #################################################
  if (junc.probe.arg){
    mydata.junction <- mydata.log.all.unadjusted[,as.character(unlist(junction.probes)), drop = F]
    junc.fus <- vector(mode = "list", length = length(junction.gene))
    names(junc.fus) <- junction.gene
    
    for (gene in junction.gene) {
      
      #subsetting the junction dataset by the gene
      mydata.subset <- mydata.junction[,junction.probes[[as.character(gene)]],drop=F]
      junc.fus[[gene]] <- apply(mydata.subset, 2, GeneralizedRobustGESD)
    }
  }
  
  return(list(raw.log = mydata.log.all,
              raw = mydata.log.all.unadjusted,
              endProbes = endProbes, 
              junctionProbes = junction.probes,
              pvalue = mypvalue, 
              junction.test = junc.fus,
              sample.QC.failed = ifelse(QC.excluded,NA,mysample.QC.failed),
              sample.QC.pass = mysample.QC.pass,
              myhk = myhk,
              mysample = sampleID))
}

################################################
###   getHeatMap
################################################

##getHeatMap function takes two arguments: the first is the dataset output from readFusionData function, 
##  getHeatMap extracts the raw data, end probes, junction probes and all sample IDs. The second argument
##  is the probe type ("ALL", "EP" for end probes, and "JUNC" for junction probes).
##  The function generate the heatmap with the probe on the column and sample IDs on the row.

getHeatMap <- function(input,args = "ALL", f.pannot, get.matrix.only = FALSE){
  
  #first get the gene names with pvalue, some genes may not have pvalue and classification
  raw <- input$raw
  probe.label <- colnames(raw)
  
  #avoid probes not used in the raw dataset
  if (length(probe.label) != nrow(f.pannot)){
    probe.label <- intersect(probe.label, f.pannot$ProbeID)
    raw <- raw[, probe.label]
  }
  
  endProbes <-  as.character(unlist(input$endProbes))
  # endProbes.label <- f.pannot$Probe.Label[which(f.pannot$ProbeID %in% endProbes)]
  endProbes.label <- f.pannot[endProbes,"Probe.Label"]
  
  junctionProbes <- as.character(unlist(input$junctionProbes))
  # junctionProbes.label <- f.pannot$Probe.Label[which(f.pannot$ProbeID %in% junctionProbes)]
  junctionProbes.label <-  f.pannot[junctionProbes,"Probe.Label"]
  tmp.ord <- order(junctionProbes.label)
  junctionProbes.label<- junctionProbes.label[tmp.ord]
  junctionProbes <- junctionProbes[tmp.ord]
  
  all.probes <- c(endProbes, junctionProbes)
  all.probes.label <- c(endProbes.label, junctionProbes.label)
  
  mysample <- input$mysample
  
  
  #generate heatmap for the whole dataset and all probes
  #load.packages("gplots")
  if (args == "ALL"){
    #complete data
    if(get.matrix.only)
      return(raw[,all.probes] %>% `colnames<-` (all.probes.label) %>% t)
    pheatmap(mat = t(as.matrix(raw[,all.probes])),cluster_rows = F,cluster_cols = F,scale = "none",labels_row = all.probes.label,main = "All")
    
  }else if (args == "EP"){
    #end.probes
    if(get.matrix.only)
      return(raw[,endProbes] %>% `colnames<-` (endProbes.label) %>% t)
    pheatmap(mat = t(as.matrix(raw[,endProbes])),cluster_rows = F,cluster_cols = F,scale = "none",labels_row = endProbes.label,main = "End Probes")
    
  }else if (args == "JUNC"){
    #junction.probes
    if(get.matrix.only)
      return(raw[,junctionProbes] %>% `colnames<-` (junctionProbes.label) %>% t)
    pheatmap(mat = t(as.matrix(raw[,junctionProbes])),cluster_rows = F,cluster_cols = F,scale = "none",labels_row = junctionProbes.label,main = "Junction Probes")
    
  }
}


################################################
###   plotBySample
################################################

#barplot
#plot by sample
plotBySample <- function(data, sample,f.pannot = fusion.pannot, gene, pvalue.ep.ttest = fusionarg.pvalue.endprobe){
  mysample <- data$mysample
  end.pvalue <- formatC(as.numeric(data$pvalue[[gene]]$p.value[which(rownames(data$pvalue[[as.character(gene)]]) == sample)]), digits = 3, format = "f")
  #thissym = ifelse(end.pvalue < pvalue.ep.ttest, "<", "=")
  #end.pvalue = ifelse(end.pvalue < pvalue.ep.ttest, pvalue.ep.ttest, end.pvalue)
  
  
  add.ep.plots <- !is.null(data$endProbes[[gene]])
  if (add.ep.plots){ 
    endprobe.p5.label <- f.pannot$Probe.Label[which(f.pannot$ProbeID %in% data$endProbes[[gene]]$p5)]
    endprobe.p3.label <- f.pannot$Probe.Label[which(f.pannot$ProbeID %in% data$endProbes[[gene]]$p3)]
    
    
    end.prb.dat <- data.frame(probes = factor(c(endprobe.p5.label, endprobe.p3.label),levels = c(endprobe.p5.label, endprobe.p3.label)),
                              log2.count = data$raw[sample,c(data$endProbes[[gene]]$p5, data$endProbes[[gene]]$p3)],
                              label = "",stringsAsFactors = F)
    end.prb.dat$label[2] <- paste0("p = ",end.pvalue)
    
    g <- g.ep <- ggplot(end.prb.dat,aes(x=probes,y=log2.count,label = label)) + geom_col() + 
      ggtitle(paste("End probes of ",gene,"for sample ","\n",sample)) +
      ylab("Log2(count)")+
      xlab("")+
      theme_bw()+
      geom_text(aes(y=max(log2.count)),size = 5,vjust = "bottom")+
      theme(axis.text.x=element_text(angle=90, hjust=1,size = 15),
            axis.title=element_text(size=20),
            axis.text.y=element_text(size=15))
    
  }
  
  add.jp.plots <- !is.null(data$junctionProbes[[as.character(gene)]])
  if(add.jp.plots){
    names.arg = data$junctionProbes[[as.character(gene)]]
    names.arg = f.pannot$Probe.Label[which(f.pannot$ProbeID %in% names.arg)]
    
    
    jnc.prb.dat <- data.frame(probes = factor(names.arg,levels = names.arg),
                              log2.count = as.numeric(data$raw[sample, data$junctionProbes[[as.character(gene)]]]),
                              label = round(as.numeric(data$junction.test[[gene]][sample, data$junctionProbes[[as.character(gene)]]]),3),
                              stringsAsFactors = F)
    
    g <- g.jp <- ggplot(jnc.prb.dat,aes(x=probes,y=log2.count,label = label)) + geom_col() + 
      ggtitle(paste("Junction probes of ", gene, "for sample ","\n",sample)) +
      ylab("Log2(count)")+
      xlab("")+
      theme_bw()+
      geom_text(size = 5,vjust = "bottom")+
      theme(axis.text.x=element_text(angle=90, hjust=1,size = 15),
            axis.title=element_text(size=20),
            axis.text.y=element_text(size=15))
  }
  
  if(add.ep.plots & add.jp.plots)
    g <- gridExtra::arrangeGrob(g.ep,g.jp,nrow = 1,ncol=2)
  
  return(g)
}



################################################
###   QCPlot
################################################

##QCPlot function takes two arguments: the first is the dataset output from readFusionData function, 
##  QCPlot extracts the transformed or untransformed raw data depending on the second argument 
#   transform = TRUE or FALSE. QCPlot output the line plot of housekeeping genes of each sample,
#   a threshold of log2(35) is set for QC exclusion.

QCPlot <- function(mydta,transform = TRUE){
  mysample <- mydta$mysample
  myhk <- mydta$myhk
  mydata <- mydta$raw[mysample,mydta$myhk]
  plot(1:length(myhk),mydata[1,], col = 1, type = "l", xlab = "Housekeeping genes", ylab = "log2 count", ylim = c(min(mydata) -0.5, max(mydata) + 0.5), xaxt = "n")
  axis(1,at = 1:length(myhk), labels = colnames(mydata),las = 2, cex.axis = 0.5)
  for (i in 2:nrow(mydata)){
    par(new = T)
    plot(1:length(myhk), mydata[i,], col = i, type = "l", xlab = "", ylab = "", ylim =  c(min(mydata) -0.5, max(mydata) + 0.5), xaxt = "n", yaxt = "n")
  }
  abline(h = log2(35), col = "red",pch = 20, lty = 6, lwd =5)
}


################################################
###   getFusionResults
################################################
getFusionResults = function(data){
  allgenes = union(names(data$endProbes), names(data$junctionProbes))
  nrow = max(unlist(lapply(data$pvalue, nrow))) #using maximal nrow
  all.endpvalue = as.data.frame(matrix(NA,nrow = nrow, ncol = length(allgenes)))
  names(all.endpvalue) = allgenes
  rownames(all.endpvalue) = rownames(data$pvalue[[1]])
  for (gene in allgenes){
    if (!gene %in% names(data$pvalue)) next
    all.endpvalue[,gene] <- data$pvalue[[gene]][,2]
  }
  allpvalue = cbind(all.endpvalue, data$junction.test)
  return(list(pvalue = allpvalue, allgenes = allgenes))
}

################################################
###   getFusionCall
################################################

##QCPlot function takes two arguments: the first is the dataset output from readFusionData function, 
##  QCPlot extracts the transformed or untransformed raw data depending on the second argument 
#   transform = TRUE or FALSE. QCPlot output the line plot of housekeeping genes of each sample,
#   a threshold of log2(35) is set for QC exclusion.
getFusionCall <- function(data, pvalue.ep.ttest = fusionarg.pvalue.endprobe){
  
  allgenes <- union(names(data$endProbes), names(data$junctionProbes))
  dd <- as.data.frame(matrix(NA, ncol = length(allgenes), nrow = nrow(data$raw)))
  names(dd) <- allgenes
  rownames(dd) <- rownames(data$raw)
  
  for (j in 1:length(allgenes)){
    gene <- allgenes[j]
    for(i in 1:nrow(dd)){
      if (!is.null(data$pvalue[[gene]][i,2])){
        if (data$pvalue[[gene]][i,2] < pvalue.ep.ttest & any(data$junction.test[[gene]][i,] != 0)) dd[i,j] <- 1 #DJ,DE   
        if (data$pvalue[[gene]][i,2] >= pvalue.ep.ttest & any(data$junction.test[[gene]][i,] != 0)) dd[i,j] <- 2#DJ,NDE
        if (data$pvalue[[gene]][i,2] < pvalue.ep.ttest & all(data$junction.test[[gene]][i,] == 0)) dd[i,j] <- 3#NDJ,DE
        if (data$pvalue[[gene]][i,2] >= pvalue.ep.ttest & all(data$junction.test[[gene]][i,] == 0)) dd[i,j] <- 5#NDJ,NDE
      }else{
        if (any(data$junction.test[[gene]][i,] != 0)) dd[i,j] <- 4  #NAE,DJ
        if (all(data$junction.test[[gene]][i,] == 0)) dd[i,j] <- 6 #NAE,NDJ
        if (all(as.logical(lapply(data$junction.test[[gene]][i,], is.na)))) dd[i,j] <- mycolor[7]#NAE,NAJ
      }
    }}
  return(dd)
  
}


################################################
###   plot_table
################################################

plot_table <- function(data){
  # data$X <- rownames(data)
  # data1 <- melt(data, id = c("X"))
  # names(data1) <- c("Sample", "Gene","Ind")
  # data1$Ind <- fusioncall.int.to.cat(data1$Ind)
  
  # Get the fusion calls labels
  data1 <- get.fusion.call.summary.mat(data)
  
  g <- ggplot(data1, aes(x = Sample, y = Gene, fill = Ind)) +
    scale_fill_discrete(drop=FALSE)+
    geom_tile(size = .2, colour = "white") +
    theme(axis.text.x = element_text(angle = 90), legend.position = "bottom") +
    labs(x = "", y = "") +
    ggtitle("Fusion Detection Summary")+
    guides(fill = guide_legend(title = "Fusion detection", ncol = 3))
  return(g)
  
}


#' This function controls encoding of calls as numeric
#' @param x a numeric vector consisting of values 1:6
#' @return Character version of 1:6

fusioncall.int.to.cat <- function(x, cat.type = c("detailed","junction","end")){
  cat.type <- match.arg(cat.type,  c("detailed","junction", "end"), several.ok = F)
  
  if(any(!x %in% 1:6))
    stop("Fusion call categories need to be in the set 1 through 6")
  if(cat.type == "detailed")
    x.cat <- as.character(factor(x,levels =1:6, labels = c("Junction+ End Probe+",
                                                           "Junction+ End Probe-",
                                                           "Junction- End Probe+",
                                                           "Junction+",
                                                           "Junction- End Probe-",
                                                           "Junction-")))
  if(cat.type == "junction")
    x.cat <- x %in% c(1, 2, 4)
  
  if(cat.type == "end")
    x.cat <- x %in% c(1, 3)
  
  return(x.cat)
}


#' This function controls encoding of calls as numeric
#' @param x data sample x call format containing numbers 1:6
#' @param mat.format either long or wide character string. It determins whether the output should be in long wor wide format 
#' @param all.fusion.res this contains all fusion analysis results and is used when output format is report
#' @param fusion.pannot fusion probe annoations used when output format is report
#' @return categorical version of table with 1:6
#' 
get.fusion.call.summary.mat <- function(data, mat.format = c("long","wide","report"), all.fusion.res = NULL, fusion.pannot = NULL){
  mat.format <- match.arg(mat.format,  c("long","wide","report"), several.ok = F)
  
  data$X <- rownames(data)
  data1 <- melt(data, id = c("X"))
  names(data1) <- c("Sample", "Gene","Ind")
  
  if(mat.format %in% c("wide", "long"))
    data1$Ind <- fusioncall.int.to.cat(data1$Ind, cat.type = "detailed") 
  
  if(mat.format == "wide"){
    data1 <- spread(data1,key = Gene, value = Ind) %>% 
      `rownames<-` (.$Sample) %>%
      dplyr:::select(- Sample)
    data1 <- data1[rownames(data), colnames(data)[colnames(data) %in% colnames(data1)]]
  }
  
  if(mat.format == "report"){
    jnc.summary <- sapply(all.fusion.res$junction.test,
           FUN = function(g) apply(g,MARGIN = 1,FUN = function(x) paste(fusion.pannot[colnames(g)[x > 0], "Probe.Label"],collapse = ", ")  ) )
    jnc.summary <- apply(jnc.summary, MARGIN = 1, function(x) paste(x[x!=""], collapse = "; "))
    jnc.summary[jnc.summary == ""] <- "none"

    data1$end <- fusioncall.int.to.cat(data1$Ind, cat.type = "end")
    
    
    tmp <- data1 %>% dplyr:::select( c(Sample,  Gene,  end)) %>%
      spread(key = Gene, value = end) %>%
      `rownames<-` (.$Sample) %>%
      dplyr:::select(- Sample)
    tmp <- apply(tmp,1,function(x) paste(colnames(tmp)[x],collapse = ", "))
    tmp[tmp==""] <- "none"
    end.summary <- tmp
    
    data1 <- data.frame("sample ID" = names(end.summary),
               "Imbalance detected" = end.summary,
               "Fusion detected" =  jnc.summary[names(end.summary)],
               check.names = F, row.names = NULL)
  }
  
  return(data1)
}




#===============================================
#             Wrapper function
#===============================================

#' This function is a wrapper for all plot function of fusion analysis
#' @param  fusion.dat the output of readFusionData
#' @param fusion.analyzed output of getFusionCall
#' @param type which type of plot to be plotted
#' @param file.extension character image type to store data as (e.g. png). Note csv generates the csv version of the plotted content
#' @param path.to.fusion.results path to store plots at
#' @param fusion.pannot fusion probe annotations
#' @param fusionarg.pvalue.endprobe end probe pvalue threshold
#' @return a list of files paths to where the plots were stored

plot.fusion <- function(fusion.dat, 
                        fusion.analyzed,
                        type = c("call.summary","heatmap","QC","by.gene"),
                        file.extension = "pdf",
                        path.to.fusion.results,
                        fusion.pannot,
                        fusionarg.pvalue.endprobe,
                        ...){
  type <- match.arg(type,c("call.summary","heatmap","QC","by.gene"), several.ok = TRUE)
  
  fls <- list()
  
  # Call summary
  #-------------
  if("call.summary" %in% type){
    dir.summary <- paste0(path.to.fusion.results,"/summary/")
    dir.create(dir.summary,showWarnings = F)
    
    
    sum.tab.plt <- plot_table(data = fusion.analyzed) 
    
    if("png" %in% file.extension){
      fl.i <- paste0(dir.summary,"summary_of_fusion_analysis.png")
      png(filename = fl.i,width = max(2000,4000*nrow(fusion.analyzed)/130),height = max(1500,2000*ncol(fusion.analyzed)/4),res = 250)
      print(sum.tab.plt)
      dev.off()
      fls$summary <- c(fls$summary,fl.i)
    }
    
    if("pdf" %in% file.extension){
      fl.i <- paste0(dir.summary,"summary_of_fusion_analysis.pdf")
      pdf(file = fl.i, width = max(8,10 *nrow(fusion.analyzed)/12),height = max(5,8*ncol(fusion.analyzed)/4))
      print(sum.tab.plt)
      dev.off()
      fls$summary <- c(fls$summary,fl.i)
    }
    
    if("csv" %in% file.extension){
      fsum <- get.fusion.call.summary.mat(data = fusion.analyzed, mat.format = "wide")
      fl.i <- paste0(dir.summary,"summary_of_fusion_analysis.csv")
      write.csv(x = fsum,file = fl.i)
      fls$summary <-c(fls$summary,fl.i)
      
      fsum <- get.fusion.call.summary.mat(data = fusion.analyzed,
                                          mat.format = "report", 
                                          all.fusion.res = fusion.dat,
                                          fusion.pannot = fusion.pannot)
      fl.i <- paste0(dir.summary,"report_of_fusion_analysis.csv")
      write.csv(x = fsum,file = fl.i,row.names = F)
      fls$report <-c(fls$report,fl.i)
    }
    
    
  }
  
  
  # Heatmap
  #--------
  if("heatmap" %in% type){
    dir.heatmap <- paste0(path.to.fusion.results,"/heatmap/")
    dir.create(dir.heatmap,showWarnings = F)
    
    for (arg.i in c("ALL","EP","JUNC")){
      prb.len <- length(unlist(fusion.dat$endProbes))+length(unlist(fusion.dat$junctionProbes))
      if(arg.i != "ALL")
        prb.len <- length(unlist(fusion.dat[[ c("endProbes","junctionProbes")[arg.i == c("EP","JUNC","ALL")] ]]))
      
      if(prb.len == 0)
        next
      
      if("png" %in% file.extension){
        fl.i <- paste0(dir.heatmap,arg.i,".png")
        png(filename = fl.i,width = max(3000,3000*nrow(raw)/12),height = max(3000,3000*prb.len/60),res = 250)
        getHeatMap(input = fusion.dat,args = arg.i,f.pannot = fusion.pannot)
        dev.off()
        fls$heatmap <- c(fls$heatmap,fl.i)
      }
      
      if("pdf" %in% file.extension){
        fl.i <- paste0(dir.heatmap,arg.i,".pdf")
        pdf(file = fl.i,width = max(8,8*nrow(raw)/12),height = max(8,8*prb.len/60))
        getHeatMap(input = fusion.dat,args = arg.i,f.pannot = fusion.pannot)
        dev.off()
        fls$heatmap <- c(fls$heatmap,fl.i)
      }
      
      
      if("csv" %in% file.extension){
        fl.i <- paste0(dir.heatmap,arg.i,".csv")
        getHeatMap(input = fusion.dat,args = arg.i,f.pannot = fusion.pannot,get.matrix.only=TRUE) %>% write.csv(x = .,file = fl.i)
        fls$heatmap <- c(fls$heatmap,fl.i)
      }
      
    }
  }
  
  
  # QC
  #----
  if("QC" %in% type){
    dir.QC <- paste0(path.to.fusion.results,"/QC/")
    dir.create(dir.QC,showWarnings = F)
    
    
    if("png" %in% file.extension){
      fl.i <- paste0(dir.QC,"/QC.png")
      png(fl.i, width = 480, height = 480, pointsize = 12, bg = "white", units = "px")
      QCPlot(mydta = fusion.dat,transform = TRUE)
      dev.off()
      fls$QC <- c(fls$QC,fl.i)
    }
    
    
    # if("pdf" %in% file.extension){
    #   fl.i <- paste0(dir.QC,"/QC.pdf")
    #   pdf(fl.i, width = 480, height = 480, pointsize = 12, bg = "white", units = "px")
    #   QCPlot(mydta = fusion.dat,transform = TRUE)
    #   dev.off()
    #   fls$QC <- c(fls$QC,fl.i)
    # }
    # 
    # if("csv" %in% file.extension){
    #   fl.i <- paste0(dir.QC,"/QC.csv")
    #   pdf(fl.i, width = 480, height = 480, pointsize = 12, bg = "white", units = "px")
    #   QCPlot(mydta = fusion.dat,transform = TRUE)
    #   dev.off()
    #   fls$QC <- c(fls$QC,fl.i)
    # }
    
    
  }
  
  
  # By gene x sample
  #-------------------
  if("by.gene" %in% type){
    dir.by.gene <- paste0(path.to.fusion.results,"/byGene/")
    dir.create(dir.by.gene,showWarnings = F)
    fls$by.gene <- list()
    for(gene in unique(c(names(fusion.dat$endProbes), names(fusion.dat$junctionProbes)))){
      
      dir.by.gene.i <- paste0(dir.by.gene,swap.illegal.char(gene))
      dir.create(dir.by.gene.i,showWarnings = F)
      fls$by.gene[[gene]] <- c()
      for (sample in fusion.dat$mysample){
        fl.i <- paste0(dir.by.gene.i,"//", swap.illegal.char(sample),".png")
        g <- plotBySample(fusion.dat,f.pannot = fusion.pannot, sample=sample, gene = gene, pvalue.ep.ttest = fusionarg.pvalue.endprobe)
        png(filename = fl.i,width = 1500,height = 1000, res = 100)
        grid.draw(g)
        dev.off()
        # ggsave(filename = fl.i,device = "png",plot = g,width = 15,height = 10,dpi = 200)
        fls$by.gene[[gene]] <- c(fls$by.gene[[gene]] ,fl.i)
      }
    }
    
  }
  
  return(fls)
  
}



#' #' This funciton of a plot only utility funciton
#' #' @param string a string in which there are illegal characters
#' #' @param illegal.pattern Regx pattern of characters separated by "|" to be replaced
#' #' @param replacement a character string to replace the illegal chars
#' 
#' remove.illegal.chars <- function(string,
#'                                  illegal.pattern = paste(c("<",">",":","\"","\\?","\\|","\\*","\\.","\\%"),collapse = "|"),
#'                                  replacement = "_"){
#'   smp.i.rename <- gsub(x = string,pattern = illegal.pattern,replacement = replacement)
#' }
#' 
#' 


#' This function receives a character vector and removes 
#' Illegal characters that break the analysis (either in html or plot saving)
#' NOTE: Decided against adding the 255 char limit check as that was very unlikely for geneset name
#' @param x a character vator

swap.illegal.char <- function(x){
  char.2b.changed <- c(STR = "\\*",
                       FR  = "<",
                       TO  = ">",
                       CL  = "\\:",
                       TLD = "\\~",
                       BNG = "\\!",
                       AT  = "\\@",
                       DLR  = "\\$",
                       PRC = "\\%",
                       CR  = "\\^",
                       AP  = "\\`",
                       EQ  = "\\=",
                       CBL = "\\{",
                       CBR = "\\}",
                       BRL = "\\[",
                       BRR = "\\]",
                       PP = "\\|",
                       QZ = "\\?",
                       PLS = "\\+",
                       SQT = "\\'",
                       DQT = "\\\"",
                       FS = "\\/",
                       BS = "\\\\")
  
  # Remove .png and .RCC
  x <- gsub("\\.png|\\/|\\.RCC","",x)
  
  # Test case
  # x <- "asdf * < > : ~ ! @ $ % ^ ` = { } [ ] | ? + ' \" / \\ "
  # x
  for(i in 1:length(char.2b.changed)){
    pt <- unname(char.2b.changed[i])
    rpl <- paste(".",names(char.2b.changed)[i],".",sep="")
    x <- gsub(pt,rpl,x = x)
    # cat(pt,"\n")
    # cat(x,"\n")
  }
  return(x)
}