# require(openxlsx)
# require(Biobase)

setClass("NanoStringExprSet", 
         contains = "ExpressionSet", 
         representation = representation(
           scalingFactor = "matrix",
           dispersion = "numeric",
           backgroundMean = "numeric", # change it to a matrix in the next version
           backgroundDispersion = "numeric"),
         prototype = prototype(scalingFactor = as.matrix(NA),
                               dispersion = as.numeric(NA)))

slotNames = c("scalingFactor", "dispersion", "backgroundMean","backgroundDispersion")
for(slotName in slotNames) {
  setGeneric(slotName, 
             eval(parse(text=paste0("function(object) 
                                    standardGeneric('",slotName,"')")))
             )
  setGeneric(paste0(slotName,"<-"), 
             eval(parse(text=paste0("function(object, value) 
                                    standardGeneric('",paste0(slotName,"<-"),"')")))
             )
  setMethod(slotName, 
            signature(object = "NanoStringExprSet"), 
            eval(parse(text=paste0("function(object) {
                                   slot(object,'",slotName,"')}")))
            )
  
  if (slotName=="scalingFactor") {
    setReplaceMethod(slotName, signature(object = "NanoStringExprSet", value = "matrix"), 
                     eval(parse(text=paste0("function(object, value) {
                                            n = nrow(exprs(object))
                                            m = ncol(exprs(object))
                                            if (slotName == 'scalingFactor') {
                                            if ((is.vector(value) & length(value)!=m) | 
                                            (is.matrix(value) & (ncol(value)!=m | nrow(value)!=n))) {
                                            stop('wrong dimension for scaling factor!')
                                            }
                                            }
                                            slot(object,'",slotName,"') <- value
                                            object
  }"))))
  } else {
    setReplaceMethod(slotName, signature(object = "NanoStringExprSet", value = "numeric"), 
                   eval(parse(text=paste0("function(object, value) {
                     n = nrow(exprs(object))
                     m = ncol(exprs(object))
                     if (slotName == 'scalingFactor') {
                       if ((is.vector(value) & length(value)!=m) | 
                           (is.matrix(value) & (ncol(value)!=m | nrow(value)!=n))) {
                         stop('wrong dimension for scaling factor!')
                       }
                     }
                     if (slotName == 'backgroundMean' & length(value) != m) 
                       stop('wrong length for background mean!')
                     if (slotName == 'dispersion' & length(value) != n) 
                       stop('wrong length for dispersion!')
                     slot(object,'",slotName,"') <- value
                     object
                   }"))))
  }
}

convert2NSDiff = function(NSDEData) {
  require(NanoStringDiff)
  x = as.data.frame(pData(NSDEData))
  y = exprs(NSDEData)
  NSDiffData = createNanoStringSet(endogenous=y, positiveControl=new("matrix"), negativeControl=new("matrix"), 
                                   housekeepingControl=new("matrix"), designs=x)
  positiveFactor(NSDiffData) = sapply(colnames(y), function(x) 1)
  negativeFactor(NSDiffData) = backgroundMean(NSDEData)
  housekeepingFactor(NSDiffData) = scalingFactor(NSDEData)
  return(NSDiffData)
}

covertfromNSDiff = function(NSDiffData) {
  x = pData(NSDiffData)
  y = exprs(NSDiffData)
  NSDEData = createNanoStringSet(endogenous=y, designs=as.data.frame(x))
  backgroundMean(NSDEData) = negativeFactor(NSDiffData)
  scalingFactor(NSDEData) = housekeepingFactor(NSDiffData)
  return(NSDEData)
}
