#' This function appends the fusion section to the report
#' @param advAnalysisReport the report object to which to append this report
#' @param SNV.dir list of plots direcotry

append.SNV.to.report <- function(advAnalysisReport, SNV.dir){
  
  
  #limit SNV.dir to png only
  #-------------------------
  SNV.dir <- rapply(SNV.dir, function(x) normalizePath( paste0(unique(gsub("*\\.png$|*\\.csv$","",x = x)),".png"),winslash = "/",mustWork = F ) , how = "list")
  
  
  smp.map <- data.frame(generic.name = paste0("smp",1:length(SNV.dir$samples)),
                        fls = SNV.dir$samples,
                        abbr.name = gsub("\\.png|\\/|\\.RCC","",basename(SNV.dir$samples)),
                        stringsAsFactors = F)
  
  # add a panel:
  addReportContent(
    element = getReportContentById(id = "moduleMenu_SNV", report = advAnalysisReport),
    panel(id = "SNVPanel",
          title = "This module shows the results of the SNV analysis"    
    )
  )
  
  
  # ADD HORIZONTAL MENU
  addReportContent(
    element = getReportContentById(id = "SNVPanel", report = advAnalysisReport),
    horizontalMenu(id = "SNVPanel_horizmenu_summary", title = "Overview", active = FALSE)
  )
  
  addReportContent(
    element = getReportContentById(id = "SNVPanel", report = advAnalysisReport),
    horizontalMenu(id = "SNVPanel_horizmenu_bySample", title = "By Samples", active = FALSE)
  )
  
  
  
  # SNV Summary
  #------------
  
  addReportContent(
    element = getReportContentById(id = "SNVPanel_horizmenu_summary", report = advAnalysisReport),
    menuItem(id = "SNVPanel_horizmenu_summary_dd", title = "sample probe matrix")
  )
  
  # Add panel
  snv.summary.panel <- paste0("SNVPanel_horizmenu_summary_dd","_panel")
  addReportContent(
    element = getReportContentById(id = "SNVPanel_horizmenu_summary_dd", report = advAnalysisReport),
    panel(id = snv.summary.panel, title = "Overview")
  )
  
  
  # Add content
  item.2.show.path <- SNV.dir$summary
  item.2.show.content.id <- paste0(snv.summary.panel,"01")
  
  addReportContent(
    element = getReportContentById(id = snv.summary.panel, report = advAnalysisReport),
    reportImage(id = item.2.show.content.id, 
                uri= item.2.show.path,
                title = "",
                annotation(id = "SNV_overview_plot_annotation",
                           title = "This heatmap shows the samples and probes where SNVs were called.\nThe legend at the bottom of the plot details the rule for calling SNVs from raw counts, p-values and fold-changes above expected background."),
                fileDownloadLink(id = paste0(item.2.show.content.id,"_tb"),
                                 uri= gsub(pattern = "png",replacement = "csv",x = item.2.show.path),
                                 visible = file.exists(gsub(pattern = "png",replacement = "csv",x = item.2.show.path)),
                                 title="Download CSV Data"))    
  )
  
  
  # SNV QC Heatmap
  #--------------
  #Drop down
  addReportContent(
    element = getReportContentById(id = "SNVPanel_horizmenu_summary", report = advAnalysisReport),
    menuItem(id = "SNVPanel_horizmenu_summary_quality", title = "Quality Map")
  )
  
  
  # ADD Right PANEL TO DROP DOWN MENU
  addReportContent(
    element = getReportContentById(id = "SNVPanel_horizmenu_summary_quality", report = advAnalysisReport),
    panel(id = "SNVPanel_horizmenu_summary_quality_panel", title = "Qpanel")
  )
  
  
  addReportContent(
    element = getReportContentById(id = "SNVPanel_horizmenu_summary_quality_panel", report = advAnalysisReport),
    verticalMenu(id = "SNVPanel_horizmenu_summary_quality_panel_vmen", title = "Quality Map type")
  )
  
  
  # Add content
  hm.map <-  data.frame(full.path = SNV.dir$QC, 
                        fl= basename(SNV.dir$QC),
                        stringsAsFactors = F)
  
  
  
  hm.map$ttl <- c(SNV_QC_raw.png = "Raw",SNV_QC_scaled.png = "Scaled")[hm.map$fl]
  
  hm.map$info <-c(SNV_QC_raw.png = "This heatmap shows the log10 raw SNV control probes values for each (reference and test) sample in the dataset.  Using this plot, one can not only detect poor quality samples, but noting the vertical color bar on the left, the cause and effect of a possible sample failure may be explained.  Probes belonging to SNV_NEG and SNV_POS groups represent the background and signal level. Those belonging to SNV_PCR_CTL capture the quality of PCR for each sample. Any sample with significantly lower levels of this family of probes may have experienced a suboptimal PCR step.  Probes belonging to SNV_UDG_CTL are the UDG control probe and show whether UDG digestion was successful.",
                  SNV_QC_scaled.png = "This heatmap shows the by probe median centered log10 raw SNV control probes values for each (reference and test) sample in the dataset.  Using this plot, one can not only detect poor quality samples, but noting the vertical color bar on the left, the cause and effect of a possible sample failure may be explained.  Probes belonging to SNV_NEG and SNV_POS groups represent the background and signal level. Those belonging to SNV_PCR_CTL capture the quality of PCR for each sample. Any sample with significantly lower levels of this family of probes may have experienced a suboptimal PCR step.  Probes belonging to SNV_UDG_CTL are the UDG control probe and show whether UDG digestion was successful.")[hm.map$fl]
  
  
  for(i in 1:nrow(hm.map)){
    
    # Add buttons to vertical menue
    panel.vbuttonMenu.content.id <- paste0("SNVPanel_horizmenu_summary_quality_panel_vmen_button", gsub(" ","",hm.map[i,"ttl"]))
    
    addReportContent(
      element = getReportContentById(id = "SNVPanel_horizmenu_summary_quality_panel_vmen", report = advAnalysisReport),
      menuItem(id = panel.vbuttonMenu.content.id, title = hm.map[i,"ttl"])
    )
    
    # Add panel
    SNV.heatmap.panel <- paste0(panel.vbuttonMenu.content.id,"_panel")
    addReportContent(
      element = getReportContentById(id = panel.vbuttonMenu.content.id, report = advAnalysisReport),
      panel(id = SNV.heatmap.panel, title = paste0(hm.map[i,"ttl"],"vbutpanel"))
    )
    
    
    addReportContent(
      element = getReportContentById(id = SNV.heatmap.panel, report = advAnalysisReport),
      reportImage(id =  paste0(SNV.heatmap.panel,"_img"),
                  uri=hm.map[i,"full.path"],
                  title = "",
                  annotation(id = "SNV_QC_plot_annotation",
                             title = hm.map[i,"info"]),
                  fileDownloadLink(id = paste0(SNV.heatmap.panel,"_tb"),
                                   uri= gsub(pattern = "png",replacement = "csv",x = hm.map[i,"full.path"]),
                                   visible = file.exists(gsub(pattern = "png",replacement = "csv",x = hm.map[i,"full.path"])),
                                   title="Download CSV Data"))    
    )
  }
  
  
  # Add by sample
  #--------------
  for(i in 1:nrow(smp.map)){
    
    smp.dropdown.id <- paste0("SNVPanel_horizmenu_bySample_",smp.map[i,"generic.name"])
    smp.dropdown.lab <- smp.map[i,"abbr.name"]
    # ADD DROP DOWN MENU
    addReportContent(
      element = getReportContentById(id = "SNVPanel_horizmenu_bySample", report = advAnalysisReport),
      menuItem(id = smp.dropdown.id, title = smp.dropdown.lab)
    )
    
    # Add panel
    smp.dropdown.panelcontent.id <- paste0(smp.dropdown.id,"_panel")
    addReportContent(
      element = getReportContentById(id = smp.dropdown.id, report = advAnalysisReport),
      panel(id = smp.dropdown.panelcontent.id, title = "")
    )
    
    # Add content
    item.2.show.path <- smp.map[i,"fls"]
    item.2.show.content.id <- paste0(smp.dropdown.panelcontent.id,"01")
    
    addReportContent(
      element = getReportContentById(id = smp.dropdown.panelcontent.id, report = advAnalysisReport),
      reportImage(id = item.2.show.content.id,
                  uri=item.2.show.path,
                  title = "",
                  annotation(id = "snv_bysample_annotation",
                             title = "This plot displays a single sample's results for each probe. Probes for which an SNV was called are highlighted in orange. Vertical bars show the estimates and confidence limits for each probe's log2 fold-change relative to its expected value in reference samples. Vertical bars overlapping the black line at 0 indicate probes that are not statistically significantly above their expected reference level. Very low-frequency SNVs may manifest as vertical bars above 0 but with estimates below the fold-change threshold.\nThe dashed orange line marks the fold-change threshold required for a SNV call. The legend on the right side of the plot details the rule for calling SNVs from raw counts, p-values and fold-changes above expected reference value."),
                  fileDownloadLink(id = paste0(item.2.show.content.id,"tb"),
                                   uri= gsub(pattern = "png",replacement = "csv",x = item.2.show.path),
                                   visible = file.exists(gsub(pattern = "png",replacement = "csv",x = item.2.show.path)),
                                   title="Download CSV Data"))#,    
      # reportTable(
      #   id = paste0(item.2.show.content.id,"tb"),
      #   uri= gsub(pattern = "png",replacement = "csv",x = item.2.show.path),
      #   visible = file.exists(gsub(pattern = "png",replacement = "csv",x = item.2.show.path)),
      #   title = smp.dropdown.lab,
      #   annotation(id = "xx", title = "Table identifying outliers in the first four PCs of the data."))
      
      )
    
    
    
  }
  
  
  
  return(advAnalysisReport) 
}