#' This function appends the fusion section to the report
#' @param advAnalysisReport the report object to which to append this report
#' @param fusion.dir list of plots direcotry

append.fusion.to.report <- function(advAnalysisReport, fusion.dir){
  
  #limit fusion.dir to png only
  #----------------------------
  fusion.dir <- rapply(fusion.dir, function(x) normalizePath( paste0(unique(gsub("*\\.png$|*\\.csv$","",x = x)),".png"),winslash = "/",mustWork = F ) , how = "list")
  
  smp.names <- gsub(pattern = ".png","",basename(unlist(fusion.dir$by.gene)))
  
  smp.map <- data.frame(generic.name = sprintf("smp%02d",as.numeric(factor(smp.names, levels = smp.names[!duplicated(smp.names)]))),
                        fls = unlist(fusion.dir$by.gene),
                        abbr.name = gsub("\\.png|\\/|\\.RCC","",smp.names),
                        genes = rep(names(fusion.dir$by.gene), sapply(fusion.dir$by.gene,length)),
                        stringsAsFactors = F)
  
  
  # add a panel:
  addReportContent(
    element = getReportContentById(id = "moduleMenu_fusion", report = advAnalysisReport),
    panel(id = "fusionPanel",
          title = "This module shows the results of the fusion analysis"    
    )
  )
  
  
  
  # ADD HORIZONTAL MENU
  addReportContent(
    element = getReportContentById(id = "fusionPanel", report = advAnalysisReport),
    horizontalMenu(id = "fusionPanel_horizmenu_overview", title = "Overview", active = FALSE) #TRUE?
  )
  
  addReportContent(
    element = getReportContentById(id = "fusionPanel", report = advAnalysisReport),
    horizontalMenu(id = "fusionPanel_horizmenu_bySample", title = "By Samples", active = FALSE)
  )
  
  
  # Fusion Summary
  #---------------
  addReportContent(
    element = getReportContentById(id = "fusionPanel_horizmenu_overview", report = advAnalysisReport),
    menuItem(id = "fusionPanel_horizmenu_overview_summary", title = "Summary")
  )
  
  
  # Add panel
  fusion.summary.panel <- paste0("fusionPanel_horizmenu_overview_summary","_panel")
  addReportContent(
    element = getReportContentById(id = "fusionPanel_horizmenu_overview_summary", report = advAnalysisReport),
    panel(id = fusion.summary.panel, title = "Overview")
  )
  
  
  # Add image
  addReportContent(
    element = getReportContentById(id = fusion.summary.panel, report = advAnalysisReport),
    reportImage(id = paste0(fusion.summary.panel,"01"),
                uri=fusion.dir$summary,
                title = "",
                annotation(id = "fusion_summary_plot_annotation", title = "This table summarizes the fusion calls made in each gene and each sample. Color indicates whether a fusion was called and the type of evidence (Junction probes, End probes (imbalance), or both) used to make the call. A color key is provided below the table. \"Detected \" calls made simultaneously by both Junction and End probes provide the strongest evidence for a fusion event. \"Detected\" calls made by end probes accompanied by \"undetected\" junction probe calls may result from fusions not targeted by the experiment's junction probes. \"Detected\" calls made by junction probes accompanied by \"undetected\" imbalance probe calls may result from fusions that are expressed at much lower levels than the wild-type transcript; these fusion calls hold weaker evidence, and examination of the raw data for these samples is recommended."),
                fileDownloadLink(id = paste0(fusion.summary.panel,"_tb"),
                                 uri= gsub("png","csv",x=fusion.dir$summary),
                                 visible = file.exists(gsub("png","csv",x=fusion.dir$summary)),
                                 title="Download CSV Data")),
    
    ## Add the results table: 
    reportTable(
      id = "fusion_report_table",
      uri= gsub("png","csv",x=fusion.dir$report), 
      title = "Fusion Summary Report",
      fileDownloadLink(id = paste0(fusion.summary.panel,"_tb"),
                       uri= gsub("png","csv",x= fusion.dir$report),
                       visible = file.exists(gsub("png","csv",x=fusion.dir$report)),
                       title="Download CSV Data")

      )
    
  )
  
  
  # Fusion Heatmap
  #---------------
  
  #Drop down
  addReportContent(
    element = getReportContentById(id = "fusionPanel_horizmenu_overview", report = advAnalysisReport),
    menuItem(id = "fusionPanel_horizmenu_overview_heatmap", title = "Heatmap")
  )
  
  
  # ADD Right PANEL TO DROP DOWN MENU
  addReportContent(
    element = getReportContentById(id = "fusionPanel_horizmenu_overview_heatmap", report = advAnalysisReport),
    panel(id = "fusionPanel_horizmenu_overview_heatmap_panel", title = "hmpanel")
  )
  
  
  addReportContent(
    element = getReportContentById(id = "fusionPanel_horizmenu_overview_heatmap_panel", report = advAnalysisReport),
    verticalMenu(id = "fusionPanel_horizmenu_overview_heatmap_panel_vmen", title = "Heatmap type")
  )
  
  
  # Add content
  hm.map <-  data.frame(full.path = fusion.dir$heatmap, 
                        fl= basename(fusion.dir$heatmap),
                        stringsAsFactors = F)
  hm.map$ttl <- c(ALL.png = "All",EP.png = "End probes", JUNC.png = "Junction probe")[hm.map$fl]
  hm.map$info <-c(ALL.png = "This heatmap shows the raw counts from every fusion probe in the experiment. Heatmap is colored by the log2 count according to the legend. Although every experiment and every probe is different, counts below 20 (4.3 in log2) are often background, and counts above 100 (6.6 in log2)  are very seldom background.",
                  EP.png = "This heatmap shows the raw counts from every imbalance probe in the experiment. Heatmap is colored by the log2 count according to the legend. Although every experiment and every probe is different, counts below 20 (4.3 in log2) are often background, and counts above 100  (6.6 in log2) are very seldom background. Fusions will manifest as consistently higher counts in a sample\'s 3\' probes for a gene than in its 5\' probes.",
                  JUNC.png = "This heatmap shows the raw counts from every junction probe in the experiment. Heatmap is colored by the log2 count according to the legend. Although every experiment and every probe is different, counts below 20 (4.3 in log2) are often background, and counts above 100  (6.6 in log2) are very seldom background. Use this heatmap to look for high-count probes corresponding to clear fusion events. Also look for probes with consistently elevated counts: these probes are experiencing unusually high background, and the calling method may behave anomalously in these probes.")[hm.map$fl]
  
  for(i in 1:nrow(hm.map)){
    
    # Add buttons to vertical menue
    panel.vbuttonMenu.content.id <- paste0("fusionPanel_horizmenu_overview_heatmap_panel_vmen_button", gsub(" ","",hm.map[i,"ttl"]))
    
    addReportContent(
      element = getReportContentById(id = "fusionPanel_horizmenu_overview_heatmap_panel_vmen", report = advAnalysisReport),
      menuItem(id = panel.vbuttonMenu.content.id, title = hm.map[i,"ttl"])
    )
    
    # Add panel
    fusion.heatmap.panel <- paste0(panel.vbuttonMenu.content.id,"_panel")
    addReportContent(
      element = getReportContentById(id = panel.vbuttonMenu.content.id, report = advAnalysisReport),
      panel(id = fusion.heatmap.panel, title = paste0(hm.map[i,"ttl"],"vbutpanel"))
    )
    
    
    addReportContent(
      element = getReportContentById(id = fusion.heatmap.panel, report = advAnalysisReport),
      reportImage(id =  paste0(fusion.heatmap.panel,"_img"),
                  uri=hm.map[i,"full.path"],
                  title = "",
                  annotation(id = "fusion_heatmap_plot_annotation", title =  hm.map[i,"info"] ),
                  fileDownloadLink(id = paste0(fusion.heatmap.panel,"_tb"),
                                   uri= gsub(pattern = "png",replacement = "csv",x = hm.map[i,"full.path"]),
                                   visible = file.exists(gsub(pattern = "png",replacement = "csv",x = hm.map[i,"full.path"])),
                                   title="Download CSV Data"))    
    )
  }
  
  
  # Add by sample
  #--------------
  for(g in unique(smp.map$genes)){
    
    gene.dropdown.id <- paste0("fusionPanel_horizmenu_bySample_",g)
    gene.dropdown.lab <- g
    
    # ADD DROP DOWN MENU
    addReportContent(
      element = getReportContentById(id = "fusionPanel_horizmenu_bySample", report = advAnalysisReport),
      menuItem(id = gene.dropdown.id, title = gene.dropdown.lab)
    )
    
    # ADD Right PANEL TO DROP DOWN MENU
    gene.dropdown.rightpanel.id <- paste0(gene.dropdown.id,"_Rpanel")
    addReportContent(
      element = getReportContentById(id = gene.dropdown.id, report = advAnalysisReport),
      panel(id = gene.dropdown.rightpanel.id, title = "")
    )
    
    gene.dropdown.rightpanel.VM.id <- paste0(gene.dropdown.rightpanel.id,"_VM")
    addReportContent(
      element = getReportContentById(id = gene.dropdown.rightpanel.id, report = advAnalysisReport),
      verticalMenu(id = gene.dropdown.rightpanel.VM.id, title = "Samples")
    )
    
    for(smp.i in unique(smp.map[smp.map$genes == g,"generic.name"])){
      
      sg.map <- smp.map[smp.map$genes == g & smp.map$generic.name == smp.i,, drop = F]
      
      # Add buttons to vertical menue
      panel.vbuttonMenu.content.id <- paste0(gene.dropdown.rightpanel.VM.id,"_bt",smp.i )
      
      addReportContent(
        element = getReportContentById(id = gene.dropdown.rightpanel.VM.id, report = advAnalysisReport),
        menuItem(id = panel.vbuttonMenu.content.id, title = sg.map$abbr.name )
      )
      
      
      # Add panel
      bygene.bysample.panel <- paste0(panel.vbuttonMenu.content.id,"_panel")
      addReportContent(
        element = getReportContentById(id = panel.vbuttonMenu.content.id, report = advAnalysisReport),
        panel(id = bygene.bysample.panel, title = paste0(sg.map$abbr.name,"vbutpanel"))
      )
      
      
      # Add content
      item.2.show.path <- sg.map$fls
      item.2.show.content.id <- paste0(bygene.bysample.panel,"01")
      
      addReportContent(
        element = getReportContentById(id = bygene.bysample.panel, report = advAnalysisReport),
        reportImage(id = item.2.show.content.id,
                    uri=item.2.show.path,
                    title = "",
                    annotation(id = "fusion_by_sample_plot_annotation", title =  "This plot displays log2 raw counts of every fusion probe for a given sample. Imbalance probes (End probes) and junction probes are shown in separate panels (when they are present). Note: Fusion calls are made as long as evidence is observed in either of the fusion or  imbalance in End probe. The End probe barplot visualizes the expression differences between a sample's 5' and 3' probes for a gene. Many fusions will manifest as substantially increased expression in the 3' probes.These fusions will be immediately obvious when looking at this plot. In genes whose wild-type transcript has high expression, fusions will appear as slight but consistent increases in 3' probes relative to 5' probes. A p-value for the null hypothesis that the mean 5' and 3' probes are equal is shown. The junction probe barplot visualizes the log2 counts of the junction probes. In the absence of a fusion, these probes will all fall in the background of the system. If a fusion is present in less than half the samples, a minimum of 6 samples are present and there is a junction probe targeting it, that probe should have expression noticeably higher than the corresponding counts for samples lacking the fusion. In some infrequent cases, splice variants will affect a fused gene and two junction probes will be elevated in the same sample. Numbers above the bars show the ranking of detection where 1 conveys the highest confidence in detection and subsequent larger ranking convey decreasing confidence. Note: 0 means undetected." ))
      )
      
    }
  }
  
  
  # for(i in 1:nrow(smp.map)){
  #   
  #   smp.dropdown.id <- paste0("fusionPanel_horizmenu_bySample_",smp.map[i,"generic.name"])
  #   smp.dropdown.lab <- smp.map[i,"abbr.name"]
  #   # ADD DROP DOWN MENU
  #   addReportContent(
  #     element = getReportContentById(id = "fusionPanel_horizmenu_bySample", report = advAnalysisReport),
  #     menuItem(id = smp.dropdown.id, title = smp.dropdown.lab)
  #   )
  #   
  #   # Add panel
  #   smp.dropdown.panelcontent.id <- paste0(smp.dropdown.id,"_panel")
  #   addReportContent(
  #     element = getReportContentById(id = smp.dropdown.id, report = advAnalysisReport),
  #     panel(id = smp.dropdown.panelcontent.id, title = "")
  #   )
  #   
  #   # Add content
  #   item.2.show.path <- smp.map[i,"fls"]
  #   item.2.show.content.id <- paste0(smp.dropdown.panelcontent.id,"01")
  #   
  #   addReportContent(
  #     element = getReportContentById(id = smp.dropdown.panelcontent.id, report = advAnalysisReport),
  #     reportImage(id = item.2.show.content.id,
  #                 uri=item.2.show.path,
  #                 title = "",
  #                 annotation(id = "fusion_by_sample_plot_annotation", title =  "This plot displays log2 raw counts of every fusion probe for a given sample. Imbalance probes and junction probes are shown in separate panels (when they are present). Note: fusion calls are made as long as evidence is observed in either of the fusion or imbalance probe.\nThe imbalance probe barplot visualizes the expression differences between a sample's 5\' and 3\' probes for a gene. Many fusions will manifest as substantially increased expression in the 3\' probes; these fusions will be immediately obvious when looking at this plot. In genes whose high wild-type transcript has high expression, fusions will appear as slight but consistent increases in 3\' probes relative to 5\' probes. A p-value for the null hypothesis that the mean 5\' and 3\' probes are equal is provided.\nThe junction probe barplot visualizes the log2 counts of the junction probes. In the absence of a fusion, these probes will all fall in the background of the system. If a fusion is present in less than half the samples, the minimum sample size is > 6 and there is a junction probe targeting it, that probe should have expression noticeably higher than the corresponding counts for samples lacking the fusion. In some infrequent cases, splice variants will affect a fused gene and two junction probes will be elevated in the same sample. Numbers above the bars show the ranking of detection where 1 conveys the highest confidence in detection and subsequent larger ranking convey decreasing confidence. Note: 0 means undetected." ))
  #   )
  #   
  #   
  # }
  
  
  
  
  return(advAnalysisReport) 
}