
#################################
#################################
#   Differential Expression (DE) analyses
#################################
#################################

# fit one model with all chosen covariates, then report results separately for each variable of interest:

# rawdata: raw count data matrix
# predictors: names of predictors in annot data frame
# confounders: names of confounders in annot data frame
# annot: annotation data frame
# predictor.var.type: vector of "cont","cat" 
# confounder.var.type: vector of "cont","cat","surv"
run.DE.analyses = function(filtered.data.list,predictors,confounders,predictor.var.type,annot,
                           confounder.var.type,sampleannot.referencelevels,pval.adjustment = c("BY","BH","Bonf","none"),
                           prb.sets=NULL,saveresults=TRUE,path.to.DE.results,log,plottypearg,path.results,path.inc,path.to.csvs, probe.annotation.id.column,prb.annots,
                           mrna.norm.factors,
                           protein.norm.factors,
                           mrna.bg.mean,
                           protein.bg.mean,
                           mrna.bg.dispersion,
                           protein.bg.dispersion,
                           optimzation = "optimal")
{
  DE.warnings.paragraph = ""
  print("Starting differential expression (DE) analysis")
  cat("LOG:Starting differential expression (DE) analysis",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting differential expression (DE) analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  # extract probe count data from rawdata
  # calculate scaling factors
  # estimate background
  # get covariates from design matrix, make changes to other arguments accordingly.
  if(TRUE){
    covariates = as.data.frame(annot[,c(predictors,confounders),drop=F])
    categorical.type = c("factor","cat","categorical")
    categorical.predictors = predictors[predictor.var.type %in% categorical.type]
    categorical.confounders = confounders[confounder.var.type %in% categorical.type]
    categorical.variables = c(categorical.predictors,categorical.confounders)
    continuous.predictors = setdiff(predictors,categorical.variables)
    continuous.confounders = setdiff(confounders,categorical.variables)
    
    # covert categorical variables to factors, and specify the reference level
    for(variable in categorical.variables) {
      covariates[,variable] = as.factor(covariates[,variable])
      covariates[,variable] = factor(covariates[,variable], levels = unique(as.character(covariates[,variable])))
      if(!(variable %in% names(sampleannot.referencelevels))) {
        # add reference level if not specified in sampleannot.referencelevels
        sampleannot.referencelevels = c(levels(covariates[,variable])[1],sampleannot.referencelevels)
        names(sampleannot.referencelevels)[1] = variable
        cat(paste0("The reference level of ",variable," is set to ",sampleannot.referencelevels[1],".\n"),
            file=log,sep='\n',append=TRUE)
      } else {
        covariates[,variable] = relevel(covariates[,variable],ref=as.character(sampleannot.referencelevels[variable]))
      }
    }
    sampleannot.referencelevels = sampleannot.referencelevels[match(categorical.variables,names(sampleannot.referencelevels))]
    
    # keep track of variable type for the design matrix
    # If there is any continuous variable, the Intercept term is continuous.
    if(length(categorical.variables) < ncol(covariates)){
      var.type = "continuous"
    } else{
      var.type = "categorical"
    }

    # Intercept will cause errors in formula
    names(var.type) = "Intercept"
    for(variable in colnames(covariates)) {
      if(variable %in% categorical.variables) {
        new.types = rep("categorical",length(levels(covariates[,variable]))-1)
        # is it the correct order in design matrix?
        names(new.types) = paste0(variable,levels(covariates[,variable])[-1])
        # names(new.types) = paste0(variable,levels(covariates[,variable])[-1],"_vs_",levels(covariates[,variable])[1])
      } else {
        new.types = "continuous"
        names(new.types) = variable
      }
      var.type = c(var.type,new.types)
    }
    
    predictor.df = covariates[,predictors,drop=F]
    confounder.df = covariates[,confounders,drop=F]
    # guard against empty counfounder.df
    predictor.design = model.matrix(~.,model.frame(~.,data=predictor.df,na.action=na.pass))
    predictor.design = predictor.design[,attr(predictor.design,"assign")!=0,drop=F]
    confounder.design = NULL
    if(ncol(confounder.df)>0) {
      confounder.design = model.matrix(~.,model.frame(~.,data=confounder.df,na.action=na.pass))
      confounder.design = confounder.design[,attr(confounder.design,"assign")!=0,drop=F]
    }
    covariates = as.data.frame(cbind(Intercept=1,predictor.design,confounder.design))
    # may need to check names(var.type)==colnames(covariates)
    # rename the design matrix to avoid duplicate names
    # Need to add a warning on renaming
    colnames(covariates) = names(var.type) = make.unique(names(var.type))
    predictor.design = covariates[,2:(1+ncol(predictor.design)),drop=F]
    if(!is.null(confounder.design)) {
      confounder.design = covariates[,(2+ncol(predictor.design)):ncol(covariates),drop=F]
    }
    predictors = colnames(predictor.design)
    confounders = colnames(confounder.design)
    
    predictor.var.type = var.type[predictors]
    confounder.var.type = var.type[confounders]
    
    samples.with.NAs = rowSums(is.na(covariates))>0
  }
  
  # remove samples that are missing values from any covariate:
  # samples.with.NAs = rowSums(is.na(annot[,c(predictors,confounders),drop=FALSE]))>0
  
  saveresults1 = "genes"
  if(!saveresults){saveresults1="pathways"}
  #covariates = as.data.frame(annot[,c(predictors,confounders)])
  #names(covariates) = c(predictors,confounders)
  
  
  if(FALSE){
    # format the categorical covariates' levels: e.g. change "Basal" to "Basal vs Luminal"  #<------- added 6-22
    for(i in 1:ncol(covariates))
    {
      tempcov = colnames(covariates)[i]
      if(predictor.var.type[tempcov] %in% cat.type)
      {
        replacement = paste("",as.character(covariates[,i]),"vs.",sampleannot.referencelevels[tempcov])
        covariates[,i] = relevel(as.factor(replacement),paste("",sampleannot.referencelevels[tempcov],"vs.",sampleannot.referencelevels[tempcov]))
      }
    }
  }

  # change the first entry intercept to FALSE
  predictor = c(FALSE,rep(TRUE,length(predictors)),rep(FALSE,length(confounders)))
  # predictor = c(rep(TRUE,length(predictors)),rep(FALSE,length(confounders))) # vector indicating which variable is a predictor
  cat("predictor variable:",file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(predictor,file=log,sep='\t',append=TRUE))
  ### evaluate missingness
  missingobs = sum(rowSums(is.na(covariates))>0)
  missingpervariable = colSums(is.na(covariates))
  line1 = line2 =  ""
  if(missingobs>0)
  {
    line1 = paste("There are",missingobs,"samples missing data from at least one covariate.  They will be removed from the differential expression analysis.")
  }
  for(i in 1:length(missingpervariable))
  {
    if(missingpervariable[i]>0)
    {
      line2 = paste(line2,paste("\n",dimnames(covariates)[[2]][i],"is missing",missingpervariable[i],"observations.  They will be removed from the differential expression analysis."),sep="")
    }
  }
  write.table(paste(line1,"\n",line2),file=paste(path.to.DE.results,"//Missing data warning.txt",sep=""),
              row.names=FALSE,col.names=FALSE)
  DE.warnings.paragraph = paste(DE.warnings.paragraph,line1,"\n",line2)
  write.csv(covariates,file=paste(path.to.csvs,"//DE - covariates",saveresults1,".csv",sep=""))  
  
  # for each predictor: log fold change (named by contrast), pval, se
  temp1 = filtered.data.list[[1]][,1]
  #temp = cbind(temp1,covariates)[!samples.with.NAs,]
  temp = cbind(temp1,predictor.df)[!samples.with.NAs,]
  
  # report the terms in the final model, and define more informative result names:
  termsinmodel = c()
  newtermnames = newtermnames.linebreak = c()
  for(i in 2:ncol(temp))
  {
    if(is.numeric(temp[,i]))
    {
      termsinmodel=c(termsinmodel,colnames(temp)[i])
      newtermnames=c(newtermnames,colnames(temp)[i])
      newtermnames.linebreak=c(newtermnames.linebreak,colnames(temp)[i])
      
    }
    if(is.factor(temp[,i]))
    {
      termsinmodel=c(termsinmodel, paste(colnames(temp)[i], unique(temp[,i]), sep=""))
      newtermnames=c(newtermnames, paste(colnames(temp)[i], ": differential expression in ",unique(temp[,i]), " vs. baseline of ", sampleannot.referencelevels[colnames(temp)[i]], sep=""))
      newtermnames.linebreak=c(newtermnames.linebreak, paste(colnames(temp)[i],": ", unique(temp[,i])," vs.", sampleannot.referencelevels[colnames(temp)[i]], sep=""))
    }
  }
  names(newtermnames)=termsinmodel
  names(newtermnames.linebreak)=termsinmodel
  
  cat("Terms entered into model:",file=log,sep='\n',append=TRUE)
  cat(termsinmodel,file=log,sep='\n\n',append=TRUE)
  
  # report the terms in the final model, and define more informative result names:
  termsinmodel2 = c()
  newtermnames2 = newtermnames.linebreak2 = c()
  for(i in 2:ncol(temp))
  {
    if(is.numeric(temp[,i]))
    {
      termsinmodel2=c(termsinmodel2,colnames(temp)[i])
      newtermnames2=c(newtermnames2,colnames(temp)[i])
      newtermnames.linebreak2=c(newtermnames.linebreak2,colnames(temp)[i])
      
    }
    if(is.factor(temp[,i]))
    {
      termsinmodel2=c(termsinmodel2, paste(colnames(temp)[i], unique(temp[,i]), sep=""))
      newtermnames2=c(newtermnames2, paste(colnames(temp)[i], ": differential expression in ",unique(temp[,i]), " vs. baseline of ", sampleannot.referencelevels[colnames(temp)[i]], sep=""))
      newtermnames.linebreak2=c(newtermnames.linebreak2, paste(colnames(temp)[i],": ", unique(temp[,i])," vs.", sampleannot.referencelevels[colnames(temp)[i]], sep=""))
    }
  }
  names(newtermnames2)=termsinmodel2
  names(newtermnames.linebreak2)=termsinmodel2

  cat("Terms entered into model:",file=log,sep='\n',append=TRUE)
  cat(termsinmodel2,file=log,sep='\n\n',append=TRUE)

  # Rewrite THE whole module
  if(FALSE) { # what if we have a covairate named temp1?
    form = paste("temp1~0+")
    for(k in 1:dim(covariates)[2]){form = paste(form,"+",dimnames(covariates)[[2]][k])}
    # enter the formula in lm():
    lm1 = lm(form,data=temp)
    
    ests = matrix(NA,dim(rawdata)[2],dim(summary(lm1)$coefficients)[1]-1)
    dimnames(ests)[[1]] = dimnames(rawdata)[[2]]; dimnames(ests)[[2]] = dimnames(summary(lm1)$coefficients)[[1]][-1]
    # ests = matrix(NA,dim(normalized)[2],length(coef(lm1))-1)
    # dimnames(ests)[[1]] = dimnames(normalized)[[2]]; dimnames(ests)[[2]] = names(coef(lm1))[-1]
  }   
  
  failed.run=FALSE  
  # if ran of DOF of residuals, don't return results:
  # if(length(temp1)-sum(is.na(temp1))<=ncol(covariates))
  if(nrow(filtered.data.list[[1]])<=ncol(covariates))
  {
    failed.run.message <- "\nWarning: There are too many parameters to estimate given the number of samples.   DE analyses will not be run.  Possible solutions: select fewer covariates for DE, or combine/reduce levels for categorical covariates, or increase the number of samples and rerun"
    cat(failed.run.message,file=log,sep='\n',append=TRUE)
    stop(failed.run.message)
  }
  
  # Not required anymore as we have added the reference level if it is not specified.
  # if a reference level is lost, don't return results:
  if(FALSE) {
    lostreflevel = FALSE
    for(i in 1:sum(predictor)) 
    {
      if(predictor.var.type[i]=="categorical")
      {
        includedlevels = lm1$xlevels[[predictors[i]]]
        if(!is.element(sampleannot.referencelevels[predictors[i]],includedlevels)){lostreflevel=TRUE}
      }
    }
    
    if(lostreflevel)
    {
      cat("Warning: No values of a variable's reference level were eligible for the linear model.  DE analyses will not be run.  Examine the sample annotation file and re-run the analysis.",file=log,sep='\n',append=TRUE)
      failed.run=TRUE
      saveresults = FALSE
    } 
  }
  
  if(FALSE) {  
    pvals.adjusted = ses = pvals = ests
    active.variables=as.list(dimnames(ests)[[2]])
    #print(paste("active vars:",active.variables,sep=""))
    Gene.sets = c()
    for(j in 1:dim(ests)[1])
    {
      # define formula:
      temp1=rawdata[,j]
      temp = cbind(temp1,covariates)
      form = paste("temp1~0+")
      for(k in 1:dim(covariates)[2]){form = paste(form,"+",dimnames(covariates)[[2]][k])}
      # enter the formula in lm():
      lm1 = lm(form,data=temp)
      mod = summary(lm1)$coefficients[-1,] 
      if(is.vector(mod)){mod = t(as.matrix(mod))}
      
      ests[j,] = mod[,1]
      ses[j,] = mod[,2]
      pvals[j,] = mod[,4]
      # and get pathway memberships:
      if(length(prb.sets)>0)
      {
        Gene.sets[j] = paste(dimnames(prb.sets)[[2]][prb.sets[dimnames(ests)[[1]][j],]==1],collapse=", ") # what does this do?
      }
    }
  }
  
  if(TRUE) {
    opt.cut = ifelse(optimzation=="optimal",10,0)
    mrna.expression.data = filtered.data.list[["mRNA"]]
    protein.expression.data = filtered.data.list[["protein"]]
    # mrna.NSEData = createNanoStringSet(endogenous=t(mrna.expression.data), positiveControl=new("matrix"), negativeControl=new("matrix"), 
    #                                    housekeepingControl=new("matrix"), designs = as.data.frame(covariates))
    # mrna.NSEData = as(mrna.NSEData, "NanoStringExprSet")
    mrna.NSDE.obj = protein.NSDE.obj = NULL
    if(!is.null(mrna.expression.data)) {
      mrna.NSEData = new("NanoStringExprSet", exprs=t(mrna.expression.data), phenoData=AnnotatedDataFrame(covariates))
      backgroundMean(mrna.NSEData) = mrna.bg.mean
      if(is.vector(mrna.norm.factors)) mrna.norm.factors = replicate(ncol(mrna.expression.data),mrna.norm.factors)
      scalingFactor(mrna.NSEData) = t(mrna.norm.factors)
      backgroundDispersion(mrna.NSEData) = mrna.bg.dispersion
      mrna.NSDE.obj = NSDE.cut(mrna.NSEData,predictors,opt.cut)
    }
    # protein.NSEData = createNanoStringSet(endogenous=t(protein.expression.data), positiveControl=new("matrix"), negativeControl=new("matrix"), 
    #                                       housekeepingControl=new("matrix"), designs = as.data.frame(covariates))
    # protein.NSEData = as(protein.NSEData, "NanoStringExprSet")
    if(!is.null(protein.expression.data) && ncol(protein.expression.data) > 0) {
      protein.NSEData = new("NanoStringExprSet", exprs=t(protein.expression.data), phenoData=AnnotatedDataFrame(covariates))
      backgroundMean(protein.NSEData) = protein.bg.mean
      if(is.vector(protein.norm.factors)) protein.norm.factors = replicate(ncol(protein.expression.data),protein.norm.factors)
      scalingFactor(protein.NSEData) = t(protein.norm.factors)
      backgroundDispersion(protein.NSEData) = protein.bg.dispersion
      protein.NSDE.obj = NSDE.cut(protein.NSEData,predictors,opt.cut)
    }
    #sink(log)

    NSDE.obj = lapply(predictors,function(x) rbind(mrna.NSDE.obj[[x]],protein.NSDE.obj[[x]]))
    #sink(type = "message")
    #sink()
    names(NSDE.obj) = predictors
    
    ests = sapply(predictors, function(x) NSDE.obj[[x]][,"Estimate"])
    ses = sapply(predictors, function(x) NSDE.obj[[x]][,"Std.Error"])
    pvals = sapply(predictors, function(x) NSDE.obj[[x]][,"pvalue"])
    methods = sapply(predictors, function(x) NSDE.obj[[x]][,"method"])
    rownames(ests) = rownames(ses) = rownames(pvals) = rownames(methods) = rownames(NSDE.obj[[predictors[1]]])
    active.variables=as.list(dimnames(ests)[[2]])
    Gene.sets = c()
    
    for(j in 1:dim(ests)[1]) {
      if(length(prb.sets)>0)
      {
        Gene.sets[j] = paste(dimnames(prb.sets)[[2]][prb.sets[dimnames(ests)[[1]][j],]==1],collapse=", ") # what does this do?
      }
    }
  }
  
  ## now identify the columns corresponding to predictors vs. confounders, removing columns of linear dependency:
  # npredictors=0
  npredictors = length(predictors)
  cat("predictor.var.type:",file=log,sep='\n',append=TRUE)
  suppressWarnings(write.table(predictor.var.type,file=log,sep='\t',append=TRUE))
  
  covariate.names = dimnames(covariates)[[2]][-1]
  for(i in 1:sum(predictor))
  {
    
    suppressWarnings(write.table(covariate.names[i],file=log,sep='\t',append=TRUE))
    cat("covariate.names[i]:",file=log,sep='\n',append=TRUE)
    cat(covariate.names[i],file=log,sep='\n',append=TRUE)
    cat(nchar(covariate.names[i]),file=log,sep='\n',append=TRUE)
    cat(nchar(covariate.names[i]),file=log,sep='\n',append=TRUE)
    short=lapply(active.variables, function(x) substr(x,1,nchar(covariate.names[i]))) # what does this mean?
    cat("short:",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(short,file=log,sep='\n',append=TRUE))
    cat("active.levels:",file=log,sep='\n',append=TRUE)
    cat(length(short[short == covariate.names[i]]),file=log,sep='\n',append=TRUE)
    
    
    if(predictor.var.type[i]=="categorical"){
      if(length(short[short == covariate.names[i]])<1){
        warning(paste("Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""))
        cat(paste("LOG:Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.</p>');",sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        DE.warnings.paragraph = paste(DE.warnings.paragraph,"Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.")
      }
      # else if(length(short[short == covariate.names[i]])<(length(levels(covariates[,i+1]))-1)){
      #   warning(paste("Warning: ",covariate.names[i]," has levels that are redundant with another covariate in the data matrix.  They will be removed from all subsequent analyses.", sep=""))
      #   cat(paste("LOG:Warning: ",covariate.names[i]," has levels that are redundant with another covariate in the data matrix.  They will be removed from all subsequent analyses.", sep=""),file=log,sep='\n',append=TRUE)
      #   cat(paste("document.write('<p>Warning: ",covariate.names[i]," has levels that are redundant with another covariate in the data matrix.  They will be removed from all subsequent analyses.</p>');",sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)   
      #   DE.warnings.paragraph = paste(DE.warnings.paragraph,"Warning: ",covariate.names[i],"has levels that are redundant with another covariate in the data matrix.  They will be removed from all subsequent analyses.")
      #   npredictors=npredictors+length(short[short == covariate.names[i]])
      # }
      else{
        # npredictors=npredictors+length(short[short == covariate.names[i]])
        cat("npredictors:",file=log,sep='\n',append=TRUE)
        cat(npredictors,file=log,sep='\n',append=TRUE)
      }
    }
    if(predictor.var.type[i]=="continuous"){
      if(length(short[short == covariate.names[i]])<1){ # what does this mean?
        warning(paste("Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""))
        cat(paste("LOG:Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.", sep=""),file=log,sep='\n',append=TRUE)
        cat(paste("document.write('<p>Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.</p>');",sep=""), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
        DE.warnings.paragraph = paste(DE.warnings.paragraph,"Warning: ",covariate.names[i]," is redundant with another covariate in the data matrix.  It will be removed from all subsequent analyses.")
      }else{
        # npredictors=npredictors+1
        cat("npredictors:",file=log,sep='\n',append=TRUE)
        cat(npredictors,file=log,sep='\n',append=TRUE)
      }
    }
  }
  
  # rename the results with mroe informative names:
  newnames = newtermnames[match(colnames(ests),termsinmodel)]
  #newnames = newtermnames[colnames(ests)]
  #newnames.linebreak = newtermnames.linebreak[colnames(ests)]
  
  ##### Check success of the 
  
  ##### begin save results
  
  if(saveresults)  
  {    
    cat("npredictors:",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(npredictors,file=log,sep='\t',append=TRUE))
    cat("head(pvals):",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(head(pvals),file=log,sep='\t',append=TRUE))
    
    
    out.df.list     <- list()
    fdr.cutoff.list <- list()
    pvals.adjusted = matrix(NA,nrow(pvals),ncol(pvals))
    rownames(pvals.adjusted) = rownames(pvals)
    colnames(pvals.adjusted) = colnames(pvals)
    # volcano plot, coloring by significance level and naming top 100 genes
    for(i in 1:npredictors)
    {
      
      # write output tables for predictors
      # out = cbind(ests[,i],ses[,i], ests[,i]-1.96*ses[,i],ests[,i]+1.96*ses[,i],pvals[,i],methods[,i])
      out = data.frame(ests[,i],ses[,i], ests[,i]-1.96*ses[,i],ests[,i]+1.96*ses[,i],
                       2^ests[,i], 2^(ests[,i]-1.96*ses[,i]),2^(ests[,i]+1.96*ses[,i]),
                       pvals[,i],methods[,i],stringsAsFactors=F)
      dimnames(out)[[2]] = c("Log2 fold change","std error (log2)", "Lower confidence limit (log2)","Upper confidence limit (log2)",
                             "Linear fold change", "Lower confidence limit (linear)","Upper confidence limit (linear)",
                             "P-value","method")
      extracolumn=FALSE
      
      if(pval.adjustment=="BY")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="BY")
        pvals.adjusted[,i]=pvals.a
        out = cbind(out,pvals.a); dimnames(out)[[2]][dim(out)[2]] = paste(toupper(pval.adjustment), ".p.value", sep = "") #"FDR"
      }
      if(pval.adjustment=="BH")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="BH")
        pvals.adjusted[,i]=pvals.a
        out = cbind(out,pvals.a); dimnames(out)[[2]][dim(out)[2]] = paste(toupper(pval.adjustment), ".p.value", sep = "") #"FDR"
      }
      if(pval.adjustment=="Bonf")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="bonferroni")
        pvals.adjusted[,i]=pvals.a
        out = cbind(out,pvals.a); dimnames(out)[[2]][dim(out)[2]] = paste(toupper(pval.adjustment), ".p.value", sep = "") #"Bonf.p.value"
      }
      if(pval.adjustment=="none")
      {
        extracolumn=TRUE
        pvals.a = p.adjust(pvals[,i],method="none")
        pvals.adjusted[,i]=pvals.a
        out = cbind(out,pvals.a); dimnames(out)[[2]][dim(out)[2]] = "Unadj.p.value"
      }
      
      #out = cbind(as.data.frame(signif(out,3)),Gene.sets)
      method.col = which(colnames(out)=="method")
      out.method = out[,method.col]
      out = data.frame(signif(out[,-method.col],3),check.names = F)
      out$method = out.method
      out$Gene.sets <- Gene.sets
      
      # order by p-val:
      out = out[order(pvals[,i]),]
      out$probe.ID <- rownames(out)
      rownames(out) <- paste(prb.annots[rownames(out),"Probe.Label"],prb.annots[rownames(out),"Analyte.Type"],sep = "-")
      write.csv(out,file=correct.filename(paste(path.to.DE.results,"//DE results - ",make.names(dimnames(ests)[[2]][i]),".csv",sep="")))  
      
      # and write a top hits version:
      out2 = out[1:min(20,nrow(out)),]
      out2[,1:4]=signif(out2[,1:4],3)
      out2[,5]=signif(out2[,5],4)
      if(extracolumn){out2[,6]=signif(out2[,6],4)}
      
      write.csv(out2,file=correct.filename(paste(path.to.DE.results,"//DE results - Top 20 - ",make.names(dimnames(ests)[[2]][i]),".csv",sep="")))
      
      ##### volcano plots:
      
      out.df           <- as.data.frame(out)
      out.df           <- merge(out.df, prb.annots, by.x = "probe.ID", by.y = probe.annotation.id.column, all = F)
      rownames(out.df) <- paste(prb.annots[out.df$probe.ID,"Probe.Label"],prb.annots[out.df$probe.ID,"Analyte.Type"],sep = "-")
      
      out.df$volcanocol = rep("black",dim(out.df)[1])
      out.df$volcanocol[out.df[,"P-value"] < 0.010] = codecols2[1]
      out.df$volcanocol[out.df[,"P-value"] < 0.001] = codecols2[2]
      
      out.df$volcanopch = rep(1,dim(out.df)[1])
      out.df$volcanopch[out.df[,"P-value"] < 0.010] = 16
      out.df$volcanopch[out.df[,"P-value"] < 0.001] = 16
      
      # if adjusted p-values are calculated, draw FDR cutoff lines:
      
      if(is.element(pval.adjustment,c("BH","BY", "Bonf")))   # check on adding in bonferroni?  jbucci - 12/15/2015
      {
        
        ### 10/14/2015 - JBUCCI: updated code to incorporate protein data
        fdr.cutoffs <- c(0,0,0,0)
        
        below.cutoff <- which(out.df[,paste(toupper(pval.adjustment), ".p.value", sep = "")] < 0.01)
        if(length(below.cutoff)>0) fdr.cutoffs[1] = max(0, max(out.df[below.cutoff,"P-value"]))
        
        below.cutoff <- which(out.df[,paste(toupper(pval.adjustment), ".p.value", sep = "")] < 0.05)
        if(length(below.cutoff)>0) fdr.cutoffs[2] = max(0, max(out.df[below.cutoff,"P-value"]))
        
        below.cutoff <- which(out.df[,paste(toupper(pval.adjustment), ".p.value", sep = "")] < 0.10)
        if(length(below.cutoff)>0) fdr.cutoffs[3] = max(0, max(out.df[below.cutoff,"P-value"]))
        
        below.cutoff <- which(out.df[,paste(toupper(pval.adjustment), ".p.value", sep = "")] < 0.50)
        if(length(below.cutoff)>0) fdr.cutoffs[4] = max(0, max(out.df[below.cutoff,"P-value"]))
        
        #print(fdr.cutoffs)
        fdr.cutoff.list[[i]] <- fdr.cutoffs
        
        # NEED to continue to work, allow for additional analytes beyond mRNA and protein
        #out.df$volcanopch = rep(7,dim(out.df)[1])
        
        # 12/9/2015 - jbucci - designate shapes for each analyte in volcano plots
        
        unfilled.Shapes <- c(1,2,0,5)
        filled.Shapes   <- c(16,17,15,18)
        
        for(l in unique(out.df$Analyte.Type))
        {
          # out.df[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] >= fdr.cutoffs[4],"volcanopch"] = unfilled.Shapes[which(unique(out.df$Analyte.Type) == l)]
          # out.df[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < fdr.cutoffs[4],"volcanopch"] = filled.Shapes[which(unique(out.df$Analyte.Type) == l)]
          out.df$volcanopch[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] >= fdr.cutoffs[4]] = unfilled.Shapes[which(unique(out.df$Analyte.Type) == l)]
          out.df$volcanopch[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < fdr.cutoffs[4]] = filled.Shapes[which(unique(out.df$Analyte.Type) == l)]
        }
        
        # 12/9/2015 - jbucci - designate colors for each analyte in volcano plots
        
        #color.Analyte     <- c("chartreuse", "magenta")
        color.Analyte     <- c("slateblue", "darkgoldenrod")
        out.df$volcanocol <- rep("azure4",dim(out.df)[1])
        
        for(l in unique(out.df$Analyte.Type))
        {
          out.df$volcanocol[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < fdr.cutoffs[4]] = paste(color.Analyte[which(unique(out.df$Analyte.Type) == l)], "4", sep = "")
          out.df$volcanocol[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < fdr.cutoffs[3]] = paste(color.Analyte[which(unique(out.df$Analyte.Type) == l)], "3", sep = "")
          out.df$volcanocol[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < fdr.cutoffs[2]] = paste(color.Analyte[which(unique(out.df$Analyte.Type) == l)], "2", sep = "")
          out.df$volcanocol[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < fdr.cutoffs[1]] = paste(color.Analyte[which(unique(out.df$Analyte.Type) == l)], "1", sep = "")
        }
      }
      
      
      if(!is.element(pval.adjustment,c("BH","BY", "Bonf")))   # check on adding in bonferroni?  jbucci - 12/15/2015
      {
        # 12/9/2015 - jbucci - designate shapes for each analyte in volcano plots
        
        unfilled.Shapes <- c(1,2,0,5)
        filled.Shapes   <- c(16,17,15,18)
        
        for(l in unique(out.df$Analyte.Type))
        {
          out.df$volcanopch[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] >= 0.010] = unfilled.Shapes[which(unique(out.df$Analyte.Type) == l)]
          out.df$volcanopch[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < 0.010] = filled.Shapes[which(unique(out.df$Analyte.Type) == l)]
        }
        
        # 12/9/2015 - jbucci - designate colors for each analyte in volcano plots
        
        #color.Analyte     <- c("chartreuse", "magenta")
        color.Analyte     <- c("slateblue", "darkgoldenrod")
        out.df$volcanocol <- rep("azure4",dim(out.df)[1])
        
        for(l in unique(out.df$Analyte.Type))
        {
          out.df$volcanocol[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < 0.010] = paste(color.Analyte[which(unique(out.df$Analyte.Type) == l)], "4", sep = "")
          out.df$volcanocol[out.df[,"Analyte.Type"] == l & out.df[,"P-value"] < 0.001] = paste(color.Analyte[which(unique(out.df$Analyte.Type) == l)], "1", sep = "")
        }
      }
      
      #head(out.df)
      
      #write.csv(out.df,file=correct.filename(paste(path.to.DE.results,"//DE results - check - ",make.names(dimnames(ests)[[2]][i]),".csv",sep="")))  
      
      #table(out.df$volcanopch)
      #table(out.df$volcanocol)
      
      ### 10/14/2015 - JBUCCI: updated code to incorporate protein data
      out.df2 = out.df
      out.df = out.df[!is.na(out.df[,"Log2 fold change"]) & !is.na(out.df[,"P-value"]),]
      
      for(r in 1:length(plottypearg))
      {
        plottype=plottypearg[r];
        tempfilename = drawplot(paste(path.to.DE.results,"//volcano plot",make.names(dimnames(ests)[[2]][i]),sep=""),plottype,width=1.3,height=1.3)
        tempfilename=gsub(path.results,"results",tempfilename)
        
        if(r==1)
        {
          strTemp=paste("document.write('      	  <li>Volcano Plot - ",dimnames(pvals)[[2]][i],"<br><img src=\"",tempfilename,".png\"></img></li>');\n",sep="")
          cat(strTemp,file=paste(path.inc,"//panel2_2.js",sep=""),append=TRUE)
        }
        
        #plot(ests[,i],-log10(pvals[,i]),xlab="log2(fold change)",ylab = "-log10(p-value)",main=newnames[i],col=volcanocol,pch=volcanopch)   
        #plot.file = file.path(paste(path.to.DE.results,"//volcano plot",make.names(dimnames(ests)[[2]][i]), ".pdf", sep=""))
        #pdf(file = plot.file, width = 8, height = 8, family = "sans")
        
        par(xpd=TRUE)
        par(mar=c(5,4,5,1))
        par(las = 1)
        if(TRUE) {
          # out.df[out.df[,"P-value"]==0,"P-value"] = .Machine$double.eps
          out.df = out.df[out.df[,"P-value"]>0,]
          pval.cutoff = quantile(out.df[,"P-value"],0.25)
          FC.cutoff = max(abs(out.df[out.df[,"P-value"]<pval.cutoff,"Log2 fold change"]),na.rm=T)
          filtered.FC = abs(out.df[,"Log2 fold change"])>FC.cutoff & out.df[,"P-value"] > pval.cutoff
          out.df = out.df[!filtered.FC,]
          ylim.max = max(-log10(out.df[,"P-value"]),na.rm=T)
          for(l in unique(out.df$Analyte.Type))
          {
            if(which(unique(out.df$Analyte.Type) == l) == 1)
            {
              ttl.i <- newnames[i]
              tmp <- unlist(strsplit(ttl.i,split = " "))
              if(length(tmp)>3)
                ttl.i <- paste(paste(tmp[1:3],collapse=" "),"\n",paste(tmp[-(1:3)],collapse = " "),sep = "")
              plot(out.df[out.df[,"Analyte.Type"] == l,"Log2 fold change"], -log10(out.df[out.df[,"Analyte.Type"] == l,"P-value"]), 
                   xlab = "log2(fold change)", ylab = "-log10(p-value)",main=ttl.i, cex.main = 1,
                   xlim = c(min(out.df[,"Log2 fold change"]), max(out.df[,"Log2 fold change"])), 
                   ylim = c(0, ylim.max),
                   col = out.df[out.df[,"Analyte.Type"] == l,"volcanocol"], 
                   pch = out.df[out.df[,"Analyte.Type"] == l,"volcanopch"])
            } else{
              points(out.df[out.df[,"Analyte.Type"] == l,"Log2 fold change"], -log10(out.df[out.df[,"Analyte.Type"] == l,"P-value"]), 
                     col = out.df[out.df[,"Analyte.Type"] == l,"volcanocol"],
                     pch = out.df[out.df[,"Analyte.Type"] == l,"volcanopch"])
            }
          }
        }
        if(FALSE) {
          xcord = log10(abs(out.df[,"Log2 fold change"])) 
          ycord = -log10(out.df[,"P-value"])
          xcord.mid = min(floor(min(xcord,na.rm=T)),0)
          xcord.max = ceiling(max(xcord,na.rm=T))
          xcord.sign = sign(out.df[,"Log2 fold change"])
          
          for(l in unique(out.df$Analyte.Type))
          {
            if(which(unique(out.df$Analyte.Type) == l) == 1)
            {
              ttl.i <- newnames[i]
              tmp <- unlist(strsplit(ttl.i,split = " "))
              if(length(tmp)>3)
                ttl.i <- paste(paste(tmp[1:3],collapse=" "),"\n",paste(tmp[-(1:3)],collapse = " "),sep="")
              # what to do with pvalue < 0
              
              plot(xcord[out.df[,"Analyte.Type"]==l & xcord.sign>0]-xcord.mid, ycord[out.df[,"Analyte.Type"]==l & xcord.sign>0], 
                   xlab = "log2(fold change)", ylab = "-log10(p-value)",main=ttl.i, cex.main = 1,
                   xlim = c((xcord.max-xcord.mid)*c(-1,1)), 
                   ylim = c(min(ycord,na.rm=T),max(ycord,na.rm=T)),
                   col = out.df[out.df[,"Analyte.Type"] == l & xcord.sign>0,"volcanocol"], 
                   pch = out.df[out.df[,"Analyte.Type"] == l & xcord.sign>0,"volcanopch"],
                   xaxt = "n")
              points(-xcord[out.df[,"Analyte.Type"]==l & xcord.sign<0]+xcord.mid, ycord[out.df[,"Analyte.Type"]==l & xcord.sign<0], 
                     col = out.df[out.df[,"Analyte.Type"] == l & xcord.sign<0,"volcanocol"], 
                     pch = out.df[out.df[,"Analyte.Type"] == l & xcord.sign<0,"volcanopch"])
            } else {
              points(xcord[out.df[,"Analyte.Type"]==l & xcord.sign>0]-xcord.mid, ycord[out.df[,"Analyte.Type"]==l & xcord.sign>0],
                     col = out.df[out.df[,"Analyte.Type"] == l & xcord.sign>0,"volcanocol"],
                     pch = out.df[out.df[,"Analyte.Type"] == l & xcord.sign>0,"volcanopch"])
              points(-xcord[out.df[,"Analyte.Type"]==l & xcord.sign<0]+xcord.mid, ycord[out.df[,"Analyte.Type"]==l & xcord.sign<0],
                     col = out.df[out.df[,"Analyte.Type"] == l & xcord.sign<0,"volcanocol"],
                     pch = out.df[out.df[,"Analyte.Type"] == l & xcord.sign<0,"volcanopch"])
            }
          }
          xcord1.at = seq(xcord.mid,xcord.max,by=(xcord.max-xcord.mid)/4) 
          xcord1.lab = c(0,10^(xcord1.at[-1]))
          xcord1.at = xcord1.at - xcord.mid
          xcord2.at = -xcord1.at[length(xcord1.at):2]
          xcord.at = c(xcord2.at,xcord1.at)
          xcord.lab = round(c(-xcord1.lab[length(xcord1.lab):2],xcord1.lab),1)
          axis(1,at=xcord.at,labels=xcord.lab)
        }
        
        # draw lines for pval cutoffs if there's no FDR calculated:
        if(!is.element(pval.adjustment,c("BH","BY", "Bonf")))
        {
          #legend("bottomright",pch=c(16,16),col=c(codecols2[1],codecols2[2]),legend = c("p < 0.01","p < 0.001"))
          legend("topleft",bty="n",inset=c(0,-0.16), lty=1:2, legend = c("p-value < 0.01","p-value < 0.001"), cex = 1)
          legend("topright",bty="n",inset=c(0,-0.16), legend = unique(out.df$Analyte.Type), pch = filled.Shapes[1:length(unique(out.df$Analyte.Type))], 
                 col = color.Analyte[1:length(unique(out.df$Analyte.Type))], cex = 1)
        }
        
        if(!is.element(pval.adjustment,c("BH","BY", "Bonf")))
        {
          par(xpd = F)
          abline(h=-log10(c(0.01,0.001)),lty= c(1:2))
        }
        
        # draw lines for FDR cutoffs if FDR has been calculated:
        if(is.element(pval.adjustment,c("BH","BY", "Bonf")))
        {
          legend("topleft",bty="n",inset=c(0,-0.16),lty=1:4,legend = paste("adj. p-value <",c("0.01","0.05","0.10","0.50")), cex = 1)
          legend("topright",bty="n",inset=c(0,-0.16), legend = unique(out.df$Analyte.Type), pch = filled.Shapes[1:length(unique(out.df$Analyte.Type))], 
                 col = color.Analyte[1:length(unique(out.df$Analyte.Type))], cex = 1)
        }
        
        # draw lines for FDR cutoffs if FDR has been calculated:
        if(is.element(pval.adjustment,c("BH","BY", "Bonf")))
        {
          par(xpd = F)
          abline(h=-log10(fdr.cutoffs),lty=1:4)
        }
        
        if(FALSE) {
          top40rows = min(nrow(out.df),40)
          top40 = out.df[order(out.df[,"P-value"],decreasing=FALSE),][1:top40rows,]
          top40.fc = top40[,"Log2 fold change"]
          top40.xcord1 = log10(top40.fc[top40.fc>0]) - xcord.mid
          top40.xcord2 = -log10(-top40.fc[top40.fc<0]) + xcord.mid
          text(x=top40.xcord1,y = (-log10(top40[top40.fc>0,"P-value"]) - 0.075),labels=top40[top40.fc>0,"Probe.Label"],cex=.8, font = 1)
          text(x=top40.xcord2,y = (-log10(top40[top40.fc<0,"P-value"]) - 0.075),labels=top40[top40.fc<0,"Probe.Label"],cex=.8, font = 1)
        }
        if(TRUE) {
          top40 = out.df[order(out.df[,"P-value"],decreasing=FALSE),][1:40,]
          text(x=top40[,"Log2 fold change"],y=(-log10(top40[,"P-value"]) - 0.075),labels=top40[,"Probe.Label"],cex=.8, font = 1)
        }
        dev.off()
      }
      
      
      ### 10/14/2015 - JBUCCI: updated code to incorporate protein data
      ### END
      
      #START HTML TABLE
      #numRow=nrow(out2)
      #numCol=ncol(out2)
      
      #genes=rownames(out2)	
      #cols=dimnames(out2)[[2]]
      out.df = out.df2
      out.df.list[[i]] <- out.df      
    }
    
  } # end if(saveresults)
  
  #####
  #####  
  
  print("Creating differential expression (DE) analysis results files")
  cat("LOG:Creating differential expression (DE) analysis results files",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Creating differential expression (DE) analysis results files</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  # tweak covariate names to avoid illegal characters:
  
  write.csv(data.frame(ests,row.names = paste(prb.annots[rownames(ests),"Probe.Label"],prb.annots[rownames(ests),"Analyte.Type"],sep = "-")),
            file=paste(path.to.csvs,"//DE results - ests",saveresults1,".csv",sep=""))
  write.csv(data.frame(ses,row.names = paste(prb.annots[rownames(ses),"Probe.Label"],prb.annots[rownames(ses),"Analyte.Type"],sep = "-")),
            file=paste(path.to.csvs,"//DE results - ses",saveresults1,".csv",sep=""))
  write.csv(data.frame(pvals,row.names = paste(prb.annots[rownames(pvals),"Probe.Label"],prb.annots[rownames(pvals),"Analyte.Type"],sep = "-")),
            file=paste(path.to.csvs,"//DE results - pvals",saveresults1,".csv",sep=""))
  dimnames(ests)[[2]] = gsub("/",".",dimnames(ests)[[2]])
  dimnames(pvals)[[2]] = dimnames(ests)[[2]]
  dimnames(ses)[[2]] = dimnames(ests)[[2]]
  
  # finally, save lists of significant genes within each covariate:
  if(pval.adjustment!="none")
  {
    fdr.thresh = 0.10
    is.sig = pvals.adjusted<fdr.thresh
  }
  if(pval.adjustment=="none")
  {
    p.thresh = 0.01
    is.sig = pvals.adjusted<p.thresh
  }
  
  tempnames = colnames(is.sig)
  sig.gene.sets = list()
  # get a list of genes for each predictor:
  
  for(i in 1:length(predictors))
  {
    # relevant columns: everything starting with the predictor name:
    relevant.columns = substr(tempnames,1,nchar(predictors[i]))==predictors[i]
    if(sum(relevant.columns)>0)
    {
      tempgenes = rownames(is.sig)[rowSums(is.sig[,relevant.columns,drop=FALSE])>0]
      if(length(tempgenes)>2)
      {
        sig.gene.sets[[length(sig.gene.sets)+1]] = tempgenes
        if(pval.adjustment=="none")
        {
          names(sig.gene.sets)[length(sig.gene.sets)] = paste(predictors[i]," p < ",p.thresh,sep="")
        }
        if(pval.adjustment!="none")
        {
          names(sig.gene.sets)[length(sig.gene.sets)] = paste(predictors[i]," Adj.p.value < ",fdr.thresh,sep="")
        }
        
      }
    }
  }
  
  out = list(out.df.list = out.df.list, fdr.cutoff.list = fdr.cutoff.list, 
             ests=ests,ses=ses,pvals=pvals,npredictors=npredictors,Pathways=Gene.sets,pvals.adjusted=pvals.adjusted,
             pval.adjustment=pval.adjustment,DE.warnings.paragraph=DE.warnings.paragraph,sig.gene.sets=sig.gene.sets,
             failed.run=failed.run,newtermnames=newtermnames,newtermnames.linebreak=newtermnames.linebreak,newtermnames2=newtermnames2)
  
  
  print("Finished differential expression (DE) analysis")
  cat("LOG:Finished differential expression (DE) analysis",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finished differential expression (DE) analysis</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  return(out)
}
