
# TFS checked in - 02/15/2016 - jbucci 
###########################################################################
###########################################################################
# data exploration and QC
###########################################################################
###########################################################################


runQC = function(normalized,
                 rawdata,
                 snv.analyzed,
                 fusion.analyzed,
                 thresholding.value,
                 probe.annot,
                 rawDataAll.list,
                 normalizedData.list,
                 HKnormalized,
                 HKs,
                 covariates,
                 covariates.type,
                 prb.sets=NULL,
                 annotcols,
                 annotcols2,
                 codecols,
                 path.to.QC.results,
                 log,
                 plottypearg,
                 path.results,
                 path.inc,
                 path.to.csvs,
                 heatmap.cols=NULL)  
{
  
  
  
  
  
  print("Starting Overview")
  cat("LOG:Starting Overview",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Starting Overview</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  
  #The philosophy of QC has been to show overview across all analyte type
  # Therefore, we are going to restrict data to complete.cases
  # normalized <- normalized[complete.cases(normalized),]
  # rawdata <- rawdata[complete.cases(rawdata),]
  # normalizedData.list <- lapply(split(data.frame(t(normalized),check.names = F),f = probe.annot[colnames(normalized),"Analyte.Type"]),t)
  # rawDataAll.list <- lapply(split(data.frame(t(rawdata),check.names = F),f = probe.annot[colnames(rawdata),"Analyte.Type"]),t)
  # 
  # # Additionally eliminate for now keep it only to mRNA and protein
  # rawDataAll.list[names(rawDataAll.list)[names(rawDataAll.list) %in% c("SNV","FUSION")]] <- NULL
  # normalizedData.list[names(normalizedData.list)[names(normalizedData.list) %in% c("SNV","FUSION")]] <- NULL
  # 
  # 
  # #Adjust covariates accordingly
  # covariates <- covariates[rownames(rawdata),]
  # vars.nlevels <- sapply(covariates,function(x) nlevels(factor(x)))
  # covariates <- covariates[,vars.nlevels>1,drop=F]
  # 
  # annotcols <- annotcols[rownames(rawdata),]
  # 
  
  
  
  ### heatmap of detected/undetected calls:
  detected = rawdata*NA
  # make detection calls analyte by analyte:
  analytes = names(thresholding.value)
  
  # Change column names: accession numbers to probe label + probe type - 02/03/2016 - jbucci
  probe.annot$probeLabel.Type <- paste(probe.annot$Probe.Label, probe.annot$Analyte.Type, sep = " - ")
  
  filtered <- filter.out.low.counts(raw = rawdata,prb.annots=probe.annot,thresholding.frequency = thresholding.frequency, thresholding.value = thresholding.value,thresholding.perform = thresholding.perform)
  
  pruneout = list()
  for(an in analytes)
  {
    # if the thresholding value is specified, use it:
    if(is.numeric(thresholding.value[[an]]))
    {
      thresh = rep(thresholding.value[[an]],nrow(rawdata))
    }
    # if it's AUTO, set the thresh at double the median of the neg controls:
    if(thresholding.value[[an]]=="AUTO")
    {
      neg.probes = rownames(probe.annot)[stringi::stri_detect(probe.annot$Control.Type,regex = "NEG|Neg")&(probe.annot$Analyte.Type==an) ]
      thresh = apply(rawdata[,neg.probes],1,median)*2
    }
    # which probes are this analyte:
    tempprobes = rownames(probe.annot)[probe.annot$Analyte.Type==an]
    # make the detection call:
    detected[,tempprobes] = sweep(rawdata[,tempprobes],1,thresh,">")
    detected[is.na(detected)] = 1
    #pruneout[[an]] = as.matrix(apply(detected[,tempprobes], 2, function(x) all(x==0)),ncol=1)
    #pruneout[[an]] = as.matrix(as.numeric(apply(rawdata[,tempprobes],2,max,na.rm=T)>mean(thresh,na.rm=T)),ncol=1)
    pruneout[[an]] = matrix(filtered$prune[tempprobes],ncol=1)
    rownames(pruneout[[an]]) = probe.annot[tempprobes,"probeLabel.Type"]
    colnames(pruneout[[an]]) = "below.threshold"
    #pruneout[[an]][,"below.threshold"] = ifelse(pruneout[[an]][,"below.threshold"]>0,"#FFFFFF","#FF0000")
    pruneout[[an]][,"below.threshold"] = ifelse(pruneout[[an]][,"below.threshold"],"blue","gray50")
  }
  
  # give the matrix informative names:
  #prune.list = ""
  #prune.list = colnames(detected)[apply(detected,2,max)==0]
  #prune.list = cbind(prune.list,probe.label=probe.annot[prune.list,"probeLabel.Type"])
  detected = rbind(probe.annot[colnames(detected),"Analyte.Type"],detected)
  colnames(detected) = probe.annot[colnames(detected),"Probe.Label"]
  detected <- detected[,tolower(detected[1,])%in% c("mrna", "protein")]
  # write the .csv:
  write.csv(detected,file=paste0(path.to.QC.results,"//above background detection call.csv"))
  #colnames(prune.list) = NULL
  #write.table(prune.list,file=paste0(path.to.QC.results,"//pruned probe list.csv"),sep=",",col.names=F,row.names=F,quote=F)
  
  # if(length(covariates)>0)
  # {
  #   rowcols = annotcols
  # }
  
  heatmap.cols.matrix <- heatmap.cols[["annotcols"]]
  # Append the DNA-calls to rowcols
  if(!is.null(snv.analyzed)){
    snv.tab <- get.snv.call.summary.mat(snv = snv.analyzed,mat.format = "wide")[rownames(heatmap.cols[["annotcols"]]), , drop=F]
    snv.sum.col <- apply(snv.tab== rev(snv.analyzed$call.rule$categories$call.type)[1], MARGIN = 1, FUN = function(x) any(x))
    snv.sum.col <- c("gray50","blue")[1+snv.sum.col]
    heatmap.cols.matrix <- cbind(heatmap.cols.matrix, SNV = snv.sum.col)
  }
  
  if(!is.null(fusion.analyzed)){
    fusion.tab <- get.fusion.call.summary.mat(data = fusion.analyzed,mat.format = "wide")[rownames(heatmap.cols[["annotcols"]]), , drop=F]
    fusion.sum.col <- apply(fusion.tab, MARGIN = 1, FUN = function(x) any(grepl(pattern = "\\+", x = x)))
    fusion.sum.col <- c("gray50","blue")[1+fusion.sum.col]
    heatmap.cols.matrix <- cbind(heatmap.cols.matrix, Fusion = fusion.sum.col)
  }
  
  rowcols = heatmap.cols.matrix
  write.csv(rowcols, file=paste0(path.to.csvs,"//heatmap_sample_annotations_bycolor.csv"))
  
  t.rawData.list <- list()
  for(l in 1:length(rawDataAll.list)){
    rawData.sub           <- rawDataAll.list[[l]]
    row.col.Match         <- match(colnames(rawData.sub), rownames(probe.annot))
    colnames(rawData.sub) <- probe.annot$probeLabel.Type[row.col.Match]
    
    if(dim(rawData.sub)[l] == 0){
      next
    } else {
      t.rawData.list[[l]]      <- t(rawData.sub)
      names(t.rawData.list)[l] <- names(rawDataAll.list)[l]
    }
  }
  
  #linearbreaks = c(min(c(50,neg.control.median)),50,100,500,10000000)
  linearbreaks = c(25,50,100,500,10000000) 
  breaks = c(0,(linearbreaks))
  offset.breaks= c(1e-10,0,0,0,0,0)
  hmcols = c("dodgerblue4","dodgerblue1","grey","orange4","orange")
  
  rowColor.list = rowColor.list0 = pruneout
  # rowColor.list <- list()
  for(i in 1:length(t.rawData.list)){
    g = names(t.rawData.list)[i]
    # for (g in names(pruneout)) {
    # change this for pruned out probes
    # rowColor.list[[g]] <-matrix(rep(c(gray.colors(5)[g]), dim(t.rawData.list[[g]])[1]))
    # if (g %in% names(pruneout)) rowColor.list[[g]] = pruneout[[g]]
    # else {
    #   rowColor.list[[g]] = as.matrix(rep("#FFFFFF",nrow(t.rawData.list[[g]])),ncol=1)
    #   rownames(rowColor.list[[g]]) = rownames(t.rawData.list[[g]])
    #   colnames(rowColor.list[[g]]) = "below.threshold"
    # }
    colname.abr.map <- strtrim(abbreviate(toupper(colnames(t.rawData.list[[g]])),minlength = 30), width = 30)
    names(colname.abr.map) <- colnames(t.rawData.list[[g]])
    attr(t.rawData.list[[g]],"colname.abr.map") <- colname.abr.map
    colnames(t.rawData.list[[g]])   <- attr(t.rawData.list[[g]],"colname.abr.map")  #colnames(t.rawData.list[[i]]) strtrim(x, width)
  }
  # rowColor.list0 = rowColor.list
  
  # for(i in 1:length(t.rawData.list))
  # {
  # rownames(rowColor.list[[i]])     <- rownames(t.rawData.list[[i]])
  # colnames(t.rawData.list[[i]])   <- strtrim(abbreviate(toupper(colnames(t.rawData.list[[i]]))), width = 30)  #colnames(t.rawData.list[[i]]) strtrim(x, width)
  # }
  
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//raw data heatmap",sep=""),plottype, width=2,height=2,heatmapres=TRUE)
    tempfilename=gsub(path.results,"results",tempfilename)
    par(oma=c(1,8,1,1)) 
    #par(xpd=T)
    par(mar=c(1,2,1,1))
    # reorder probes
    heatmap.plus.multipanel(Xlist = t.rawData.list, rowcolorlist = rowColor.list, ColSideColors = as.matrix(rowcols), scale="none", breaks=breaks, margins=c(12,6), col=hmcols, cexCol = 0.85, cexRow = 0.75)
    #par(xpd=T)
    #legend(x=0.85, y=1.0, legend = c("True","False"), fill = c("darkorchid1","lightgoldenrod1"), cex=0.75, title = c("QC Flags"), bty = "n", horiz = T)
    #non.qc.annot <- which(grepl(pattern = "\\.Flag", names(heatmap.cols[["annotcols2"]])) == F)
    #for(i in rev(non.qc.annot)){
    #  legend(x=0.85, y=(1.0-(which(rev(non.qc.annot) == i)*0.1)), legend = names(heatmap.cols[["annotcols2"]][[i]]), fill = as.character(heatmap.cols[["annotcols2"]][[i]]), cex=0.75, title = names(heatmap.cols[["annotcols2"]][i]), bty = "n")
    #}
    #
    #lgnd.ttl <- "flag/prune"
    #
    #if(any(c("SVN","Fusion") %in% colnames(rowcols))){
    #  lgnd.ttl <- paste0(lgnd.ttl,"/\n")
    #  
    #  if("SNV" %in% colnames(rowcols))
    #    lgnd.ttl <- paste0(lgnd.ttl,"snv/")
    #  
    #  if("Fusion" %in% colnames(rowcols))
    #    lgnd.ttl <- paste0(lgnd.ttl,"fusion")
    #}


    
    #legend(x=0.85, y=0.875, legend = c("yes","no"), fill = c("blue","gray50"), cex=1, title = lgnd.ttl, bty = "n")
    #legend(x=-0.85, y=0.15, legend = c("yes","no"), fill = c("blue","gray50"), cex=1, title = lgnd.ttl, bty = "n")
    # for (i in (ncol(covariates)+1):length(heatmap.cols$annotcols2)) {
    #   annot.name = names(heatmap.cols$annotcols2)[i]
    #   annot.ypos = 0.85 - (i-ncol(covariates)-1)/(length(heatmap.cols$annotcols2)-ncol(covariates)-1)*0.5
    #   legend(x=0.85,y=annot.ypos, legend=names(heatmap.cols$annotcols2[[annot.name]]), fill=heatmap.cols$annotcols2[[annot.name]],cex=0.7,title=annot.name,bty="n")
    # }
    dev.off()
    par(xpd=F)
  }
  
  for(i in 1:length(heatmap.cols[["annotcols2"]])){
    heatmap.cols[["annotcols3"]][[i]]      <- ceiling(length(names(heatmap.cols[["annotcols2"]][[i]]))/3)
    names(heatmap.cols[["annotcols3"]])[i] <- names(heatmap.cols[["annotcols2"]])[i]
  }
  
  non.qc.annot       <- which(grepl(pattern = "\\.Flag", names(heatmap.cols[["annotcols3"]])) == F)
  legend.total.lines <- sum(as.numeric(heatmap.cols[["annotcols3"]][non.qc.annot])) + 
    length(heatmap.cols[["annotcols3"]][non.qc.annot]) + 2 # add 2 for the QC Flags key
  
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//raw data heatmap - color key",sep=""),plottype,width=0.5,height=1.25)
    tempfilename=gsub(path.results,"results",tempfilename)
    
    if(legend.total.lines <= 6){
      cex.legend   <- 1.0
      vertical.adj <- 0.175
    } else if(legend.total.lines == 7){
      cex.legend <- 0.95
      vertical.adj <- 0.175
    } else if(legend.total.lines >= 8 & legend.total.lines <= 12){
      cex.legend <- 0.90
      vertical.adj <- 0.15
    } else if(legend.total.lines >= 13 & legend.total.lines <= 17){
      cex.legend <- 0.85
      vertical.adj <- 0.135
    } else if(legend.total.lines >= 18){
      cex.legend <- 0.75
      vertical.adj <- 0.115
    }

    par(mfrow=c(2,1))
    par(mar=c(0,0,0,0))
    par(oma=c(0,0,0,0))
    #frame(); legend("center",lty=1,lwd=20,col = hmcols,legend = c("< Neg ERCC median","< 50","< 100","< 500",">= 500"),cex=2)
    frame();
    text(x=0, y=1, labels = "Sample Annotations", cex=1, adj=0, font=2)
    legend(x=0, y=0.95, legend = c("True","False"), fill = c("darkorchid1","lightgoldenrod1"), cex = cex.legend, title = c("QC Flags"), title.adj=0, bty = "n", horiz = T)
    
    vertical.adj.2 <- 0
    for(i in rev(non.qc.annot)){
      legend(x=0, y=(0.95-((which(rev(non.qc.annot) == i) + vertical.adj.2)*vertical.adj)), 
             legend = names(heatmap.cols[["annotcols2"]][[i]]), 
             fill = as.character(heatmap.cols[["annotcols2"]][[i]]), cex = cex.legend, 
             title = names(heatmap.cols[["annotcols2"]][i]), title.adj=0, bty = "n", ncol=3) # horiz = T
      if(heatmap.cols[["annotcols3"]][[i]] > 1) vertical.adj.2 <- (heatmap.cols[["annotcols3"]][[i]]) - 1 - (vertical.adj*i)
    }
    
    frame();
    text(x=0, y=1, labels = "Probe Annotations", cex=1, adj=0, font=2)
    legend(x=0, y=0.95, cex=1, lty=1, lwd=12, col = hmcols, legend = c("< 25", "< 50", "< 100", "< 500", ">= 500"), title = "raw counts", title.adj=0, bty = "n")
    
    lgnd.ttl <- "flag/prune"
    
    if(any(c("SVN","Fusion") %in% colnames(rowcols))){
      lgnd.ttl <- paste0(lgnd.ttl,"/\n")
      
      if("SNV" %in% colnames(rowcols))
        lgnd.ttl <- paste0(lgnd.ttl,"snv/")
      
      if("Fusion" %in% colnames(rowcols))
        lgnd.ttl <- paste0(lgnd.ttl,"fusion")
    }
    legend(x=0, y=0.45, legend = c("yes", "no"), fill = c("blue", "gray50"), cex=1, title = lgnd.ttl, title.adj=0, bty = "n")
    dev.off()
  }
  
  # table of detected/not:
  # (first turn rawData.list into the matrix raw:)
  raw = c(); for(i in 1:length(rawDataAll.list)){raw = cbind(raw,rawDataAll.list[[i]])}
  #tab = cbind(colMeans(raw<neg.control.median),colMeans((raw>=neg.control.median)&(raw<50)),colMeans((raw>=50)&(raw<100)),colMeans((raw>=100)))
  #colnames(tab) = c("< Neg control median","Neg control median - 50","50 - 100","> 100")
  tab = cbind(colMeans(raw<25),colMeans((raw>=25)&(raw<50)),colMeans((raw>=50)&(raw<100)),colMeans((raw>=100)))
  colnames(tab) = c("< 25","25 - 50","50 - 100","> 100")
  tab = tab[order(tab[,1],decreasing=TRUE),]
  head(tab)
  write.csv(tab,file = paste0(path.to.QC.results,"//detected rates for all genes.csv"))
    
  ## now perform the rest of QC on the normalized data:
  alldata = cbind(normalized,HKnormalized)

  row.col.Match <- match(colnames(alldata), rownames(probe.annot))
  colnames(alldata) <- probe.annot$probeLabel.Type[row.col.Match]
  
  genecols = c(rep(codecols[111],dim(normalized)[2]))
  if(!is.null(HKnormalized))
    genecols <- c(genecols,rep(codecols[2],dim(HKnormalized)[2]))
  #genecols[is.element(dimnames(alldata)[[2]],HKs)]=codecols[3]
  genecols[which(rownames(probe.annot)[row.col.Match] %in% HKs)] <- codecols[3]
  
  # plot mean vs. sd in normalized data, color points by type: endog, HK (selected), HK (unselected)
  means = apply(alldata,2,mean,na.rm=T)
  vars = apply(alldata,2,var,na.rm=T)
  
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//mean vs var for all genes",sep=""),plottype)
    tempfilename=gsub(path.results,"results",tempfilename)
    par(xpd=T)
    par(mar=c(5,4,4,1))
    plot(vars ~ means, 
         col=genecols,
         pch = 1+15*probe.annot$ProbeID[match(colnames(alldata),probe.annot$probeLabel.Type)] %in% colnames(HKnormalized),
         xlab ="Mean(log2 expression)",
         ylab="Variance(log2 expression)")
    

    legend("topleft",
           bty="n",
           inset=c(0,-0.16),
           col= codecols[c(111,3,2)][codecols[c(111,3,2)] %in% genecols],
           pch = c(1,16,16)[codecols[c(111,3,2)] %in% genecols],
           legend=c("Endogenous genes","Housekeepers used in normalization","Housekeepers - unused")[codecols[c(111,3,2)] %in% genecols])
    
    topvars = order(vars,decreasing=TRUE)[1:20]
    text(means[topvars],vars[topvars],dimnames(alldata)[[2]][topvars], cex = 0.75)
    dev.off()
  }
  
  mean.var.normalized.df <- cbind(means,vars)
  dimnames(mean.var.normalized.df)[[2]] <- c("Average-normalized count", "Variance-normalized count")
  
  write.csv(mean.var.normalized.df, file = paste0(path.to.QC.results,"//QC module - Mean.Var.normalized.counts.csv", sep = ""), row.names = TRUE)
  
  #### plot relationships btw covariates:
  print("Creating Overview plots")
  cat("LOG:Creating Overview plots",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Creating Overview plots</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  for(i in 1:(dim(covariates)[2]))
  {
    # make a univariate histogram or barplot:
    for(r in 1:length(plottypearg)){
      plottype=plottypearg[r];
      tempfilename = drawplot(filename=paste(path.to.QC.results,"//univariate plot of covariates - ",dimnames(covariates)[[2]][i],sep=""),plottype)
      tempfilename=gsub(path.results,"results",tempfilename)
      if(covariates.type[i]=="continuous")
      {
        hist(covariates[,i],xlab="",main=paste("Distribution of",colnames(covariates)[i]))
      }
      if(covariates.type[i]=="categorical")
      {
        barplot(table(covariates[,i]),main=paste("Distribution of",colnames(covariates)[i]),las=2,border=annotcols2[[i]])
      }  
      dev.off()
    }
  }
  if(dim(covariates)[2]>1)
  {
    # plot a matrix of results.  Scatterplots/boxplots/tables on top right, p-values for association on bottom left:
    for(i in 1:(dim(covariates)[2]-1))
    {
      for(j in (i+1):dim(covariates)[2])
      {
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          #if(!(covariates.type[i]=="categorical")&(covariates.type[j]=="categorical"))
          #{
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//pairs plot of covariates - ",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],sep=""),plottype)
          tempfilename=gsub(path.results,"results",tempfilename)
          #}
          # scatterplot between continuous covariates:
          if((covariates.type[i]=="continuous")&(covariates.type[j]=="continuous"))
          {
            plot(covariates[,i],covariates[,j],
                 xlab=dimnames(covariates)[[2]][i],
                 ylab=dimnames(covariates)[[2]][j],
                 main = paste(dimnames(covariates)[[2]][i],"vs.",dimnames(covariates)[[2]][j]))
          }
          # boxplot between cont and cat:
          par(mar=c(5.1,6,4.1,2.1))
          if((covariates.type[i]=="continuous")&(covariates.type[j]=="categorical"))
          {
            par(mar=c(8,6,2,1))
            boxplot(covariates[,i]~covariates[,j],ylab=dimnames(covariates)[[2]][i],las=2,border=annotcols2[[j]],
                    main=paste(dimnames(covariates)[[2]][i],"vs.",dimnames(covariates)[[2]][j])) 
            
          }
          if((covariates.type[j]=="continuous")&(covariates.type[i]=="categorical"))
          {
            par(mar=c(8,6,2,1))
            boxplot(covariates[,j]~covariates[,i],ylab=dimnames(covariates)[[2]][j],las=2,border=annotcols2[[i]],
                    main=paste(dimnames(covariates)[[2]][j],"vs.",dimnames(covariates)[[2]][i]))  
          }
          # compare cat to cat:
          if((covariates.type[i]=="categorical")&(covariates.type[j]=="categorical"))
          {
            tab = as.matrix(table(covariates[,c(i,j)]))
            flip = dim(tab)[1]>dim(tab)[2]
            annotcodecol = i
            if(flip)
            {
              tab = t(tab)
              # color by i or j?
              annotcodecol = j
            }
            barplot(tab,col=annotcols2[[annotcodecol]],las=2,main=paste(dimnames(covariates)[[2]][j],"vs.",dimnames(covariates)[[2]][i]))  
            
            #write.csv(tab,file=paste(path.to.QC.results,"//pairs plot of covariates - ",dimnames(covariates)[[2]][i]," vs ",dimnames(covariates)[[2]][j],".csv",sep=""))
          }
          #if(!(covariates.type[i]=="categorical")&(covariates.type[j]=="categorical")){dev.off()}}          
          dev.off()}
      }
    }
  }
  write.csv(covariates,file="QC module - sample annot.csv")
  
  ### Implementation of multi.panel/multi.analyte heatmaps - 1/28/2016 - jbucci
  ### heatmap of all data, arranged by pathway membership  
  #hm1 = heatmap.2(t(normalized),scale="row")
  # first, fill in the row colors matrix:
  if(length(covariates)>0)
  {
    # rowcols = annotcols
    rowcols = heatmap.cols.matrix
    cat("dimnames(covariates):",file=log,sep='\n',append=TRUE)
    suppressWarnings(write.table(dimnames(covariates)[[2]],file=log,sep='\t',append=TRUE))
    suppressWarnings(write.table(dimnames(covariates)[[1]][1:5],file=log,sep='\t',append=TRUE))
    cat("dim(rowcols)",file=log,sep='\n',append=TRUE)
    cat(dim(rowcols),file=log,sep='\n',append=TRUE)
    # dimnames(rowcols)=dimnames(covariates)
    # dimnames(rowcols)[[1]]=dimnames(covariates)[[1]][1:25]
    # dimnames(rowcols)[[2]]=dimnames(covariates)[[2]]
    #if(i==1){rowcols = cbind(rowcols,"white")}
    #if(i==1){rowcols = cbind(rowcols,rowcols)}
  }
  #  if(length(prb.sets)>0)
  #  {
  #    bigmatrix = t(normalized)
  #    pathsizes = dim(bigmatrix)[1]
  #  }
  if(length(prb.sets)>0)
  {
    #    bigmatrix = c()
    #    pathsizes = c()
    #    pathbreaks = c()
    #    pathcolors = c()
    prb.sets = prb.sets[match(dimnames(normalized)[[2]],dimnames(prb.sets)[[1]]),]
    for(k in 1:dim(prb.sets)[2])
    {
      print(paste("Creating Overview heatmaps for gene set ",k,"/",dim(prb.sets)[2]))
      cat(paste("LOG: Creating Overview heatmaps for gene set ",k,"/",dim(prb.sets)[2]),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating Overview heatmaps for gene set ",k,"/",dim(prb.sets)[2],"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      
      # isolate data matrix from the current gene set:
      #pathmatrix = t(normalized[,dimnames(prb.sets)[[1]][prb.sets[,k]==1]])
      
      pathmatrix.list <- list()
      for(l in 1:length(normalizedData.list)){
        #(colnames(normalizedData.list[[1]]) %in% dimnames(prb.sets)[[1]][prb.sets[,k]==1])
        pathmatrix.sub           <- normalizedData.list[[l]][,colnames(normalizedData.list[[l]]) %in% dimnames(prb.sets)[[1]][prb.sets[,k]==1], drop = FALSE]
        row.col.Match            <- match(colnames(pathmatrix.sub), rownames(probe.annot))
        colnames(pathmatrix.sub) <- probe.annot$probeLabel.Type[row.col.Match]
        
        if(dim(pathmatrix.sub)[l] == 0){
          next
        } else {
          pathmatrix.list[[l]]      <- t(pathmatrix.sub)
          names(pathmatrix.list)[l] <- names(normalizedData.list)[l]
        }
      }
      
      # run a heatmap on the pathway matrix to get a better ordering of the genes
      if(sum(unlist(lapply(pathmatrix.list,function(x)nrow(x))))>1){
        
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of data - ",make.names(dimnames(prb.sets)[[2]][k]),sep=""),plottype,
                                  #width=(1.5+1*(dim(normalized)[1]>50)),height=1.75,
                                  width = 2, height = 2, heatmapres=TRUE)
          
          #width=(3+2*(dim(normalized)[1]>50)),height=3.5)
          tempfilename=gsub(path.results,"results",tempfilename)
          par(oma=c(1,8,1,1)) 
          #par(xpd=T)
          par(mar=c(1,2,2,1))
          hmcols<-colorRampPalette(c("cornflowerblue","black","orange"), bias=1.0)(256)
          #hmcols<-colorRampPalette(c("dodgerblue2","grey","orange"), bias=1.0)(256)
          
          rowColor.list <- list()
          for(analyte.i in names(pathmatrix.list)){
            
            path.gene.names = rownames(pathmatrix.list[[analyte.i]])
            rowColor.list[[analyte.i]] = rowColor.list0[[analyte.i]][rownames(rowColor.list0[[analyte.i]])%in%path.gene.names,,drop=F]
          }
          
          for(analyte.i in names(pathmatrix.list))
          {
            rownames(rowColor.list[[analyte.i]])     <- rownames(pathmatrix.list[[analyte.i]])
            colnames(pathmatrix.list[[analyte.i]])   <- strtrim(abbreviate(toupper(colnames(pathmatrix.list[[analyte.i]])),minlength = 30), width = 30) #colnames(pathmatrix.list[[i]])
          }
          
          heatmap.plus.multipanel(Xlist = pathmatrix.list, rowcolorlist = rowColor.list, ColSideColors = as.matrix(rowcols), scale = "row", margins=c(12,12), col=hmcols, cexCol = 0.85, cexRow = 0.75) 
          #par(xpd=T)
          #legend(x=0.85, y=1.0, legend = c("True","False"), fill = c("darkorchid1","lightgoldenrod1"), cex=0.75, title = "QC Flags", bty = "n")
          #non.qc.annot <- which(grepl(pattern = "\\.Flag", names(heatmap.cols[["annotcols2"]])) == F)
          #for(i in rev(non.qc.annot)){
          #  legend(x=0.85, y=(1.0-(which(rev(non.qc.annot) == i)*0.075)), legend = names(heatmap.cols[["annotcols2"]][[i]]), fill = as.character(heatmap.cols[["annotcols2"]][[i]]), cex=0.75, title = names(heatmap.cols[["annotcols2"]][i]), bty = "n")
          #}
          #legend(x=0.9,y=1, legend=c("yes","no"), fill=c("blue","gray50"),cex=1,title="",bty="n")
          # for (i in (ncol(covariates)+1):length(heatmap.cols$annotcols2)) {
          #   annot.name = names(heatmap.cols$annotcols2)[i]
          #   annot.ypos = 0.85 - (i-ncol(covariates)-1)/(length(heatmap.cols$annotcols2)-ncol(covariates)-1)*0.5
          #   legend(x=0.85,y=annot.ypos, legend=names(heatmap.cols$annotcols2[[annot.name]]), fill=heatmap.cols$annotcols2[[annot.name]],cex=0.7,title=annot.name,bty="n")
          # }
          dev.off()
          par(xpd=F)
          
        }
        
      }
      
      
      #      #### correlation matrix of all samples (just the pathway's data):  
      #      cormat = cor((pathmatrix))
      #      #neg.elements = sign(cormat)==-1
      #      #cormat.rescaled = cormat/abs(min(cormat))
      #      #cormat2 = replace(cormat,neg.elements,cormat.rescaled[neg.elements])
      #      for(r in 1:length(plottypearg)){
      #        plottype=plottypearg[r];
      #        tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap - ",make.names(dimnames(prb.sets)[[2]][k]),sep=""),plottype,width=2,height=2,heatmapres=TRUE)
      #        tempfilename=gsub(path.results,"results",tempfilename)
      #        par(mar=c(10,4,2,1))
      #        hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
      #        hmcols = hmcols
      #        breaks = seq(-1,max(abs(cormat)),length.out=length(hmcols)+1)
      #        #if(length(covariates)>0){ heatmap.plus.modified(cormat,symm=TRUE,margins=c(8,8),ColSideColors=as.matrix(rowcols),RowSideColors=as.matrix(rowcols)) }
      #        #if(length(covariates)>0){ heatmap.3(cormat,symm=TRUE,margins=c(8,8),RowSideColors=t(as.matrix(rowcols)),ColSideColors=as.matrix(rowcols),col=hmcols,Colv=TRUE) }
      #        if(length(covariates)>0){ heatmap.plus.modified(cormat,symm=TRUE,margins=c(8,8),RowSideColors=as.matrix(rowcols),ColSideColors=as.matrix(rowcols),col=hmcols,Rowv=TRUE,Colv=TRUE) }
      #        if(length(covariates)==0){ heatmap.plus.modified(cormat,symm=TRUE,margins=c(8,8),col=hmcols,Rowv=TRUE,Colv=TRUE) }
      #        dev.off()
      #      }
      
      
      pathmatrix = t(normalized[,dimnames(prb.sets)[[1]][prb.sets[,k]==1]])
      row.Match            <- match(rownames(pathmatrix), rownames(probe.annot))
      rownames(pathmatrix) <- probe.annot$probeLabel.Type[row.Match]
      
      if(nrow(pathmatrix)>1){
        # and draw a PCA plot (just the pathway data):
        pc = prcomp(t(pathmatrix))
        for(i in 1:dim(covariates)[2])
        {
          pointcols = annotcols[,i]
          for(r in 1:length(plottypearg)){
            plottype=plottypearg[r];
            tempfilename = drawplot(filename=paste(path.to.QC.results,"//PCA colored by ",dimnames(covariates)[[2]][i]," - ",make.names(dimnames(prb.sets)[[2]][k]),sep=""),plottype)
            tempfilename=gsub(path.results,"results",tempfilename)
            
            pch = 1+15*(nrow(pc$x)<100)
            pairs.adaptive.cex(dat = pc$x[,1:min(4,dim(pc$x)[2]),drop=FALSE],
                               col=pointcols,
                               xaxt="n",
                               yaxt="n",
                               pch=pch,
                               labels = paste("PC",1:4," - ",round((pc$sdev^2)[1:4]/sum(pc$sdev^2),2),sep=""))
            dev.off()
          }
          
        }
        
        # and draw the PC loadings:  #$#
        for(r in 1:length(plottypearg)){
          plottype=plottypearg[r];
          tempfilename = drawplot(filename=paste(path.to.QC.results,"//PCA loadings - ",make.names(dimnames(prb.sets)[[2]][k]),sep=""),plottype,height=2)
          tempfilename=gsub(path.results,"results",tempfilename)
          showpcs = min(4,ncol(pc$rotation))
          #par(mfrow = c(showpcs,1))        
          layout(matrix(c(1:showpcs)),heights = c(rep(1,showpcs-1),.8)) 
          par(mar = c(2,4,0,0))
          for(j in 1:showpcs)
          {
            if(j<showpcs){barplot(pc$rotation[,j],las=2,ylab=paste0("PC",j," loading"),names.arg="")}
            if(j==showpcs){par(mar = c(6,4,0,0));barplot(pc$rotation[,j],las=2,ylab=paste0("PC",j," loading"))}
          }
          dev.off()
        }
      }
    }
  }
  
  # plot the final heatmap:
  
  t.normalizedData.list <- list()
  for(l in 1:length(normalizedData.list)){
    normalizedData.sub           <- normalizedData.list[[l]]
    row.col.Match                <- match(colnames(normalizedData.sub), rownames(probe.annot))
    colnames(normalizedData.sub) <- probe.annot$probeLabel.Type[row.col.Match]
    
    if(dim(normalizedData.sub)[l] == 0){
      next
    } else {
      t.normalizedData.list[[l]]      <- t(normalizedData.sub)
      names(t.normalizedData.list)[l] <- names(normalizedData.list)[l]
    }
  }
  
  ################################################ BEGIN NEW ALEX STUFF
  # *for use with the 'interactiveHeatmap' API method
  # Converts the given list of data frames into a single data frame with rows and columns in the
  # order they appear in a heatmap.plus.multipanel png image
  # NOTE: most of this code is copied/modified from the heatmap.plus.multipanel function. The main
  # difference is that the data frame produced by this function does not contain rows of -Inf
  # values. The purpose of this function is to rearrange the data matrix into exactly the order that
  # it should appear in the interactive heatmap, with columns reading left to right and rows reading
  # top to bottom
  getClusteredData = function(Xlist) {
    # step 0: pre-processing:
    # if only a matrix is entered, turn it into a list of length 1. 
    if(is.matrix(Xlist)){Xlist = list(Xlist)}
    
    #drop any element which has any dimension zero
    Xlist[sapply(Xlist,function(x)any(dim(x)==0))] <- NULL
    
    # create a single matrix from the column-aligned component matrices:
    bigmat = c()
    for(i in 1:length(Xlist)) {
      tmp.d <- Xlist[[i]]
      if(!is.null(names(attr(tmp.d,"colname.abr.map"))))
        colnames(tmp.d) <- names(attr(tmp.d,"colname.abr.map"))[match(colnames(tmp.d), attr(tmp.d,"colname.abr.map"))]
      bigmat = rbind(bigmat,tmp.d)
    }
    
    #drop all na columns: note apply fails if there is only 1 row! assumption is we have more than that always
    bigmat <- bigmat[,apply(bigmat,2,function(x) !all(is.na(x)))]
    
    # step 1: get the dendogram of each matrix in the list, and combine into a reordering of the big matrix
    reorder.row = c(); 
    for(i in 1:length(Xlist)) {
      tempmat = Xlist[[i]]
      if(nrow(Xlist[[i]])>1){tempclust = hclust(dist(tempmat))$order}
      if(nrow(Xlist[[i]])==1){tempclust = 1}
      reorder.row = c(reorder.row,tempclust+length(reorder.row))
    }
    # step 2: get the dendrogram of the combined data:
    reorder.column = hclust(dist(t(bigmat)))$order
    # matrix to enter into heatmap.plus:
    x = bigmat[reorder.row,reorder.column]
    return(x)
  }
  
  # Here, we write out the CSV files necessary to generate an interactive heatmap for the "Heatmap of All Data",
  # the heatmap which sits in the Overview/Summary/Heatmap panel.

  # raw heatmap data
  iRawHeatmapData = getClusteredData(t.rawData.list)
  write.csv(iRawHeatmapData, file = paste0(path.to.QC.results, "/heatmap of all genes X samples rawData.csv"))

  # heatmap data
  iHeatmapData = getClusteredData(t.normalizedData.list)
  write.csv(iHeatmapData, file = paste0(path.to.QC.results, "/heatmap of all genes X samples normalizedData.csv"))
  
  # col annotations
  covariates.copy = covariates[colnames(iHeatmapData),]
  # rownames(covariates.copy) = colnames(iHeatmapData)
  write.csv(covariates.copy, file = paste0(path.to.QC.results, "/heatmap of all genes X samples covariates.csv"))
  
  # row annotations
  probe.annot.copy = probe.annot
  probe.annot.copy$below.threshold <- NA
  #probe.annot.copy[names(filtered$prune),"below.threshold"] <- c("gray50","blue")[1+filtered$prune] 
  probe.annot.copy[names(filtered$prune),"below.threshold"] <- c("False","True")[1+filtered$prune] 
  rownames(probe.annot.copy) = probe.annot$probeLabel.Type
  probe.annot.copy = probe.annot.copy[rownames(iHeatmapData),]
  write.csv(probe.annot.copy, file = paste0(path.to.QC.results, "/heatmap of all genes X samples probe.annot.csv"))
  ################################################ END NEW ALEX STUFF
  
  #dimnames(pathcolors)[[2]]=c("","")
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of all genes X samples",sep=""), plottype,height=2,width=2,heatmapres=TRUE)
    #tempfilename = drawplot(filename=paste(path.to.QC.results,"//heatmap of all genes X samples",sep=""),plottype,height=4,width=3)
    tempfilename=gsub(path.results,"results",tempfilename)
    
    #lgnd.ttl <- "flag/prune"
    #
    #if(any(c("SVN","Fusion") %in% colnames(rowcols))){
    #  lgnd.ttl <- paste0(lgnd.ttl,"/\n")
    #  
    #  if("SNV" %in% colnames(rowcols))
    #    lgnd.ttl <- paste0(lgnd.ttl,"snv/")
    #  
    #  if("Fusion" %in% colnames(rowcols))
    #    lgnd.ttl <- paste0(lgnd.ttl,"fusion")
    #}
    
    par(oma=c(1,8,1,1)) 
    par(xpd=T)
    par(mar=c(1,2,2,1))
    hmcols <- colorRampPalette(c("cornflowerblue", "black", "orange"), bias=1.0)(256)
    #hmcols <- colorRampPalette(c("dodgerblue2", "grey", "orange"), bias=1.0)(256)
    
    rowColor.list <- list()
    for(analyte.i in names(t.normalizedData.list)){
      g = analyte.i
      # change this as well
      # rowColor.list[[g]] <-matrix(rep(c(gray.colors(5)[g]), dim(t.normalizedData.list[[g]])[1]))
      all.gene.names = rownames(t.normalizedData.list[[g]])
      rowColor.list[[g]] = rowColor.list0[[g]][rownames(rowColor.list0[[g]])%in%all.gene.names,,drop=F]
    }
    
    for(analyte.i in names(t.normalizedData.list))
    {
      rownames(rowColor.list[[analyte.i]])           <- rownames(t.normalizedData.list[[analyte.i]])
      colnames(t.normalizedData.list[[analyte.i]])   <- strtrim(abbreviate(toupper(colnames(t.normalizedData.list[[analyte.i]])),minlength = 30), width = 30) #colnames(t.normalizedData.list[[i]])
    }
    
    heatmap.plus.multipanel(Xlist = t.normalizedData.list, rowcolorlist = rowColor.list, ColSideColors = as.matrix(rowcols), scale = "row", margins=c(12,6), col=hmcols, cexCol = 0.85, cexRow = 0.75)
    # heatmap.plus.multipanel(Xlist = t.normalizedData.list, rowcolorlist = rowColor.list, ColSideColors = as.matrix(rowcols), margins=c(12,12), scale="n", breaks=breaks,col=hmcols, cexCol = 0.85, cexRow = 0.75)
    # if(length(covariates)>=0){ heatmap.plus.multipanel(Xlist = t.normalizedData.list, rowcolorlist = rowColor.list, ColSideColors = as.matrix(rowcols), margins=c(12,12), col=hmcols, cexCol = 0.85, cexRow = 0.75) }
    # if(length(covariates)>0){ heatmap.plus.modified(t(normalized),symm=FALSE,Colv=TRUE,Rowv=TRUE,labRow=NA,margins=c(8,8),ColSideColors=as.matrix(rowcols),col=hmcols,scale="row") }
    # commented out
    # if(length(covariates)==0){ heatmap.plus.modified(t(normalized),symm=FALSE,Colv=TRUE,Rowv=TRUE,labRow=NA,margins=c(8,8),col=hmcols,scale="row") }
    par(xpd=T)
    #legend(x=0.85,y=1, legend=c("yes","no"), fill=c("blue","gray50"),cex=1,title=lgnd.ttl,bty="n")
    # for (i in (ncol(covariates)+1):length(heatmap.cols$annotcols2)) {
    #   annot.name = names(heatmap.cols$annotcols2)[i]
    #   annot.ypos = 0.85 - (i-ncol(covariates)-1)/(length(heatmap.cols$annotcols2)-ncol(covariates)-1)*0.5
    #   legend(x=0.85,y=annot.ypos, legend=names(heatmap.cols$annotcols2[[annot.name]]), fill=heatmap.cols$annotcols2[[annot.name]],cex=0.7,title=annot.name,bty="n")
    # }
    dev.off()
    par(xpd=F)
  }
  
  # new legend
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    tempfilename = drawplot(filename=paste(path.to.QC.results,"//normalized data heatmap - color key",sep=""),plottype,width=0.5,height=1.25)
    tempfilename=gsub(path.results,"results",tempfilename)
    
    if(legend.total.lines <= 6){
      cex.legend   <- 1.0
      vertical.adj <- 0.175
    } else if(legend.total.lines == 7){
      cex.legend <- 0.95
      vertical.adj <- 0.175
    } else if(legend.total.lines >= 8 & legend.total.lines <= 12){
      cex.legend <- 0.90
      vertical.adj <- 0.15
    } else if(legend.total.lines >= 13 & legend.total.lines <= 17){
      cex.legend <- 0.85
      vertical.adj <- 0.135
    } else if(legend.total.lines >= 18){
      cex.legend <- 0.75
      vertical.adj <- 0.115
    }
    
    par(mfrow=c(2,1))
    par(mar=c(0,0,0,0))
    par(oma=c(0,0,0,0))
    #frame(); legend("center",lty=1,lwd=20,col = hmcols,legend = c("< Neg ERCC median","< 50","< 100","< 500",">= 500"),cex=2)
    frame();
    text(x=0, y=1, labels = "Sample Annotations", cex = 1, adj = 0, font = 2)
    legend(x=0, y=0.95, legend = c("True","False"), fill = c("darkorchid1","lightgoldenrod1"), cex=cex.legend, title = c("QC Flags"), title.adj = 0, bty = "n", horiz = T)
    
    vertical.adj.2 <- 0
    for(i in rev(non.qc.annot)){
      legend(x=0, y=(0.95-((which(rev(non.qc.annot) == i) + vertical.adj.2)*vertical.adj)), 
             legend = names(heatmap.cols[["annotcols2"]][[i]]), 
             fill = as.character(heatmap.cols[["annotcols2"]][[i]]), cex = cex.legend, 
             title = names(heatmap.cols[["annotcols2"]][i]), title.adj=0, bty = "n", ncol=3) # horiz = T
      if(heatmap.cols[["annotcols3"]][[i]] > 1) vertical.adj.2 <- (heatmap.cols[["annotcols3"]][[i]]) - 1 - (vertical.adj*i)
    }
    
    frame();
    text(x=0, y=1, labels = "Probe Annotations", cex = 1, adj = 0, font = 2)
    #legend(x=0, y=0.95, lty=1, lwd=12, col = hmcols, legend = c("< 25", "< 50", "< 100", "< 500", ">= 500"), title = "raw counts", title.adj = 0, bty="n")
    hmcols <- colorRampPalette(c("cornflowerblue", "black", "orange"), bias=1.0)(256)
    #hmcols <- colorRampPalette(c("dodgerblue2", "grey", "orange"), bias=1.0)(256)
    legend.image <- as.raster(matrix(hmcols, nrow=1))
    rasterImage(legend.image, xleft = 0, ybottom = 0.70, xright = 0.25, ytop = 0.95)
    text(x = seq(0, 0.25, length.out = 7), y = 0.675, labels = seq(-3, 3, by = 1), cex = 0.9)
    text(x = 0.125, y = 0.625, labels = "z-scores", cex = 0.9)
    
    lgnd.ttl <- "flag/prune"
    
    if(any(c("SVN","Fusion") %in% colnames(rowcols))){
      lgnd.ttl <- paste0(lgnd.ttl,"/\n")
      
      if("SNV" %in% colnames(rowcols))
        lgnd.ttl <- paste0(lgnd.ttl,"snv/")
      
      if("Fusion" %in% colnames(rowcols))
        lgnd.ttl <- paste0(lgnd.ttl,"fusion")
    }
    legend(x=0, y=0.50, legend = c("yes", "no"), fill = c("blue", "gray50"), cex=1, title = lgnd.ttl, title.adj = 0, bty = "n")
    dev.off()
  }
  
  # Commenting out the sample correlation 
  # plot as it is not being used any more
  #--------------------------------------
  #### correlation matrix of all samples:  
  #cormat = cor(t(normalized))
  
  # corNorm.list <- list()
  # for(l in 1:length(normalizedData.list)){
  #   normalizedData.sub           <- normalizedData.list[[l]]
  #   row.col.Match                <- match(colnames(normalizedData.sub), rownames(probe.annot))
  #   colnames(normalizedData.sub) <- probe.annot$probeLabel.Type[row.col.Match]
  #     
  #   if(ncol(normalizedData.sub) <3){
  #     next
  #   } else {
  #       corNorm.list[[l]]      <- cor(t(normalizedData.sub))
  #       names(corNorm.list)[l] <- names(normalizedData.list)[l]
  #   }
  # }
  # for(r in 1:length(plottypearg)){
  #   plottype=plottypearg[r];
  #   tempfilename = drawplot(filename=paste(path.to.QC.results,"//correlation between samples - heatmap",sep=""),plottype,width=2,height=2,heatmapres=TRUE)
  #   tempfilename=gsub(path.results,"results",tempfilename)
  #   par(oma=c(1,4,1,1))  
  #   par(mar=c(10,8,2,10))
  #   hmcols<-colorRampPalette(c("blue","antiquewhite3",codecols2[4]))(256)
  #   breaks = seq(-1,1,length.out=length(hmcols)+1)
  # 
  #   rowColor.list <- list()
  #   for(g in 1:length(corNorm.list)){
  #     if(!is.null(corNorm.list[[g]]))
  #       rowColor.list[[g]] <-matrix(rep(c(gray.colors(5)[g]), dim(corNorm.list[[g]])[1]))
  #   }
  # 
  #   for(i in 1:length(corNorm.list))
  #   {
  #     if(!is.null(corNorm.list[[i]])){
  #       rownames(rowColor.list[[i]])  <- strtrim(abbreviate(toupper(rownames(corNorm.list[[i]]))), width = 30) #rownames(corNorm.list[[i]])
  #       rownames(corNorm.list[[i]])   <- strtrim(abbreviate(toupper(rownames(corNorm.list[[i]]))), width = 30) #colnames(corNorm.list[[i]])
  #       colnames(corNorm.list[[i]])   <- strtrim(abbreviate(toupper(colnames(corNorm.list[[i]]))), width = 30) #colnames(corNorm.list[[i]])
  #     }
  #   }
  #   
  #   #Prune empty elments
  #   #-------------------
  #   corNorm.list <- corNorm.list[which(unlist(lapply(corNorm.list,function(x)!is.null(x) )))]
  #   rowColor.list <- rowColor.list[which(unlist(lapply(rowColor.list,function(x)!is.null(x) )))]
  # 
  #   if(length(covariates)>0){ heatmap.plus.multipanel(Xlist = corNorm.list, rowcolorlist = rowColor.list, ColSideColors = as.matrix(rowcols), margins=c(12,12), col=hmcols, cexCol = 0.85, cexRow = 0.85) }
  #   if(length(covariates)==0){ heatmap.plus.modified(cormat,symm=TRUE,margins=c(8,8),col=hmcols,Colv=TRUE) }
  #   dev.off()
  # }
  
  
  ### PCA-based QC:
  pc = prcomp(normalized[complete.cases(normalized),])
  ## automatically identify outliers in PCA:
  x = pc$x[,1:min(4,dim(pc$x)[2])]
  
  # outliers: anything > 3sds from mean:
  outliers = matrix(FALSE,dim(x)[1],dim(x)[2])
  rownames(outliers) <- rownames(normalized[complete.cases(normalized),])
  colnames(outliers) = paste("Outlier on PC",1:ncol(x))
  for(i in 1:dim(x)[2])
  {
    outliers[abs(x[,i]-mean(x[,i]))>3*sd(x[,i]),i]=TRUE   
  }
  # output list of outliers along with PC on which they're an outlier:
  if(sum(rowSums(outliers)>0)>1){outliers = outliers[rowSums(outliers)>0,]}
  if(sum(rowSums(outliers)>0)==1)
  {
    outliername = dimnames(outliers)[[1]][rowSums(outliers)>0] 
    outliers = t(as.matrix(outliers[rowSums(outliers)>0,]))
    dimnames(outliers)[[1]]=outliername
  }
  if(sum(rowSums(outliers)>0)>0){write.csv(outliers,file=paste(path.to.QC.results,"//outliers in first 4 principal components.csv",sep=""))}
  
  ## PC pairs plots colored by covariates:   #<---- modification: now using annotcols to harmonize colors.  if(FALSE) after this code chunk was the old coloring scheme
  #covariates = as.data.frame(covariates)
  if(dim(covariates)[2]>0)
  {
    for(i in 1:dim(covariates)[2])
    {
      pointcols = annotcols[,i]
      for(r in 1:length(plottypearg)){
        plottype=plottypearg[r];
        tempfilename = drawplot(filename=paste(path.to.QC.results,"//PCA colored by ",dimnames(covariates)[[2]][i],sep=""),plottype)
        tempfilename=gsub(path.results,"results",tempfilename)
        pch = 1+15*(nrow(pc$x)<100)
        pairs.adaptive.cex(pc$x[,1:min(4,dim(pc$x)[2])],col=pointcols,xaxt="n",yaxt="n",pch=pch,
                           labels = paste("PC",1:4," - ",round((pc$sdev^2)[1:4]/sum(pc$sdev^2),2),sep=""))
        dev.off()}
      #for(r in 1:length(plottypearg)){
      #  plottype=plottypearg[r];
      #  tempfilename = drawplot(filename=paste(path.to.QC.results,"//PCA colored by ",dimnames(covariates)[[2]][i]," legend",sep=""),plottype,width=.5)
      #  tempfilename=gsub(path.results,"results",tempfilename)
      #  frame()
      #  legend("center",pch=1,col=c(NA,"deepskyblue2","firebrick2"),legend=c(main=names(covariates)[i],"low","high"))
      #  dev.off()}      
    }
    write.csv(covariates,file=paste(path.to.csvs,"//QC - covariates used in p-value histograms.csv",sep=""))
    
    ### p-value histograms of univariate analyses of ALL variables:
    for(r in 1:length(plottypearg)){
      print(paste("Creating Overview histograms ",r,"/",length(plottypearg)))
      cat(paste("LOG: Creating Overview histograms ",r,"/",length(plottypearg)),file=log,sep='\n\n',append=TRUE)
      cat(paste("document.write('<p>Creating Overview histograms ",r,"/",length(plottypearg),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
      plottype=plottypearg[r];
      
      if(ncol(covariates) > 2)
      {
        nrows = ceiling(ncol(covariates)/2)
        ncols = ceiling(ncol(covariates)/2)
      } else {
        nrows = 2
        ncols = 1
      }
      
      tempfilename = drawplot(filename=paste(path.to.QC.results,"//p-value histograms",sep=""),plottype,height=nrows*.8,width=ncols*.8)
      tempfilename=gsub(path.results,"results",tempfilename)    
      #if(dim(covariates)[[2]]<=4)
      #{
      #  par(mfrow=c(ceiling(dim(covariates)[[2]]/2),min(dim(covariates)[[2]],2)))
      #}
      #if(dim(covariates)[[2]]>4)
      #{
      #  par(mfrow=c(ceiling(dim(covariates)[[2]]/3),min(dim(covariates)[[2]],3)))
      #}
      par(mfrow=c(nrows,ncols), las = 1)
      for(i in 1:dim(covariates)[2])
      {
        pvals = c()
        for(j in 1:dim(normalized)[2])
        {
          mod = lm(cbind(normalized[,j],as.data.frame(covariates[,i])))      
          mod2 = lm(normalized[,j]~1) 
          # run LRT:
          LLR = logLik(mod)-logLik(mod2)
          df = length(mod$coef)-length(mod2$coef)
          pvals[j] = 1-pchisq(2*LLR,df)
        }
        hist(pvals,main=dimnames(covariates)[[2]][i],breaks=(0:20)/20,xlim=c(0,1),cex.main=1.25,cex.lab=1.25,xlab="p-values")
      }
      dev.off()
    }
  }
  
  print("Creating Overview HTML infrastructure")
  cat("LOG: Creating Overview HTML infrastructure",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Creating Overview HTML infrastructure</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  print("Finished Overview")
  cat("LOG:Finished Overview",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Finished Overview</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
} 
