
# NOTE: path.results is referenced from global environment! path.results is only used for generating legend. I consider
#       this to be temporary so didn't fix the global env issue. Ultimately, I plan to generate and pass back legend graphics
#       to main and leave it to main to right to the path.results.

# this is a wrapper to mimic "load.and.clean.data" function, which takes the data and metadata input
# and put them in the proper form (if it can or else generates error). The main three tasks are
# 1- construct.data: loads count (raw or processed) data and construct the data object (see construct.data for details)
# 2- construct.sample.annotation: loads sample annotations and construct the smp.annot object (see. construct.sample annotation for details)
# 3- construct.probe.annotation: loads probe annotations and construct the prb.annot object (see construct.probe.annotation for details)

construct.data.and.annotations <- function(path.data,
                                           path.sampleannot,
                                           path.probeannot,
                                           snvarg.ref.ID,
                                           prb.annotation.id.column,             # <------------ This may become static as opposed to a func input "UniqueID"
                                           prb.annotation.KEGG.id.column = NULL, # <------------ This may become static as opposed to a func input "KEGG.Pathways"
                                           prb.annotation.KEGG.Gene.Name = NULL, # <------------ This may become static as opposed to a func input "Gene.Name"
                                           prb.annotation.celltype.id.column = NULL, # <------------ This may become static as opposed to a func input "Cell.Type"
                                           prb.set.column = NULL,
                                           min.prb.set.size,
                                           min.KEGG.set.size,
                                           sample.annotation.id.column,   
                                           sampleannot.variables,
                                           sampleannot.variabletypes,
                                           sampleannot.referencelevels,
                                           exclude.probes = NULL,    #<-------------- This could be ERCC by default
                                           exclude.samples = NULL,   #<-------------- This could be NULL by default
                                           draw.color.legend,
                                           min.samples,
                                           plottypearg,
                                           log){
  
  # Initiate
  #---------
  warnings.paragraph = character(0)
  
  
  
  # Load and construct data
  #------------------------
  cat("loading data",file=log,sep='\n\n',append=TRUE)
  
  
  # Validate the probe annotation entry
  #------------------------------------
  construct.default.prb.annot <- is.null(path.probeannot)
  
  if(!construct.default.prb.annot){
    cat("loading probe annotation files",file=log,sep='\n\n',append=TRUE)
    
    tmp <- tryCatch(expr = {validate.pannot.entry(prb.annot = read.csv(path.probeannot,check.names = F,stringsAsFactors = F),
                                                  prb.annotation.id.column)}
                    , error = function(er){
                      ertmp <- paste("Provided probe annotations did not pass requirement check:\n",er)
                      cat(ertmp)
                      class(ertmp) <- "probe.annotation.check.error"
                      return(ertmp)
                    }
                    , finally ={})
    
    if(class(tmp) == "probe.annotation.check.error")
      construct.default.prb.annot <- TRUE
  }
  
  # if no probe annotation path is provided or if the provided annotation failed to meet the requirement
  # default annotation is inferred
  
  if(construct.default.prb.annot){
    cat("constructing default probe annotation files",file=log,sep='\n\n',append=TRUE)
    tmp <- list(prb.annot = get.default.pannot())
  }
  
  validated.prb.annot <- tmp
  
  #   # 170628: if multiple analyte types, include analyte types as additional geneset category
  # if(length(unique(validated.prb.annot$Analyte.Type)) > 1){
  #   for(i in 1:dim(validated.prb.annot)[1]){
  #     if(validated.prb.annot$Probe.Annotation[i] == ""){
  #       validated.prb.annot$Probe.Annotation[i] <- validated.prb.annot$Analyte.Type[i]
  #     } else{
  #       validated.prb.annot$Probe.Annotation[i] <- paste(validated.prb.annot$Analyte.Type[i], validated.prb.annot$Probe.Annotation[i], sep = ";")
  #     }
  #   }
  # }
  
  # warnings.paragraph <- paste(warnings.paragraph,tmp$warnings.paragraph)
  
  
  
  
  # Load up all existing data types (i.e. raw, normalized, background subtracted)
  # Assumption: raw file always exists. It errors out if rawData.csv doesn't exist.
  # Also if other files don't match raw file dimnames they are disregarded and NULL
  # is returend for the dimnmanes-mismatched files.
  #------------------------------------------------------------------------------
  raw <- processed <- backgroundSubtracted <- smp.names <- prb.names <- NULL
  smp.names <- prb.names <- character(0)
  validate.annot.dset <- F
  
  #RAW
  if(isTRUE(file.exists(path.data[grep("RawData",path.data)]))){
    colnum <- ncol(read.csv(path.data[grep("RawData",path.data)]))
    raw <- raw.tmp <- read.csv(path.data[grep("RawData",path.data)],row.names=1,colClasses = c("character",rep("numeric",colnum-1)),check.names = F)
    raw <- construct.data(dat = raw,
                          min.samples = min.samples,
                          dat.type = "raw",
                          snvarg.ref.ID = snvarg.ref.ID,
                          validated.prb.annot = validated.prb.annot,
                          exclude.samples = exclude.samples,
                          exclude.probes = exclude.probes)
    
    warnings.paragraph <- paste(warnings.paragraph,raw$warnings.paragraph)
    raw <- raw$dat
    
    if(!is.null(raw)){
      if(nrow(raw)>0)
        smp.names <- rownames(raw)
      if(ncol(raw)>0){
        prb.names <- colnames(raw)
        validate.annot.dset <- TRUE
      }
      
        
      prb.names.all <- prb.names
      smp.names.all <- smp.names
      if(length(attr(raw,"dat.subsets"))>0){
        attributes(prb.names) <- lapply(attributes(raw)[attr(raw,"dat.subsets")],colnames)
        attributes(smp.names) <- lapply(attributes(raw)[attr(raw,"dat.subsets")],rownames)
        attributes(validate.annot.dset) <- sapply(attributes(prb.names), function(x) length(x) > 0, simplify = F)
        prb.names.all <- colnames(raw.tmp)[colnames(raw.tmp) %in% unique(c(prb.names,unlist(attributes(prb.names))))]
        smp.names.all <- rownames(raw.tmp)[rownames(raw.tmp) %in% unique(c(smp.names,unlist(attributes(smp.names))))]
      }
      
    }
  }else{
    stop("rawData is not provided")
  }
  
  #Background Subtracted: This is the data with background thresholded to zero
  if(isTRUE(file.exists(path.data[grep("BackgroundSubtractedValues",path.data)]))){
    colnum <- ncol(read.csv(path.data[grep("BackgroundSubtractedValues",path.data)]))
    backgroundSubtracted.tmp <- read.csv(path.data[grep("BackgroundSubtractedValues",path.data)],row.names=1,colClasses = c("character",rep("numeric",colnum-1)),check.names = F)
    
    if(all(smp.names.all %in% rownames(backgroundSubtracted.tmp)) & all(prb.names.all %in% colnames(backgroundSubtracted.tmp)) & length(prb.names.all)>0 & length(smp.names.all)>0){
      
      backgroundSubtracted.tmp <- as.matrix(backgroundSubtracted.tmp[smp.names.all ,prb.names.all])
      backgroundSubtracted.tmp[is.na(backgroundSubtracted.tmp)] <- 0
      backgroundSubtracted.tmp <- raw.tmp[smp.names.all ,prb.names.all] - backgroundSubtracted.tmp[smp.names.all ,prb.names.all]
      backgroundSubtracted.tmp[backgroundSubtracted.tmp < 0] <- 0
      
      if(isTRUE(all.equal(backgroundSubtracted.tmp,raw.tmp[smp.names.all ,prb.names.all]))){
        backgroundSubtracted <- raw
        attr(backgroundSubtracted,"preprocessing.status") <- "background.subtracted"
      }else{
        backgroundSubtracted <- construct.data(dat = backgroundSubtracted.tmp,
                                               min.samples = min.samples,
                                               dat.type = "background.subtracted", 
                                               snvarg.ref.ID = snvarg.ref.ID,
                                               validated.prb.annot = validated.prb.annot,
                                               exclude.samples = exclude.samples, 
                                               exclude.probes = exclude.probes)
        warnings.paragraph <- paste(warnings.paragraph,backgroundSubtracted$warnings.paragraph)
        backgroundSubtracted <- backgroundSubtracted$dat
        backgroundSubtracted[backgroundSubtracted < 1] <- 1
      }
      
      
    }else{
      if(length(prb.names)>0){
        wtmp <- "rows and/or columns of background subtracted data file don\\'t match the raw data file. The analysis will use only raw data"
        warning(wtmp)
        warnings.paragraph <- paste(warnings.paragraph,wtmp)
      }
      
    }
    
  }
  
  #Processed: This is the data which may be normalized and/or background subtracted
  if(isTRUE(file.exists(path.data[grep("ProcessedData",path.data)]))){
    colnum <- ncol(read.csv(path.data[grep("ProcessedData",path.data)]))
    processed.tmp <- read.csv(path.data[grep("ProcessedData",path.data)],row.names=1,colClasses = c("character",rep("numeric",colnum-1)),check.names = F)
    
    if(isTRUE(all.equal(processed.tmp,raw.tmp))){
      processed <- raw
      attr(processed,"preprocessing.status") <- "processed"
    }else{
      processed.tmp <- construct.data(dat = processed.tmp,
                                      min.samples = min.samples,
                                      dat.type = "processed",
                                      snvarg.ref.ID = snvarg.ref.ID,
                                      validated.prb.annot = validated.prb.annot,
                                      exclude.samples = exclude.samples, 
                                      exclude.probes = exclude.probes)
      
      warnings.paragraph <- paste(warnings.paragraph,processed.tmp$warnings.paragraph)
      processed.tmp <- processed.tmp$dat 
      if(all(smp.names %in% rownames(processed.tmp)) & all(prb.names %in% colnames(processed.tmp))){
        processed <- processed.tmp
      }else{
        wtmp <- "rows and/or columns of processed data file don\\'t match the raw data file. The anlaysis will use only raw data"
        warning(wtmp)
        warnings.paragraph <- paste(warnings.paragraph,wtmp)
      }
    }
    
    
    
    
  }
  
  
  
  # Apply make.name to sample annotation variables
  sampleannot.variables2 <- sampleannot.variables
  if(length(sampleannot.variables)>0)
    sampleannot.variables=make.names(sampleannot.variables)
  
  
  
  # Load and construct probe annotation
  #------------------------------------
  
  # construct.default.prb.annot <- is.null(path.probeannot)
  
  if(!construct.default.prb.annot){
    cat("Constructing probe annotation object",file=log,sep='\n\n',append=TRUE)
    
    tmp <- tryCatch(expr = {construct.probe.annotation(validated.prb.annot = validated.prb.annot,
                                                       prb.names = prb.names,
                                                       prb.annotation.id.column = prb.annotation.id.column,
                                                       prb.set.column = prb.set.column,
                                                       prb.annotation.KEGG.id.column = prb.annotation.KEGG.id.column,
                                                       prb.annotation.KEGG.Gene.Name = prb.annotation.KEGG.Gene.Name,
                                                       prb.annotation.celltype.id.column = prb.annotation.celltype.id.column,
                                                       min.prb.set.size= min.prb.set.size,
                                                       min.KEGG.set.size = min.KEGG.set.size)}
                    , error = function(er){
                      ertmp <- paste("Provided probe annotations did not pass requirement check:\n",er)
                      cat(ertmp)
                      class(ertmp) <- "probe.annotation.check.error"
                      return(ertmp)
                    }
                    , finally ={})
    
    if(class(tmp) == "probe.annotation.check.error")
      construct.default.prb.annot <- TRUE
  }
  
  # if no probe annotation path is provided or if the provided annotation failed to meet the requirement
  # default annotation is inferred
  
  if(construct.default.prb.annot){
    cat("constructing default probe annotation files",file=log,sep='\n\n',append=TRUE)
    tmp <- list(prb.annot = get.default.pannot())
  }
  
  pannot <- tmp$prb.annot
  prb.set.matrix <- tmp$prb.set.matrix
  KEGG.set.matrix <- tmp$KEGG.set.matrix
  celltype.set.matrix <- tmp$celltype.set.matrix
  warnings.paragraph <- paste(warnings.paragraph,tmp$warnings.paragraph)
  
  suppressWarnings(write.table(pannot[1:5,1:5],file=log,sep='\t',append=TRUE))
  cat("loaded probe annotation",file=log,sep='\n\n',append=TRUE)
  cat("dimensions of pannot:",file=log,sep='\n',append=TRUE)
  cat(dim(pannot),file=log,sep='\n',append=TRUE)
  
  
  # Load and construct sample annotation
  #-------------------------------------
  cat("loading sample annotation files",file=log,sep='\n\n',append=TRUE)
  annot <- read.csv(path.sampleannot,row.names=sample.annotation.id.column,colClasses="character",check.names = F)
  colnames(annot)[which(colnames(annot) %in% sampleannot.variables2)] <- sampleannot.variables
  
  ## add heatmap annotations
  annot.names = intersect(c("QC.Flag","Imaging.QC.Flag","Binding.Density.QC.Flag","Positive.Control.QC.Flag"),colnames(annot)[colnames(annot)!="run.batch.id",drop=F])
  heatmap.annot = annot[,annot.names,drop=F]
  
  #   annot <- merge(x = data.frame(mrg = rownames(raw)),y = data.frame(mrg = rownames(annot),annot),by = "mrg",all.x = T)
  #   rownames(annot) <- annot$mrg ; annot$mrg <- "NULL"
  tmp <- construct.sample.annotation(smp.annot = annot, 
                                     smp.names = smp.names,
                                     sampleannot.variables = sampleannot.variables,
                                     sampleannot.variabletypes = sampleannot.variabletypes,
                                     sampleannot.referencelevels = sampleannot.referencelevels,
                                     validate.annot.dset = validate.annot.dset)
  annot <- tmp$smp.annot
  #heatmap.annot <- heatmap.annot[rownames(annot),,drop=F]
  sampleannot.variabletypes <- tmp$sampleannot.variabletypes
  sampleannot.referencelevels <- tmp$sampleannot.referencelevels
  warnings.paragraph <- paste(warnings.paragraph,tmp$warnings.paragraph)
  
  
  cat("loaded sample annotation",file=log,sep='\n\n',append=TRUE)
  cat("dimensions of annot:",file=log,sep='\n',append=TRUE)
  cat(dim(annot),file=log,sep='\n',append=TRUE)
  cat("dimensions of raw:",file=log,sep='\n',append=TRUE)
  cat(dim(raw),file=log,sep='\n',append=TRUE)
  
  
  print("Loaded user files")  
  cat("LOG:Loaded user files",file=log,sep='\n\n',append=TRUE)
  
  print("Finished setup; starting annotations and data checking")
  cat("LOG:Finished setup; starting annotations and data checking",file=log,sep='\n\n',append=TRUE)
  
  
  # generate aesthetics for sample annotation
  #------------------------------------------
  smp.annot.aes <- list()
  if(validate.annot.dset)
    smp.annot.aes <- annot.to.aes(annot = annot[,!colnames(annot) %in% c("run.batch.id","Batch.ID"),drop=F])
  
  heatmap.annot <- heatmap.annot[,!(colnames(heatmap.annot)%in%colnames(annot)),drop=F]
  for (i in 1:ncol(heatmap.annot)) {
    if (class(heatmap.annot[,i])=="character") {
      heatmap.annot[,i] <- as.factor(heatmap.annot[,i])
    }
  }
  heatmap.annot <- cbind(annot[,!colnames(annot) %in% c("run.batch.id","Batch.ID"),drop=F], heatmap.annot[match(rownames(annot), rownames(heatmap.annot)),,drop=F])
  
  # render the aesthetic legends
  #-----------------------------
  if (draw.color.legend & validate.annot.dset)
    render.annot.aes.legends(annotcols = smp.annot.aes$annotcols,annotcols2 = smp.annot.aes$annotcols2,plottypearg = plottypearg,path.results = path.results)
  
  
  
  out <- list(rawdata=raw,
              processed = processed,
              backgroundSubtracted = backgroundSubtracted, 
              annot=annot,
              pannot=pannot,
              prb.set.matrix = prb.set.matrix,
              KEGG.set.matrix = KEGG.set.matrix,
              celltype.set.matrix = celltype.set.matrix,
              annotcols=smp.annot.aes$annotcols,
              annotcols2=smp.annot.aes$annotcols2,
              sampleannot.variables=colnames(annot)[!colnames(annot) %in% c("run.batch.id","Batch.ID")],
              sampleannot.variabletypes=sampleannot.variabletypes,
              sampleannot.referencelevels=sampleannot.referencelevels,
              #prune=prune,
              warnings.paragraph=warnings.paragraph,
              heatmap.annot = heatmap.annot)
  
  return(out)
  
}




#' construct a data matrix that meets the basic requirements of the analysis (i.e. is numeric, has more than 1 sample, and doesn't have duplicate observations)
#' and optionally allows for additional filtering via definition of "filter" function
#' @param dat: nxp numeric data.frame or matrix
#' @param min.sample: smallest number of rows allowed
#' @param dat.type: "raw","background.subtracted","processed"
#' @param exclude.samples: a character vector of rownames to be excluded
#' @param excluce.probes: a character vector of colnames to be excluded
#' @param filter: optional function to which dat will be passed for additional modifications
# value:
#' @return dat:  nxp matrix that passes the basic requirements of the analysis. The attribute "preprocessing.status" is also set to raw or normalized based on flag
#' @return warnings.paragraph
construct.data <- function(dat, 
                           min.samples = 1,
                           dat.type,
                           snvarg.ref.ID = NULL,
                           validated.prb.annot,
                           exclude.samples=NULL,
                           exclude.probes=NULL,
                           filter = function(x,...){return(x)},...){
  
  warnings.paragraph <- character(0)
  
  min.snv.ref.ID <- 10
  
  if(!(is.data.frame(dat) | is.matrix(dat)))
    stop("dat needs to be data.frame or matrix")
  
  
  if(any(exclude.probes %in% colnames(dat)))
    dat <- dat[,!exclude.probes,drop=F]
  if(any(exclude.samples %in% colnames(dat)))
    dat <- dat[!exclude.samples,,drop=F]
  
  #Drop columns with all NA <-------------this is intended to correct read.csv sometimes reading an extra empty column 
  dat <- dat[,!sapply(as.data.frame(dat),function(x)all(is.na(x))),drop=F]
  
  if(any(!sapply(dat,FUN = class) %in% c("numeric","integer")))
    stop("all columns of dat needs to numeric")
  
  # filter dat per specified filter
  dat <- filter(dat,...)
  
  dat <- as.matrix(dat)
  
  # If all dat==0 return NULL
  #--------------------------
  if(all(dat==0,na.rm = T))
    return(NULL)
  
  # Remove constant lanes
  #----------------------
  constant.lanes <- names(which(apply(dat,1,function(x) sd(x,na.rm = T))==0))
  if(length(constant.lanes)>0){
    dat <- dat[!rownames(dat) %in% constant.lanes,, drop=F]
    cat(paste("document.write('<p>",paste(constant.lanes,collapse = ","),"are constant and are removed","</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  }
  
  
  # Exit duplicate sample row
  #------------------------------
  samplecor = cor(t(dat),use="pairwise.complete.obs")
  diag(samplecor) <- 0
  if(max(abs(samplecor),na.rm = T)==1)
  {
    cat("document.write('<p>There are at least two samples with identical expression profiles.  Remove duplicate RCCs and re-run the analysis.</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
    stop("There are at least two samples with identical expression profiles.  Remove duplicate RCCs and re-run the analysis.")       
  }
  
  
  
  
  # Extract the SNV ref and add to attribute
  #-----------------------------------------
  probe.by.analyte <- split(validated.prb.annot$ProbeID, validated.prb.annot$Analyte.Type)
  names(probe.by.analyte) <- tolower(names(probe.by.analyte))
  probe.by.analyte <- sapply(probe.by.analyte,FUN = function(x) x[x %in% colnames(dat)],USE.NAMES = T,simplify = F)
  
  
  if(any(!snvarg.ref.ID %in% rownames(dat))){
    wtmp <- paste0("Invalid SNV ref sample name(s):",paste(snvarg.ref.ID[!snvarg.ref.ID %in% rownames(dat)],collapse = ","),"\nSNV ref names need to be in the row names of the data")
    cat(paste0(paste0("document.write('<p>",gsub("\n","",wtmp)),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
    warning(wtmp)
    warnings.paragraph <- paste(warnings.paragraph,wtmp)
    
    snvarg.ref.ID <- snvarg.ref.ID[snvarg.ref.ID %in% rownames(dat)]
    if(length(snvarg.ref.ID) <= min.snv.ref.ID){
      snvarg.ref.ID <- NULL
      wtmp <- paste("Valid SNV ref sample provided are less than minimum required", min.snv.ref.ID, "SNV analysis can not be completed!")
      cat(paste0("document.write('<p>",wtmp ," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
      warning(wtmp)
      warnings.paragraph <- paste(warnings.paragraph,wtmp)
    }
  }
  
  
  if(length(snvarg.ref.ID) > 0){
    
    snv.ref.dat <- dat[snvarg.ref.ID, probe.by.analyte$snv, drop = F]
    dat.tmp <- dat[!rownames(dat) %in% snvarg.ref.ID, , drop = F]
    
    if( min.snv.ref.ID <= nrow(snv.ref.dat) & ncol(snv.ref.dat) > 0 & all(dim(dat.tmp)>0) ){
      dat <- dat.tmp
      attr(dat,"snv.ref") <- snv.ref.dat
      attr(dat,"dat.subsets") <- c(attr(dat,"dat.subsets"),"snv.ref")
    }else{
      snvarg.ref.ID <- NULL
      wtmp <- paste("Valid SNV ref sample provided needs to be >", min.snv.ref.ID, ". SNV analysis can not be completed!")
      cat(paste0(paste0("document.write('<p>",gsub("\n","",wtmp)),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
      warning(wtmp)
      warnings.paragraph <- paste(warnings.paragraph,wtmp)
    }
    rm(dat.tmp,snv.ref.dat)
  }
  
  
  # Add control probes to attributes
  #---------------------------------
  ctrl.prbs <- intersect(validated.prb.annot$ProbeID[validated.prb.annot$Is.Control==1],colnames(dat))
  if(length(ctrl.prbs)>0){
    attr(dat,"ctrl") <- dat[,ctrl.prbs,drop=F]
    attr(dat,"dat.subsets") <-  c(attr(dat,"dat.subsets"),"ctrl")
  }
  
  
  attr(dat,"dat.subsets") <- c(attr(dat,"dat.subsets"),names(probe.by.analyte)[!names(probe.by.analyte) %in% c("mrna","protein")])
  
  # separate non mRNA/protein data as attribute
  #--------------------------------------------
  for(analyte.i.prbs  in names(probe.by.analyte)[!names(probe.by.analyte) %in% c("mrna","protein")] ){
    attr(dat, analyte.i.prbs) <- dat[,probe.by.analyte[[analyte.i.prbs]],drop = F]
    dat <- subset.mat.obj(dat = dat,keep.cols = !colnames(dat) %in% probe.by.analyte[[analyte.i.prbs]])
    
  }
  
  # restrict the data to complete cases (and later perhaps find another options e.g. biggest contiguous chunk)
  #-----------------------------------------------------------------------------------------------------------
  get.contiguous <- function(dat){
    select.rows <- complete.cases(dat)
    if(any(!select.rows)){
      wtmp <- paste0("Samples ",paste(rownames(dat[!select.rows,]),collapse = ","),"will be dropped as they contain missing values for some probes")
      cat(paste0("document.write('<p>",wtmp ," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
      warning(wtmp)
      warnings.paragraph <- paste(warnings.paragraph,wtmp)
    }
    dat <- subset.mat.obj(dat,keep.rows = select.rows)
    return(dat)
  }
  
  dat <- get.contiguous(dat)
  
  for(dset.i  in  attr(dat,"dat.subsets")){
    attr(dat, dset.i) <- get.contiguous(attr(dat, dset.i))
    if(any(dim(attr(dat, dset.i))==0)){
      attr(dat, dset.i) <- NULL
      attr(dat,"dat.subsets") <- attr(dat,"dat.subsets")[attr(dat,"dat.subsets")!=dset.i]
    }
    
  }
  
  
  # Remove analyte type if only control type?
  #------------------------------------------
  tb <- table(validated.prb.annot[colnames(dat),"Analyte.Type"],factor(validated.prb.annot[colnames(dat),"Is.Control"],levels = c("0","1")))
  eliminate.analytes <- character(0)
  if(nrow(tb)>0 & ncol(tb)>0)
    eliminate.analytes <- rownames(tb)[tb[,"0"]==0]
  if(length(eliminate.analytes)>0){
    wtmp <- paste(paste(eliminate.analytes,collapse = ","), "will be eliminated as they only consist of controls")
    cat(paste0("document.write('<p>",wtmp ," </p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
    warning(wtmp)
    warnings.paragraph <- paste(warnings.paragraph,wtmp)
    
    for(analyte.i.eliminate in eliminate.analytes){
      dat <- subset.mat.obj(dat,keep.cols = !colnames(dat) %in% probe.by.analyte[[tolower(analyte.i.eliminate)]])
    }
    
  }
  
  
  # Exit if less than min sample size:
  #-----------------------------------
  if(nrow(dat)<min.samples)
    stop(paste("Less than",min.samples," samples were included in the analysis.  Re-run with multiple samples."))
  
  
  
  # set the data preprocessing.status
  #----------------------------------
  attr(dat,"preprocessing.status") <- dat.type
  #   if(normalized)
  #     attr(dat,"preprocessing.status") <-  "normalized"
  
  return(list(dat = dat, warnings.paragraph = warnings.paragraph))
}





#' construct probe annotation that meets the basic requirement of the analysis(i.e. is matrix or data.frame)
#' @param prb.annot: a matrix or a data.frame containing probe annotation information
#' @param prb.names: a character vector containing the probe names for which we have data. It ensures that the output has a row for each of the prb.names in order
#' @param prb.annotation.id.column: a character defining the name of the column containing unique probe ids
#' @param prb.set.column: NULL or a character string, the name of the column in prb.annot containing probe set information
#' @param prb.annotation.KEGG.id.column: a character. it is the name of the column with KEGG ids
#' @param prb.annotation.KEGG.Gene.Name: a character. It is the name of the column with the gene names to be used in KEGG pathway calls
#' @param prb.annotation.celltype.id.column: a character. It is the name of the column with the celltype info
#' @param min.prb.set.size: min number of probes needed to present in probeset for the probeset to be valid
#' @param min.KEGG.set.size: min number of probes present in the KEGG pathway
#' @param NA.handle.fun: a function that receives the prb.annotation matrix/data.frame and handles how to replace NA
# value:
#' @return prb.annot: constructed annotation data.frame to meet the analysis requirements
#' @return prb.set.matrix: a 0/1 matrix of probe membership in probesets. if prb.set.column is NULL, this will be NULL too
#' @return KEGG.set.matrix: a 0/1 matrix of probe membership in KEGG pathways
#' @return celltype.set.matrix: a 0/1 matrix of probe membership in cell type marker sets
#' @return warnings.paragraph

construct.probe.annotation <- function(validated.prb.annot,
                                       prb.names,
                                       prb.annotation.id.column,
                                       prb.set.column = NULL,
                                       prb.annotation.KEGG.id.column = NULL,
                                       prb.annotation.KEGG.Gene.Name = NULL,
                                       prb.annotation.celltype.id.column = NULL,
                                       min.prb.set.size,
                                       min.KEGG.set.size,
                                       NA.handle.fun = function(x,...){return(x)},
                                       ...){
  
  
  
  warnings.paragraph <- character(0)
  
  
  # if(sum(rownames(validated.prb.annot) %in% prb.names)== 0)
  #   return(NULL)
  # stop("No match between the probe ids in the annotation and probe ids in the dataset was found")
  
  
  # Break the validated.prb.annot into structure of prb.names
  #----------------------------------------------------------
  dats.in.attr <- names(attributes(prb.names))
  attr(validated.prb.annot,"dat.subsets") <- dats.in.attr
  for(dat.i.in.attr in dats.in.attr){
    attr(validated.prb.annot,dat.i.in.attr) <- validated.prb.annot[rownames(validated.prb.annot) %in% attr(prb.names,dat.i.in.attr),]
    if(any(dim(attr(validated.prb.annot,dat.i.in.attr))==0))
      attr(validated.prb.annot,"dat.subsets") <- attr(validated.prb.annot,"dat.subsets")[attr(validated.prb.annot,"dat.subsets")!=dat.i.in.attr]
  }
  # validated.prb.annot <- subset.mat.obj(dat = validated.prb.annot,keep.rows = rownames(validated.prb.annot) %in% prb.names)
  stored.attr <- c(dat.subsets=list(attr(validated.prb.annot,"dat.subsets")),sapply(attr(validated.prb.annot,"dat.subsets"),FUN = function(x) attr(validated.prb.annot,x),simplify = F,USE.NAMES = T))
  
  
  
  # keep only validated.prb.annot for probes in prb.names
  tmp.types <- sapply(validated.prb.annot,FUN = class)
  tmp <- as.data.frame(matrix(nrow = length(prb.names),ncol=ncol(validated.prb.annot),dimnames = list(prb.names,colnames(validated.prb.annot))))
  tmp <- data.frame(sapply(colnames(tmp),FUN = function(x) {y <- tmp[,x]; class(y) <- tmp.types[x]; return(y)},simplify = F),stringsAsFactors = F,row.names = prb.names)
  tmp[rownames(validated.prb.annot)[rownames(validated.prb.annot) %in% prb.names], ] <- validated.prb.annot[rownames(validated.prb.annot)[rownames(validated.prb.annot) %in% prb.names],]
  validated.prb.annot <- tmp
  
  
  # Make probe labels unique within analyte by appending index
  # This is for the case of multiple probes of the same analyte
  # type having the same name (different probe ID) as in RLF merge
  # could happen
  #-----------------------------------------------------------
  for(analyte in unique(validated.prb.annot$Analyte.Type)){
    validated.prb.annot[validated.prb.annot$Analyte.Type == analyte,"Probe.Label"] <- make.unique(validated.prb.annot[validated.prb.annot$Analyte.Type == analyte,"Probe.Label"])
  }
  
  
  # Check probeset info
  if(!isTRUE(prb.set.column %in% colnames(validated.prb.annot))){
    if(length(prb.names) > 0){
      print("Warning: No probe set annotation is provided")
      warnings.paragraph <- paste(warnings.paragraph,"Warning: No probe set annotation is provided\n")
      warning("colnames(validated.prb.annot) needs to include prb.set.column")
    }
    prb.set.matrix <- NULL
  }
  
  
  # Check KEGG id and Gene.Name
  if(!isTRUE(prb.annotation.KEGG.id.column %in% colnames(validated.prb.annot)) ){
    if(length(prb.names) > 0){
      print("Warning: No KEGG id annotation is provided")
      warnings.paragraph <- paste(warnings.paragraph,"Warning: No KEGG id annotation is provided\n")
      warning("colnames(validated.prb.annot) needs to include prb.annotation.KEGG.id.column")
    }
    prb.annotation.KEGG.id.column <- NULL
  }
  
  
  # Check KEGG id and Gene.Name
  if(!isTRUE(prb.annotation.KEGG.Gene.Name %in% colnames(validated.prb.annot))){
    if(length(prb.names) > 0){
      print("Warning: No KEGG gene name annotation is provided")
      warnings.paragraph <- paste(warnings.paragraph,"Warning: No KEGG gene name annotation is provided\n")
      warning("colnames(validated.prb.annot) needs to include prb.annotation.KEGG.Gene.Name")
    }
    prb.annotation.KEGG.Gene.Name <- NULL
  }
  
  # Check cell type annotation
  if(!isTRUE(prb.annotation.celltype.id.column %in% colnames(validated.prb.annot))){
    if(length(prb.names) > 0){
      print("Warning: No celltype probe annotation is provided")
      warnings.paragraph <- paste(warnings.paragraph,"Warning: No celltype probe annotation is provided\n")
      warning("colnames(validated.prb.annot) needs to include prb.annotation.celltype.id.column")
    }
    prb.annotation.celltype.id.column <- NULL
  }
  
  
  
  # handle NA per specified NA.handle.func
  validated.prb.annot <- NA.handle.fun(validated.prb.annot,...)
  
  
  
  # construct the probeset matrix
  #------------------------------
  prb.set.matrix <- NULL
  if(!is.null(prb.set.column)){
    
    
    #Swap illegal characters in prb.set.column with their legal char combo
    #---------------------------------------------------------------------
    validated.prb.annot[,prb.set.column] <- swap.illegal.char(validated.prb.annot[,prb.set.column])
    
    tmp <- tryCatch(expr = get.set.matrix(validated.prb.annot = validated.prb.annot,set.column.id = prb.set.column,min.set.size = min.prb.set.size,ensure.cases.insensitivity = TRUE),
                    error = function(er){
                      ertmp <- paste("Geneset information is discarded: the following error occurred\n",er)
                      cat(ertmp)
                      cat("document.write('<p> Geneset information is discarded: see summary for more info</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
                      class(ertmp) <- "error"
                      return(ertmp)
                    },
                    finally = {})
    if(class(tmp) == "error"){
      warnings.paragraph <- paste(warnings.paragraph,tmp)
    }else{
      warnings.paragraph <- paste(warnings.paragraph,tmp$warnings.paragraph)
      prb.set.matrix <- tmp$set.matrix
    }
  }
  
  KEGG.set.matrix <- NULL
  if(!is.null(prb.annotation.KEGG.id.column) & !is.null(prb.annotation.KEGG.Gene.Name)){
    
    
    tmp <- tryCatch(expr = get.set.matrix(validated.prb.annot = validated.prb.annot,set.column.id = prb.annotation.KEGG.id.column,min.set.size = min.KEGG.set.size,ensure.cases.insensitivity = FALSE),
                    error = function(er){
                      ertmp <- paste("KEGG information is discarded: the following error occurred\n",er)
                      cat(ertmp)
                      cat("document.write('<p> KEGG information is discarded: see summary for more info</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
                      class(ertmp) <- "error"
                      return(ertmp)
                    },
                    finally = {})
    
    if(class(tmp) == "error"){
      warnings.paragraph <- paste(warnings.paragraph,tmp)
    }else{
      warnings.paragraph <- paste(warnings.paragraph,tmp$warnings.paragraph)
      KEGG.set.matrix <- tmp$set.matrix
    } 
  }
  
  
  celltype.set.matrix <- NULL
  if(!is.null(prb.annotation.celltype.id.column)){
    tmp <- tryCatch(expr = get.set.matrix(validated.prb.annot = validated.prb.annot,set.column.id = prb.annotation.celltype.id.column,min.set.size = 0,ensure.cases.insensitivity = TRUE),
                    error = function(er){
                      ertmp <- paste("Cell Type information is discarded: the following error occurred\n",er)
                      cat(ertmp)
                      cat("document.write('<p> Cell Type information is discarded: see summary for more info</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
                      class(ertmp) <- "error"
                      return(ertmp)
                    },
                    finally = {})
    
    if(class(tmp) == "error"){
      warnings.paragraph <- paste(warnings.paragraph,tmp)
    }else{
      warnings.paragraph <- paste(warnings.paragraph,tmp$warnings.paragraph)
      celltype.set.matrix <- tmp$set.matrix
    } 
  }
  
  # Make analyte.type class insensetive
  #------------------------------------
  validated.prb.annot$Control.Type[tolower(validated.prb.annot$Control.Type) == tolower("Protein_NEG")] <- "Protein_NEG"
  
  #Attach attributes
  #-----------------
  attributes(validated.prb.annot) <- c(attributes(validated.prb.annot),stored.attr)
  
  return(list(prb.annot = validated.prb.annot,
              prb.set.matrix = prb.set.matrix,
              KEGG.set.matrix = KEGG.set.matrix,
              celltype.set.matrix = celltype.set.matrix,
              warnings.paragraph=warnings.paragraph))
  
}



#' Wrapper for construct.sample.annotation.per.dset. This function calls construct.sample.annotation.per.dset for the main
#' dataset (mRNA/protein) as well as for each dataset structured as attributes (e.g. SNV, fusion).

construct.sample.annotation <- function(smp.annot,
                                        smp.names,
                                        sampleannot.variables,
                                        sampleannot.variabletypes,
                                        sampleannot.referencelevels,
                                        cart.infer.smp.annot.cols = c("Cartridge.Id", "Scanned.Date","Scanner.Id"),
                                        validate.annot.dset){
  warnings.paragraph <- character(0)
  
  if(!is.data.frame(smp.annot))
    stop("smp.annot needs to be a data.frame")
  
  if(any(!sampleannot.variables %in% colnames(smp.annot)))
    stop("sampleannot.variables needs to be in colnames(smp.annot)")
  
  if(!is.character(smp.names))
    stop("smp.names need to be character vector")
  
  infer.cart.id <- TRUE
  if(any(!cart.infer.smp.annot.cols %in% colnames(smp.annot))){
    wtmp <- paste("The following attribute(s) missing from sample annotation and cartridge id can\\'t be inferred (not unusual if merged experiments):",
                  paste(cart.infer.smp.annot.cols[!cart.infer.smp.annot.cols %in% colnames(smp.annot)],collapse=","))
    cat(paste0(paste0("document.write('<p>",gsub("\n","",wtmp)),"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE) 
    warning(wtmp)
    warnings.paragraph <- paste(warnings.paragraph,wtmp)
    infer.cart.id <- FALSE
  }
  
  
  # Construct uqniue cartridge id  and add as run.batch used in snv
  #----------------------------------------------------------------
  
  #Check the "run.batch.id" exists if so change it and generate a warning
  if("run.batch.id" %in% colnames(smp.annot) & "run.batch.id" %in% sampleannot.variables){
    
    tmp.colname <- make.unique(c("run.batch.id",colnames(smp.annot)))[1+which(colnames(smp.annot) == "run.batch.id")]
    colnames(smp.annot)[which(colnames(smp.annot) == "run.batch.id")] <- tmp.colname
    sampleannot.variables[which(sampleannot.variables == "run.batch.id")] <- tmp.colname
    warnings.paragraph <- paste(warnings.paragraph,"\nrun.batch.id is an internal key name. The variable names was changed to",tmp.colname,"\n")
  }
  
  smp.annot$run.batch.id  <- "undefined" # default
  if(infer.cart.id){
    smp.annot$run.batch.id <- paste0(smp.annot$Cartridge.Id,smp.annot$Scanned.Date,smp.annot$Scanner.Id)
    smp.annot$run.batch.id <- as.character(factor(smp.annot$run.batch.id,labels = paste0("run.batch.",1:length(unique(smp.annot$run.batch.id)))))
  }
  
  
  
  #Check the "Batch.ID" exists if so change it and generate a warning
  if("Batch.ID" %in% colnames(smp.annot)){
    if("Batch.ID" %in% sampleannot.variables){
      tmp.colname <- make.unique(c("Batch.ID",colnames(smp.annot)))[1+which(colnames(smp.annot) == "Batch.ID")]
      smp.annot[[tmp.colname]] <- smp.annot$Batch.ID
      sampleannot.variables[which(sampleannot.variables == "Batch.ID")] <- tmp.colname
      # warnings.paragraph <- paste(warnings.paragraph,"\nBatch.ID is an internal key name. The variable names was changed to",tmp.colname,"\n")
    }
  }else
    smp.annot$Batch.ID <- "undefined"
  if(any(is.na(smp.annot$Batch.ID)))
    smp.annot$Batch.ID <- "undefined"
  
  
  sannot <- construct.sample.annotation.per.dset(smp.annot = smp.annot,
                                                 smp.names = smp.names,
                                                 sampleannot.variables = sampleannot.variables,
                                                 sampleannot.variabletypes = sampleannot.variabletypes,
                                                 sampleannot.referencelevels = sampleannot.referencelevels,
                                                 sampleannot.run.batch.id = character(0),
                                                 sampleannot.userdf.batch.id = character(0),
                                                 validate.annot.dset = validate.annot.dset)
  
  
  
  sannot.per.dset <- sapply(names(attributes(smp.names)), FUN =  function(atr.i) construct.sample.annotation.per.dset(smp.annot = smp.annot,
                                                                                                           smp.names = attr(smp.names,atr.i),
                                                                                                           sampleannot.variables = sampleannot.variables,
                                                                                                           sampleannot.variabletypes = sampleannot.variabletypes,
                                                                                                           sampleannot.referencelevels = sampleannot.referencelevels,
                                                                                                           sampleannot.run.batch.id = "run.batch.id",
                                                                                                           sampleannot.userdf.batch.id = "Batch.ID"["Batch.ID" %in% colnames(smp.annot)],
                                                                                                           validate.annot.dset = attr(validate.annot.dset,atr.i)),USE.NAMES = T, simplify = F)
  
  smp.annot <- sannot$smp.annot
  attr(smp.annot,"dat.subsets") <- names(sannot.per.dset)
  attributes(smp.annot) <- c(attributes(smp.annot) ,lapply(sannot.per.dset,FUN = function(x) x$smp.annot))
  
  
  sampleannot.variabletypes <- sannot$sampleannot.variabletypes
  attr(sampleannot.variabletypes,"dat.subsets") <- names(sannot.per.dset)
  attributes(sampleannot.variabletypes) <- c( attributes(sampleannot.variabletypes) ,lapply(sannot.per.dset,FUN = function(x) x$sampleannot.variabletypes))
  
  
  sampleannot.referencelevels <- sannot$sampleannot.referencelevels
  attr(sampleannot.referencelevels,"dat.subsets") <- names(sannot.per.dset)
  attributes(sampleannot.referencelevels) <- c( attributes(sampleannot.referencelevels) ,lapply(sannot.per.dset,FUN = function(x) x$sampleannot.referencelevels))
  
  warnings.paragraph <- unique(c(sannot$warnings.paragraph,unlist(lapply(sannot.per.dset,function(x)x$warnings.paragraph))))
  
  return(list(smp.annot = smp.annot,
              sampleannot.variabletypes = sampleannot.variabletypes,
              sampleannot.referencelevels = sampleannot.referencelevels,
              warnings.paragraph = warnings.paragraph))
  
}


#' Given the input sample annotation data.frame and desired list of variables and their specifications, it construct a valid sample annotation data.frame
#' that meets the analysis requirement. Specifically it handles type casting the variables and removes variables that are constant.
#' NOTE: no check on legality of names are performed. Only legal names are expected here.

#' @param smp.annot: a data.frame containing annotations for samples on variables of interest
#' @param smp.names: a character vector containing the sample names for which we had data (in constructed data object). It ensures the ouput has a row for each smp.names in order
#' @param sampleannot.variables: character vector of length m specifying the variables of interest. The elements need to be contained within the colnames of smp.annot
#' @param sampleannot.variabletypes: a character vector of length m describing the type of each sampleannot.variables element to be cast in. 
#'                             It needs to consist of element "categorical","continuous" and "boolean"
#' @param  sampleannot.referencelevels: a character vector of length m, containig the reference level for the vector sampleannot.variables. If sampleannot.variables is 
#'                               "continuous", NA is the convention (although not necessary as any value will be ignored)
#'  
#' @return  smp.annot: a data.frame smp.annot, constructed to meet the analysis requirement
#' @return sampleannot.variabletypes: named character vector of variable types
#' @return sampleannot.referencelevels: named character vector of reference levels for categorical variables or NA
#' @return warnings.paragraph: this is a character string to be appended to warning paragraph


construct.sample.annotation.per.dset <- function(smp.annot,
                                                 smp.names,
                                                 sampleannot.variables,
                                                 sampleannot.variabletypes,
                                                 sampleannot.referencelevels,
                                                 sampleannot.run.batch.id,
                                                 sampleannot.userdf.batch.id,
                                                 validate.annot.dset,
                                                 enforce.nlevel = length(sampleannot.run.batch.id)==0){
  
  warnings.paragraph <- character(0)
  
  # Get only smp.annots for which we have samples and restrict the colnames to sampleannot.variables
  smp.annot <- smp.annot[rownames(smp.annot) %in% smp.names,unique(c(sampleannot.variables,sampleannot.run.batch.id,sampleannot.userdf.batch.id)),drop=F]
  tmp <- as.data.frame(matrix(nrow = length(smp.names),ncol =ncol(smp.annot),dimnames = list(smp.names,colnames(smp.annot))))
  tmp[rownames(smp.annot),] <-smp.annot
  smp.annot <- tmp
  
  if(!validate.annot.dset)
    return(list(smp.annot = smp.annot,
                sampleannot.variabletypes = sampleannot.variabletypes,
                sampleannot.referencelevels = sampleannot.referencelevels,
                warnings.paragraph = warnings.paragraph))
  
  # Add batch.id if present to sample annot variables
  #---------------------------------------------------
  sampleannot.variables <- c(sampleannot.variables,sampleannot.run.batch.id,sampleannot.userdf.batch.id)
  sampleannot.variabletypes <- c(sampleannot.variabletypes, 
                                 "categorical"[length(sampleannot.run.batch.id) ==1],
                                 "categorical"[length(sampleannot.userdf.batch.id) ==1])
  
  r.batches <- levels(factor(unlist(smp.annot[,sampleannot.run.batch.id])))
  ud.batches <- levels(factor(unlist(smp.annot[,sampleannot.userdf.batch.id])))
  
  sampleannot.referencelevels <- c(sampleannot.referencelevels,
                                   r.batches[0:min(1,length(r.batches))],
                                   ud.batches[0:min(1,length(ud.batches))])
  
  
  # construct the smp.annot metadata
  #---------------------------------
  annot.vars <- data.frame(type = sampleannot.variabletypes, 
                           sampleannot.referencelevels,
                           row.names = sampleannot.variables,
                           stringsAsFactors = F)
  
  # type cast
  #----------
  vars.to.remove <- warnings.paragraph <- NULL
  for(v in sampleannot.variables){
    tmp <- tryCatch(expr = type.cast(x = smp.annot[,v],ref = annot.vars[v,"sampleannot.referencelevels"], type = annot.vars[v,"type"])
                    , error = function(er){
                      ertmp <- paste("Error in assigning variable",v,"as",annot.vars[v,"type"],"\n",er)
                      class(ertmp) <- "type.cast.error"
                      return(ertmp)
                    }
                    , warning = function(w){
                      wtmp <- paste("Warning in assigning variable type for",v,"as",annot.vars[v,"type"],"\n", w)
                      class(wtmp) <- "type.cast.warning"
                      return(wtmp)
                    }
                    ,finally ={})
    
    if(class(tmp) == "type.cast.error"){
      vars.to.remove <- c(vars.to.remove,v)
      warnings.paragraph <- paste(warnings.paragraph,tmp)
      next
    }
    if(class(tmp) == "type.cast.warning"){
      warnings.paragraph <- paste(warnings.paragraph,tmp)
      smp.annot[,v] <- suppressWarnings(type.cast(x = smp.annot[,v],ref = annot.vars[v,"sampleannot.referencelevels"], type = annot.vars[v,"type"]))
    }
    if(any(!class(tmp) %in% c("type.cast.error","type.cast.warning")))
      smp.annot[,v] <- tmp
  }
  
  
  if(enforce.nlevel){
    annot.vars[,"nlevels"] <- sapply(smp.annot,function(x) nlevels(factor(x)) )[sampleannot.variables]
    
    # Flag single level varaibles for removal (other than run.batch.id)
    #-----------------------------------------------------------------
    # if(sum(vars.nlevels[setdiff(names(vars.nlevels),sampleannot.run.batch.id)]==1)>0)
    single.level.vars <- setdiff(rownames(annot.vars)[annot.vars$nlevels==1],sampleannot.run.batch.id)
    if(length(single.level.vars)>0)
      warnings.paragraph <- paste(warnings.paragraph,"constant variable(s) are removed:",paste(single.level.vars,"\n",collapse = ",") )
    
    vars.to.remove <- union(vars.to.remove,single.level.vars)
    
    # Flag categorical variables with as many levels as nrow for removal (comment from n.level.vars to vars.to.remove related to )
    #-------------------------------------------------------------------
    # n.level.vars <- setdiff(rownames(annot.vars)[annot.vars$nlevels==nrow(smp.annot) & annot.vars$type == "categorical"],sampleannot.run.batch.id)
    # if(length(n.level.vars)>0)
    #   warnings.paragraph <- paste(warnings.paragraph,"categorical variable(s) with as many levels as overall sample size are removed:\n",paste(n.level.vars,collapse = ",") )
    # 
    # vars.to.remove <- union(vars.to.remove,n.level.vars)
  }
  
  
  # Get final smp.annot
  #--------------------
  smp.annot <- smp.annot[,sampleannot.variables[!sampleannot.variables %in% vars.to.remove],drop=F]
  
  if(ncol(smp.annot)==0)
    stop("No viable sample annotation was provided, each sample annotation needs to have at least two distinict levels")
  
  # Reset sample annot variable type
  # NOTE: right now boolean is cast as categorical so it is not used
  #-----------------------------------------------------------------
  sampleannot.variabletypes <- c(factor = "categorical",numeric = "continuous",integer="continuous",logical = "boolean")[sapply(smp.annot,class)]
  names(sampleannot.variabletypes) <- colnames(smp.annot) #setdiff(colnames(smp.annot),sampleannot.run.batch.id)
  
  # Reset sample reference
  #-----------------------
  sampleannot.referencelevels <- unlist(sapply(smp.annot,function(x) ifelse(is.factor(x),levels(x)[1],NA)))
  
  return(list(smp.annot = smp.annot,
              sampleannot.variabletypes = sampleannot.variabletypes,
              sampleannot.referencelevels = sampleannot.referencelevels,
              warnings.paragraph = warnings.paragraph))
}



#' This function uses the "GeneAnnotations_additional.csv" file to build probe annotation file containing the following
#' columns: "UniqueID","Probe.Label","Analyte.Type","Is.Control","Control.Type",  and if available "Control.Conc".
#' Additionally it generates a default probeset including all probes
#' @param fl: the path to where nSolver outputs the default "GeneAnnotations_additional.csv"
#' NOTE: this is not working currently because nSovler going from 3.x to 4.0 changed GeneAnnotations_Addtional Probe.Name from id to actual name :( 

get.default.pannot <- function(fl = "./GeneAnnotations_additional.csv"){
  require(stringr)
  
  control.names <- c("Positive","Negative","Housekeeping",
                     "Protein_NEG","Protein_CELLNORM",
                     "Invariant","RestrictionSite","SpikeIn","Ligation",
                     "SNV_INPUT_CTL", "SNV_PCR_CTL","SNV_UDG_CTL","SNV_NEG", "SNV_POS", 
                     "Purification","Binding")
  
  pannot<- read.csv(fl,check.names = F,stringsAsFactors = F)
  UniqueID <- pannot$Probe.Name
  Probe.Label <- unlist(lapply(pannot$Gene.Name.Class.Name,function(x) paste(rev(rev(unlist(strsplit(x = x,split = "\\."),recursive = F))[-1]),collapse=".")))
  class.name <- unlist(lapply(pannot$Gene.Name.Class.Name,function(x) rev(unlist(strsplit(x,split = "\\.")))[1]))
  Is.Control <- as.numeric(class.name %in% control.names)
  Control.Type <- rep("",nrow(pannot));Control.Type[Is.Control==1] <- class.name[Is.Control==1]
  
  
  
  
  #protein
  prt.indices <- grep("^nProt_",UniqueID) # starts with nProt_
  prtCTRL.indices <- grep("^nPCtl_",UniqueID) # starts with nPCtl_
  
  #miRNA
  miRNA.indices <- grep("^nmiR[0-9]{5}\\.",UniqueID) #starts with nmiR followed by 5 numbers and followed by "."
  
  
  #Make everything not protein or miRNA into mRNA
  Analyte.Type <- rep("mRNA",nrow(pannot))
  Analyte.Type[unique(c(prt.indices,prtCTRL.indices))] <- "protein"
  Analyte.Type[miRNA.indices] <- "miRNA"
  
  #Distinguish between CNV and mRNA based on Assay type as both are type Endogenous
  Analyte.Type[pannot$Assay.Type == "CNV"] <- "DNA"
  
  
  prb.annot <- data.frame(ProbeID= UniqueID,
                          Probe.Label= Probe.Label,
                          Analyte.Type= Analyte.Type,
                          Is.Control=Is.Control,
                          Control.Type = Control.Type,
                          row.names = UniqueID,
                          check.names = F,
                          stringsAsFactors = F)
  
  # Make probe labels unique within analyte by appending index
  # This is for the case of multiple probes of the same analyte
  # type having the same name (different probe ID) as in RLF merge
  # could happen
  #-----------------------------------------------------------
  for(analyte in unique(prb.annot$Analyte.Type)){
    prb.annot[prb.annot$Analyte.Type == analyte,"Probe.Label"] <- make.unique(prb.annot[prb.annot$Analyte.Type == analyte,"Probe.Label"])
  }
  
  
  
  # Add the concentration column if info exists
  mRNA.control.indices <- grep("(^POS_[A-Z]{1}\\([0-9]+[\\.]*[0-9]*\\))|(^NEG_[A-Z]{1}\\([0-9]+[\\.]*[0-9]*\\))" ,prb.annot$Probe.Label)
  if(length(mRNA.control.indices) > 0){
    
    conc <- str_extract(string = prb.annot$Probe.Label[mRNA.control.indices],pattern = "\\([^()]+\\)")
    conc <- substring(conc, 2, nchar(conc)-1)
    prb.annot$Control.Conc <- NA
    prb.annot$Control.Conc[mRNA.control.indices] <- as.numeric(conc)
  }
  
  return(prb.annot)
  
}



#' This function validates the probe annotation entry
#' @param prb.annot
#' @param prb.annotation.id.column
#' @param required.names
#' @param required.complete


validate.pannot.entry <- function(prb.annot,
                                  prb.annotation.id.column,
                                  required.names= c(prb.annotation.id.column, "Probe.Label","Analyte.Type", "Is.Control","Control.Type"),
                                  required.complete = c(prb.annotation.id.column, "Probe.Label","Analyte.Type")){
  
  # warnings.paragraph <- NULL
  # required.names <- c(prb.annotation.id.column, "Probe.Label","Analyte.Type", "Is.Control","Control.Type")
  # required.complete <- c(prb.annotation.id.column, "Probe.Label","Analyte.Type")
  
  missing.req.cols <- incomplete.cols <- character(length = 0)
  
  if(!(is.data.frame(prb.annot) | is.matrix(prb.annot)))
    stop("prb.annot needs to be data.frame or matrix")
  
  # cast as data.frame
  if(is.matrix(prb.annot))
    prb.annot <- data.frame(prb.annot,stringsAsFactors = F,check.names = F)
  
  if(!prb.annotation.id.column %in% colnames(prb.annot))
    stop("No probe ID column is provided in the probe annotations")
  
  if(any(sapply(prb.annot[prb.annotation.id.column],function(x) isTRUE(x =="") | is.na(x))))
    stop("Probe ID in probe annotations has missing elements")
  
  if(any(!required.names %in% colnames(prb.annot)))
    missing.req.cols <- required.names[!required.names %in% colnames(prb.annot)]
  
  
  if(sum(is.na(prb.annot[,required.complete]))!=0 | sum(prb.annot[,required.complete]=="")!=0)
    incomplete.cols <- required.complete[apply(prb.annot[,required.complete],MARGIN = 2,FUN = function(x) (sum(is.na(x)) + sum(x == ""))!=0 )]
  
  
  
  #Drop empty or NA columns:
  #This corrects read.csv sometimes reading an extra empty column
  #It also more broadly allows testing content availablity by colname
  #because if the colname is not there ==> content is not there!
  prb.annot <- prb.annot[,!sapply(as.data.frame(prb.annot),function(x)all(is.na(x)) | all(x=="")),drop=F]
  
  
  
  # If any of the required info is missing (after 
  # removing empty cols) bring them in from default
  #-------------------------------------------------
  cols.to.infer <- unique(c(missing.req.cols,incomplete.cols))
  if(length(cols.to.infer)>0){
    default.prb.annot <- get.default.pannot()
    default.prb.annot <- default.prb.annot[match(prb.annot[,prb.annotation.id.column],default.prb.annot[,prb.annotation.id.column]),]
    for(col.to.infer in cols.to.infer){
      prb.annot[[col.to.infer]] <- switch(class(default.prb.annot[,col.to.infer]),
                                          numeric = NA,
                                          character = "")
      prb.annot[,col.to.infer] <- default.prb.annot[,col.to.infer]
    }
  }
  
  
  
  
  # remove any annotations with NA as unique id
  prb.annot <- prb.annot[!is.na(prb.annot[,prb.annotation.id.column]), ]
  
  if(sum(duplicated(prb.annot[,prb.annotation.id.column]))>0)
    stop("probe ids are not unique")
  
  
  # Harmonize is.Control into 0/1 across ("","true"),(FALSE,TRUE)
  #--------------------------------------------------------------
  if("Is.Control" %in% colnames(prb.annot)){
    if(length(unique(prb.annot$Is.Control))!=2)
      stop("Is.Control column in probe annotation needs to have 2 unique levels/values")
    
    if(is.character(prb.annot$Is.Control))
      prb.annot$Is.Control <- as.numeric(prb.annot$Is.Control %in% c("true","TRUE"))
    prb.annot$Is.Control <- as.numeric(prb.annot$Is.Control)
    
  }
  
  # Assign rownames to be the probe unique ids
  rownames(prb.annot) <- prb.annot[,prb.annotation.id.column]
  
  return(prb.annot)
}



#' This function receives a vector x and cast it as defined by the character vector of the same length. For categorical variables where the desired reference is
#' missing, and the values are digits, it approximate the reference level
#' The supported types are "categorical" or "continuous" 
#' @param x: a vector of length m to be type cast
#' @param ref: a character designating the reference level for the vector x. If x is "continuous", NA is the convention (although not necessary as any value will be ignored)
#' @param type: a character describing the type the variable x to be cast in. It takes on values "categorical","continuous" or "boolean"
# value:
# It returns the variable in the specified type and with the correct reference in case of categorical type.

type.cast <- function(x,ref,type){
  
  # This fucntion is expected to only be used here and possibly might be eliminate, so kept
  # the code self-contained within the type.cast.  Future work could either eliminate or embed
  # the functionality within type.cast
  approximate.reference.level <- function(x,ref){
    
    ref0 = ref
    # if it looks like a numeric ref level:
    if(!is.na(suppressWarnings(as.numeric(ref0))) & !any(is.na(suppressWarnings(as.numeric(x)))) )
    {
      # look for the same number amongst the levels of the variable:
      levels0 = as.numeric(as.character(unique(x)))
      closestlevel = order(abs(as.numeric(ref0) - levels0))[1]
      replacelevel = levels0[closestlevel]
      warning(paste("Assigned reference level",ref0,"had no match in the data.  It is being reassigned to the most similar observed level,",replacelevel))
      return(as.character(replacelevel))
    }else{# and if it's not a numeric reference level:
      stop(paste("Assigned reference level",ref0,"had no match in the data, and we were unable to guess which level should be used in its place"))
    }
    
    
    
  }
  
  if(type == "categorical"){
    if(ref %in% x)
      return(relevel(factor(x),ref = ref))
    
    ref.ap <- approximate.reference.level(x = x,ref = ref)
    
    return(relevel(factor(x),ref = ref.ap))
  }
  
  if(type == "continuous")
    return(as.numeric(x))
  
  # Note we re caset boolean as categorical
  if(type == "boolean")
    return(as.factor(x))
}




#' This function is intended for extracting the ";" seprated set names columns of probe annotation and making a matrix of
#' probe x set 0/1 membership matrix
#' @param validated.prb.annot: probe annotation data.frame
#' @param set.column.id: a character string. Name of the column from which the ";" content is to be extracted
#' @param min.set.size: minimum size of the set of probes within a set. At least one set needes to be larger than min.set.size and sets smaller than min.set.size are dropped.
# 
# @value
# set.matrix a 0/1 matrix of probes by set ids
# warnings.paragraph: warning texts to be passed back for logging purposes

get.set.matrix <- function(validated.prb.annot,set.column.id, min.set.size = 0 ,ensure.cases.insensitivity = F){
  
  warnings.paragraph <- character(0)
  set.membership <- lapply(as.character(validated.prb.annot[,set.column.id]),FUN = function(x) unlist(strsplit(x = gsub(";[[:blank:]]*",";",x),split = ";")))
  set.names <- levels(factor(unlist(set.membership)))
  
  # check if any of the probsets differ only in letter case
  if(sum(duplicated(tolower(set.names)))>0 & ensure.cases.insensitivity){
    warnings.paragraph <- paste(warnings.paragraph,"Warning: Some names in", set.column.id,"differ only in lower/upper case letters used, they will be combined\n")
    require(reshape2)
    tmp <- melt(table(id = tolower(set.names),set.names))
    tmp <- tmp[tmp[,"value"]==1,c("id","set.names")]
    tmp2 <- tmp[!duplicated(tmp$id),]; tmp3 <- tmp2$set.names; names(tmp3) <- tmp2$id
    tmp4 <- data.frame(tmp,value = tmp3[tmp$id])
    map <- as.character(tmp4$value); names(map) <- tmp4$set.names
    set.membership <- lapply(as.character(validated.prb.annot[,set.column.id]),FUN = function(x) unname(map[unlist(strsplit(x = gsub(";[[:blank:]]*",";",x),split = ";"))]))
    validated.prb.annot[,set.column.id] <- unlist(lapply(set.membership,function(x)paste(x,collapse = ";")))
  }
  names(set.membership) <- rownames(validated.prb.annot)
  
  #Ensure at least there is 1 set with larger than "min.set.size" member
  if(sum(min.set.size <= table(unlist(set.membership)))==0)
    stop("At least 1 set needs to have size equal or larger than", min.set.size)
  
  set.names <- levels(factor(unlist(set.membership)))
  set.matrix <- t(as.matrix(data.frame(lapply(set.membership,function(x) as.numeric(table(factor(x,levels = set.names))) ),row.names = set.names,check.names = F)))
  
  # Drop any probeset with fewer than min.set.size members
  too.few.prbs <- colnames(set.matrix)[colSums(set.matrix) < min.set.size]
  if(length(too.few.prbs)>0)
  {
    set.matrix <- set.matrix[,!colnames(set.matrix) %in% too.few.prbs]
    print(paste("Warning: the following probe sets hold less than",min.set.size,"probes and will be discarded:"))
    print(too.few.prbs)
    warnings.paragraph <- paste(warnings.paragraph,"Warning: the following probe sets hold less than",min.set.size,"probes and will be discarded:",paste(too.few.prbs,collapse=", "),"\n")        
  }
  return(list(set.matrix = set.matrix, warnings.paragraph = warnings.paragraph))
}



# Given an annotation data matrix, it translates the annot to aesthetic data.frame of the same structure
annot.to.aes <- function(annot,reverse.columns=F){
  codecols2 = c("coral1","bisque4","cyan3","blueviolet","gold1","brown","darkorchid4","plum","darkslategray","darkturquoise",
                "blue3","darkgoldenrod2","burlywood3","darkgoldenrod4","cornflowerblue","slateblue3",
                "chocolate2","tan","gray60","deepskyblue2","royalblue4")
  codecols2 = c(codecols2,colors())
  contcols = list()
  contcols[[1]] = c("brown","grey","blue")
  contcols[[2]] = c("slateblue","grey","tan1")
  contcols[[3]] = c("gold3","grey","lightslateblue")
  contcols[[4]] = c("magenta4","grey","tan3")
  contcols[[5]] = c("chocolate","grey","darkslateblue")
  for(i in 6:100){contcols[[i]]=contcols[[i-5]]}
  
  if(dim(as.data.frame(annot))[2]>0)
  {
    # matrix for coloring each observation by each variable:
    annotcols = matrix("white",dim(as.data.frame(annot))[1],dim(as.data.frame(annot))[2])
    # list of the unique colors for each annotation
    annotcols2 = list()
    dimnames(annotcols)[[1]] = dimnames(annot)[[1]]
    dimnames(annotcols)[[2]] = dimnames(annot)[[2]]
    colsused = 0; contcolsused=0
    annot.order.1 = which(grepl("\\.Flag", colnames(annot)) == F)
    annot.order.2 = which(grepl("\\.Flag", colnames(annot)) == T)
    #annot.order.1 = 1:dim(as.data.frame(annot))[2]
    if (reverse.columns) annot.order.1 = rev(annot.order.1) #dim(as.data.frame(annot))[2]:1
    for(i in annot.order.1)
    {
      # ID type of covariate
      if(class(annot[,i]) == "numeric")
      {
        annotcols[,i] = colorRampPalette(contcols[[1+contcolsused]])( 101 )[round((annot[,i]-min(annot[,i],na.rm=T))/(max(annot[,i],na.rm=T)-min(annot[,i],na.rm=T))*100+1,0)]
        annotcols2[[i]] = contcols[[1+contcolsused]]
        names(annotcols2[[i]]) = c("Low","Average","High")  
        names(annotcols2)[i] = dimnames(annot)[[2]][i]
        contcolsused = contcolsused+1
      }
      if(class(annot[,i]) == "factor")
      {
        annotcols[,i] = codecols2[(colsused+1):length(codecols2)][annot[,i]]
        #print(unique(annotcols[,i]))
        #annotcols2[[i]] = codecols2[(colsused+1):length(codecols2)][(colsused+1):(length(levels(annot[,i]))-1*is.element(NA,annot[,i])+colsused)]  #<---- changed to not assign a color to missing elements
        annotcols2[[i]] = codecols2[(colsused+1):length(codecols2)][1:(length(levels(annot[,i]))-1*is.element(NA,annot[,i]))]
        #print(annotcols2[[i]])
        names(annotcols2)[i] = dimnames(annot)[[2]][i]
        names(annotcols2[[i]]) = levels(annot[,i])  
        colsused = length(levels(annot[,i]))-1*is.element(NA,annot[,i])+colsused
      }
    }
    for(j in annot.order.2){
      annotcols[which(annot[,j] == "false"),j] <- "lightgoldenrod1"
      annotcols[which(annot[,j] == "true"),j]  <- "darkorchid1"
      annotcols2[[j]] <- c("darkorchid1", "lightgoldenrod1")
      names(annotcols2)[j] = dimnames(annot)[[2]][j]
      names(annotcols2[[j]]) = c("True", "False") 
      
    }
  }
  return(list(annotcols = annotcols, annotcols2 = annotcols2))
}

# Given aes object of annotation it output legneds plots of the specified type in the specified path
render.annot.aes.legends <- function(annotcols, annotcols2,plottypearg, path.results){
  #   ### draw the color legend: for all variables at once:
  
  allcolors = alllegends = c()
  for(i in 1:length(annotcols2))
  {
    allcolors = c(allcolors,col=c(NA,annotcols2[[i]]),NA)
    alllegends = c(alllegends,c(main=names(annotcols2)[i],names(annotcols2[[i]]),NA))
  }
  for(r in 1:length(plottypearg)){
    plottype=plottypearg[r];
    maxwidth = max(nchar(alllegends),na.rm = T)
    tempfilename = drawplot(filename=paste(path.results,"//color legend",sep=""),
                            plottype,width=max(.4,.1+maxwidth*.02),
                            height=1+.3*(length(allcolors)>20)+.3*(length(allcolors)>30))
    tempfilename=gsub(path.results,"results",tempfilename)
    par(mar=c(0,0,0,0))
    frame()
    legend("center",lty=1,lwd=12,col=allcolors,legend=alllegends)
    dev.off()}   
  
  
  
  ### draw the color legend: one for each variable:
  
  for(i in 1:length(annotcols2))
  {
    for(r in 1:length(plottypearg)){
      plottype=plottypearg[r];
      maxwidth = max(nchar(alllegends),na.rm = T)
      tempfilename = drawplot(filename=paste(path.results,"//color legend - ",names(annotcols2)[i],sep=""),
                              plottype,width=max(.4,.1+maxwidth*.02),
                              height=1+.3*(length(annotcols2[[i]])>20)+.3*(length(annotcols2[[i]])>30))
      tempfilename=gsub(path.results,"results",tempfilename)
      par(mar=c(0,0,0,0))
      frame()
      legend("center",lty=1,lwd=12,col=c(NA,annotcols2[[i]]),legend=c(main=names(annotcols2)[i],names(annotcols2[[i]])))
      dev.off()}   
  }
  
}



#' This function receives a character vector and removes 
#' Illegal characters that break the analysis (either in html or plot saving)
#' NOTE: Decided against adding the 255 char limit check as that was very unlikely for geneset name
#' @param x a character vator

swap.illegal.char <- function(x){
  char.2b.changed <- c(STR = "\\*",
                       FR  = "<",
                       TO  = ">",
                       CL  = "\\:",
                       TLD = "\\~",
                       BNG = "\\!",
                       AT  = "\\@",
                       DLR  = "\\$",
                       PRC = "\\%",
                       CR  = "\\^",
                       AP  = "\\`",
                       EQ  = "\\=",
                       CBL = "\\{",
                       CBR = "\\}",
                       BRL = "\\[",
                       BRR = "\\]",
                       PP = "\\|",
                       QZ = "\\?",
                       PLS = "\\+",
                       SQT = "\\'",
                       DQT = "\\\"",
                       FS = "\\/",
                       BS = "\\\\")
  
  # Test case
  # x <- "asdf * < > : ~ ! @ $ % ^ ` = { } [ ] | ? + ' \" / \\ "
  # x
  for(i in 1:length(char.2b.changed)){
    pt <- unname(char.2b.changed[i])
    rpl <- paste(".",names(char.2b.changed)[i],".",sep="")
    x <- gsub(pt,rpl,x = x)
    # cat(pt,"\n")
    # cat(x,"\n")
  }
  return(x)
}


#' This function subsets a matrix while keeping the attributes
#' @param dat a matrix
#' @keep.cols a vector of columns to subset
#' @keep.rows a vector of rows to subset

subset.mat.obj <- function(dat,keep.cols=0:ncol(dat),keep.rows=0:nrow(dat)){
  special.attrs <- c("class","comment", "dim", "dimnames", "names", "row.names","tsp")
  special.attrs.present <- names(attributes(dat))[names(attributes(dat)) %in%  special.attrs]
  attr.i <- attributes(dat)
  attr.i <- attr.i[!names(attr.i) %in% special.attrs]
  
  dat <- dat[keep.rows,keep.cols,drop = F]
  
  attr.list <- list()
  attributes(dat) <- c(sapply(special.attrs.present,FUN = function(x) attr(dat,x),simplify = F,USE.NAMES = T),attr.i)
  
  return(dat)
}


#===========================================
#     get mRNA background from raw
#===========================================
estimate.log2.mRNA.BG.by.lane <- function(raw.dat,
                                          neg.colnames){
  # validation of input
  #--------------------
  if(is.data.frame(raw.dat)){
    numeric.cols <- apply(raw.dat,2,function(x) is.numeric(x))
    if(!all(numeric.cols)){
      warning("non-numeric mRNA raw.dat columns were dropped")
      raw.dat <- raw.dat[,numeric.cols]
    }
    raw.dat <- as.matrix(raw.dat)
  }
  
  
  if(!is.matrix(raw.dat))
    stop("raw.dat mRNA needs to be matrix")
  
  if(!all(neg.colnames %in% colnames(raw.dat)))
    stop("neg.colnames for mRNA data need to be in colnames(raw.dat)")
  
  # Simple Log normal model
  
  bg <- apply(log2(raw.dat[,neg.colnames]+1),1,mean) + qnorm(.975)*apply(log2(raw.dat[,neg.colnames]+1),1,sd)
  names(bg) <- rownames(raw.dat)
  return(bg)
}


#===========================================
#     get protein background from raw
#===========================================
estimate.log2.prot.BG.by.lane <- function(raw.dat,
                                          neg.colnames){
  get.orth.proj <- function(y,x,mod){
    er <- (y - (cbind(1,x) %*% coef(mod)) )/as.numeric(sqrt(t( c(1,-coef(mod)[2])  ) %*% c(1,-coef(mod)[2])))
    ort.proj <- cbind(x,y) - er %*% t(c(-coef(mod)[2],1)/sqrt(c(1,-coef(mod)[2])%*% c(1,-coef(mod)[2])))
    return(ort.proj)
  }
  
  # validation of input
  #--------------------
  if(is.data.frame(raw.dat)){
    numeric.cols <- apply(raw.dat,2,function(x) is.numeric(x))
    if(!all(numeric.cols)){
      warning("non-numeric protein raw.dat columns were dropped")
      raw.dat <- raw.dat[,numeric.cols]
    }
    raw.dat <- as.matrix(raw.dat)
  }
  
  
  if(!is.matrix(raw.dat))
    stop("raw.dat protein needs to be matrix")
  
  if(!all(neg.colnames %in% colnames(raw.dat)))
    stop("neg.colnames for protein data need to be in colnames(raw.dat)")
  
  if(length(neg.colnames)<2)
    stop("for protein data, at least two neg.colnames are needed to estimate varaibility in lane background counts")
  
  #log2 tranform
  prot <- log2(raw.dat + 1)
  
  
  #=============================================
  # Find background (via deming reg model)
  #=============================================
  
  # Here we use the deming package with stdpat(1,0,1,0), which is the classical deming, 
  # note if the ratio of SDs are to be changed via stdpat, pay attention to scaling of sigma accordingly
  # when making CI for the IgGs
  
  # require(deming)
  # dmod <- deming(prot[,neg.colnames[2]]~prot[,neg.colnames[1]],stdpat=c(1,0,1,0))
  # sigma <- dmod$sigma
  # 
  # alpha <- atan(coef(dmod)[2])
  # var.y.on.line <- sigma^2 * sin(alpha)
  # var.x.on.line <- sigma^2 * coef(dmod)[2]
  # var.on.line <- var.x.on.line + var.y.on.line 
  # 
  # background.stats <- data.frame(prot[,neg.colnames])
  # background.stats[[make.names(paste(neg.colnames[2],"hat",sep="."))]] <- cbind(1,prot[,neg.colnames[1]]) %*% dmod$coefficients
  # background.stats[["UL"]] <- background.stats[[make.names(paste(neg.colnames[2],"hat",sep="."))]] +
  #   qnorm(.975)* sqrt(var.on.line * sin(alpha))
  # background.stats[["LL"]] <- background.stats[[make.names(paste(neg.colnames[2],"hat",sep="."))]] -
  #   qnorm(.975)* sqrt(var.on.line * sin(alpha))
  # background.stats[["out"]] <- prot[,neg.colnames[2]]<background.stats[["LL"]]| prot[,neg.colnames[2]]>background.stats[["UL"]]
  # 
  # 
  # # Note: currently the estimate of the variability in background, sigma, 
  # #       is taken wihtout excluding the outliers. The plot below shows the
  # #       difference in trend after excluding the outliers (for eval purposes)
  # visualize <- F
  # if(visualize){
  #   plot(prot[,neg.colnames[1]],prot[,neg.colnames[2]],col = 1+background.stats[["out"]],pch = c(1,19)[1+background.stats[["out"]]],
  #        main = "Deming regression of negatives", xlab =neg.colnames[1], ylab =neg.colnames[2])
  #   abline(coef = dmod$coefficients,lty=1,col=1)
  #   abline(coef = deming(prot[!background.stats[["out"]],neg.colnames[2]]~prot[!background.stats[["out"]],neg.colnames[1]],stdpat=c(1,0,1,0))$coefficients,lty=2,col=2)
  #   abline(coef = coef(lm(prot[,neg.colnames[2]]~prot[,neg.colnames[1]])),lty=3,col=3)
  #   abline(coef = c(mean(prot[,neg.colnames[2]]-prot[,neg.colnames[1]]),1),lty=4,col=4)
  #   legend("bottomright",legend = c("Deming","Deming w/o outliers","OLS","Best fit slope=1"),col = 1:4,lty = 1:4)
  #   
  # }
  # 
  # orth.proj <- get.orth.proj(y = prot[,neg.colnames[2]],x = prot[,neg.colnames[1]],mod = dmod)
  # 
  # # Here we get a representation of the "mean" of the two negatives along with the CI around this est.
  # background.stats[["orth.proj.bar"]] <- apply(orth.proj,1,mean) 
  # background.stats[["opbar.UL"]] <- background.stats[["orth.proj.bar"]] + qnorm(.975)* sqrt( (var.on.line * sin(alpha)+ var.on.line * cos(alpha))/2 )
  # background.stats[["opbar.LL"]] <- background.stats[["orth.proj.bar"]] + qnorm(.025)* sqrt( (var.on.line * sin(alpha)+ var.on.line * cos(alpha))/2 )
  # 
  # # The upper limit of the orthogonal projection is chosen as the threshold for background
  # background.stats[["background.threshold"]] <- background.stats[["opbar.UL"]]
  # deming fails if there is only one point, so perform deming only when there are at least three points to regress
  
  visualize <- F
  if(length(neg.colnames)==2 && nrow(unique(prot[,neg.colnames])) > 2){
    require(deming)
    dmod <- deming(prot[,neg.colnames[2]]~prot[,neg.colnames[1]],stdpat=c(1,0,1,0))
    sigma <- dmod$sigma
    
    alpha <- atan(coef(dmod)[2])
    var.y.on.line <- sigma^2 * sin(alpha)
    var.x.on.line <- sigma^2 * coef(dmod)[2]
    var.on.line <- var.x.on.line + var.y.on.line 
    
    background.stats <- data.frame(prot[,neg.colnames])
    background.stats[[make.names(paste(neg.colnames[2],"hat",sep="."))]] <- cbind(1,prot[,neg.colnames[1]]) %*% dmod$coefficients
    background.stats[["UL"]] <- background.stats[[make.names(paste(neg.colnames[2],"hat",sep="."))]] +
      qnorm(.975)* sqrt(var.on.line * sin(alpha))
    background.stats[["LL"]] <- background.stats[[make.names(paste(neg.colnames[2],"hat",sep="."))]] -
      qnorm(.975)* sqrt(var.on.line * sin(alpha))
    background.stats[["out"]] <- prot[,neg.colnames[2]]<background.stats[["LL"]]| prot[,neg.colnames[2]]>background.stats[["UL"]]
    
    
    # Note: currently the estimate of the variability in background, sigma, 
    #       is taken wihtout excluding the outliers. The plot below shows the
    #       difference in trend after excluding the outliers (for eval purposes)
    
    if(visualize){
      par(las = 1)
      plot(prot[,neg.colnames[1]],prot[,neg.colnames[2]],col = 1+background.stats[["out"]],pch = c(1,19)[1+background.stats[["out"]]],
           main = "Deming regression of negatives", xlab =neg.colnames[1], ylab =neg.colnames[2])
      abline(coef = dmod$coefficients,lty=1,col=1)
      abline(coef = deming(prot[!background.stats[["out"]],neg.colnames[2]]~prot[!background.stats[["out"]],neg.colnames[1]],stdpat=c(1,0,1,0))$coefficients,lty=2,col=2)
      abline(coef = coef(lm(prot[,neg.colnames[2]]~prot[,neg.colnames[1]])),lty=3,col=3)
      abline(coef = c(mean(prot[,neg.colnames[2]]-prot[,neg.colnames[1]]),1),lty=4,col=4)
      legend("bottomright",legend = c("Deming","Deming w/o outliers","OLS","Best fit slope=1"),col = 1:4,lty = 1:4)
      
    }
    
    orth.proj <- get.orth.proj(y = prot[,neg.colnames[2]],x = prot[,neg.colnames[1]],mod = dmod)
    
    # Here we get a representation of the "mean" of the two negatives along with the CI around this est.
    background.stats[["orth.proj.bar"]] <- apply(orth.proj,1,mean) 
    background.stats[["opbar.UL"]] <- background.stats[["orth.proj.bar"]] + qnorm(.975)* sqrt( (var.on.line * sin(alpha)+ var.on.line * cos(alpha))/2 )
    background.stats[["opbar.LL"]] <- background.stats[["orth.proj.bar"]] + qnorm(.025)* sqrt( (var.on.line * sin(alpha)+ var.on.line * cos(alpha))/2 )
    
    # The upper limit of the orthogonal projection is chosen as the threshold for background
    background.stats[["background.threshold"]] <- background.stats[["opbar.UL"]]
    
  } else{
    
    background.stats <- data.frame(prot[,neg.colnames])
    background.stats[["mean"]] <- apply(prot[,neg.colnames],MARGIN = 1,mean)
    background.stats[["sd"]] <- apply(prot[,neg.colnames],MARGIN = 1,sd)
    background.stats[["background.threshold"]] <- background.stats[["mean"]] + qnorm(.975) * background.stats[["sd"]]
    
  }
  
  bg <- background.stats[rownames(prot),"background.threshold"]
  names(bg) <- rownames(raw.dat)
  return(bg)
}



# thresholding.frequency = 0.5
# thresholding.value = 20
# thresholding.perform = TRUE

filter.out.low.counts <- function(raw, prb.annots,thresholding.frequency, thresholding.value,thresholding.perform){
  
  #Validate filtering parameter
  #----------------------------
  thresholding.frequency <- thresholding.frequency[unique(prb.annots[colnames(raw),"Analyte.Type"])]
  thresholding.value <- thresholding.value[unique(prb.annots[colnames(raw),"Analyte.Type"])]
  
  warnings <- NULL
  pruneflag <- FALSE
  prune <- rep(FALSE,ncol(raw)); names(prune) <- colnames(raw)
  pruned.probes <-  character(0)
  if(attr(raw,"preprocessing.status") != "raw" | !thresholding.perform){
    if(!thresholding.perform){
      warnings<- "Warning: raw data was not provided for filtering out low count genes - retaining all genes\n"
      warning(warnings)
    }
    
    return(list(pruneflag = pruneflag, prune = prune,pruned.probes= pruned.probes,warnings=warnings))
  }
  
  
  # Get background if AUTO
  #-----------------------
  for(analyte in names(thresholding.frequency)){
    analyte.prbs <- prb.annots[prb.annots$Analyte.Type==analyte,"ProbeID"]
    
    
    if(thresholding.value[[analyte]] == "AUTO"){
      
      #Default value (i.e. no threshold)
      thresholding.value[[analyte]] <- apply(raw[,analyte.prbs],1,min)
      
      if(analyte == "mRNA"){
        neg.colnames <- rownames(prb.annots)[prb.annots$Control.Type == "Negative"]
        if(length(neg.colnames)>0)
          thresholding.value[[analyte]] <- 2^(estimate.log2.mRNA.BG.by.lane(raw.dat = raw,neg.colnames = neg.colnames))
      }
      
      if(analyte == "protein"){
        neg.colnames <- rownames(prb.annots)[prb.annots$Control.Type == "Protein_NEG"]
        if(length(neg.colnames)>0)
          thresholding.value[[analyte]] <- 2^(estimate.log2.prot.BG.by.lane(raw.dat = raw,neg.colnames = neg.colnames))
      }
      
    }else{
      thresholding.value[[analyte]] <- rep(thresholding.value[[analyte]],nrow(raw))
      names(thresholding.value[[analyte]]) <- rownames(raw)
    }
    
    if(thresholding.frequency[[analyte]] == "AUTO")
      thresholding.frequency[[analyte]] <- .5
    
    
    
        # Prune based on user setting of thresholding frequency
    if (thresholding.frequency[[analyte]] == 0) {
      # Prune all probes if 0 selected
      prune[analyte.prbs] <- TRUE
      if(analyte == "mRNA") {
        warning("Warning: 100% of mRNA probes were pruned. Reduce the frequency threshold above 0.")
        cat("LOG:Warning: 100% of mRNA probes were pruned. Reduce the frequency threshold above 0.",
                file=log,
                sep='\n',
                append=TRUE)
        warnings <- paste(warnings, 
                                            "100% of mRNA probes were pruned. Reduce the frequency threshold above 0.", 
                                            "\n")
      }
      if(analyte == "protein") {
        warning("Warning: 100% of protein probes were pruned. Reduce the frequency threshold above 0.")
        cat("LOG:Warning: 100% of protein probes were pruned. Reduce the frequency threshold above 0.",
                file=log,
                sep='\n',
                append=TRUE)
        warnings <- paste(warnings, 
                                            "100% of protein probes were pruned. Reduce the frequency threshold above 0.", 
                                            "\n")
      }
    } else {  
      # Prune probes with 100% of samples below count threshold if  selected
      if (thresholding.frequency[[analyte]] == 1) {
        prune[analyte.prbs] <- apply(sweep(raw[, analyte.prbs],
                                                                              MARGIN = 1,
                                                                              STATS = thresholding.value[[analyte]],
                                                                              FUN = "<"), 2, mean, na.rm=T) == thresholding.frequency[[analyte]]
      } else {
        # Prune probes with greater than frequency threshold samples below count threshold if between 0 and 1
        if (thresholding.frequency[[analyte]] < 1 & thresholding.frequency[[analyte]] > 0) {
          prune[analyte.prbs] <- apply(sweep(raw[, analyte.prbs],
                                                                                MARGIN = 1,
                                                                                STATS = thresholding.value[[analyte]],
                                                                                FUN = "<"), 2, mean, na.rm=T) > thresholding.frequency[[analyte]]
        }
      }
    }
    
  }
  
  #prune <- apply((raw<thresholding.value),2,mean,na.rm=T)>thresholding.frequency
  prune[is.na(prune)] <- TRUE
  
  # if(mean(prune)>0.8)
  # {
  #   pruneflag <- FALSE
  #   warning("Warning: More than 80% of genes were pruned using these cutoffs - retaining all genes")
  #   cat("LOG:Warning: More than 80% of genes were pruned using these cutoffs - retaining all genes",file=log,sep='\n',append=TRUE)
  #   warnings <- paste(warnings,"Warning: More than 80% of genes were pruned using the given cutoffs - retaining all genes","\n")
  #   prune[prune] <- FALSE
  # }
  
  pruned.probes <- suppressWarnings(as.matrix(dimnames(raw)[[2]][prune]))
  colnames(pruned.probes) <- paste("The below genes were removed for falling below the background level too frequently")
  write.csv(as.matrix(pruned.probes),file = paste(path.to.QC.results, "//genes removed for low signal.csv", sep = ""),row.names=FALSE)
  return(list(pruneflag = pruneflag, prune = prune,pruned.probes= pruned.probes,warnings=warnings))
  
}

#===============
# test case
#===============

test.construct.data.and.annotations <- function(path = "S:/AMashadi-Hossein/Take2/2015_11_02/nsolverDev/JohnBucciCode/nSolver_advanced_local_NEW"){
  setwd(dir = path)
  #draw plot function
  drawplot = function(filename,type = c("png","pdf","tiff","jpg","bmp"),width = 1, height = 1,heatmapres=FALSE)
  {
    # modify filename to remove illegal characters:
    illegalcharacters = c("<",">",":","\"","\\?","\\|","\\*")  #
    # only work on the later part of the filename after the "C://":
    filename1 = substr(filename,1,4)
    filename2 = substr(filename,5,length(strsplit(filename,split="")[[1]]))
    for(i in 1:length(illegalcharacters))
    {
      filename2 = gsub(illegalcharacters[i],"_",filename2);  #print(illegalcharacters[i]); print(filename2)
    }
    filename = paste(filename1,filename2,sep="")
    # modify resolution for heatmaps:
    res = NA; pointsize = 12
    if(heatmapres){res = 200;pointsize = 7}
    
    # run plotting function:
    if(type=="png")
    {
      png(filename=paste(filename,type,sep="."),width=480*width,height=480*height,res=res,pointsize=pointsize)
    }
    if(type=="jpg")
    {
      jpeg(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
    }
    if(type=="bmp")
    {
      bmp(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
    }
    if(type=="pdf")
    {
      pdf(file=paste(filename,type,sep="."),width=7*width,height=7*height)
    }
    if(type=="tiff")
    {
      tiff(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
    }
    return(filename) 
  }
  g <- globalenv()
  assign(x = "data.type",value = "raw",envir = g)
  assign(x = "log",value = "output/log_file.txt", envir = g)
  g$drawplot <- drawplot
  
  #   data.type <- "raw"
  #   log = "log_file.txt"
  
  path.results <- paste(path,"/output",sep="")
  assign("path.results",value = path.results,envir = g)
  dir.create(path = path.results,showWarnings = F)
  
  
  #####
  ##### edited by jbucci 12/1/2015
  #####
  
  path.data <- c("ProcessedData_12012015.csv",
                 "RawData_12012015.csv",
                 "BackgroundSubtractedValues_12012015.csv")
  path.sampleannot <- "SampleAnnotations_20151214.csv"
  path.probeannot <- "ProbeAnnotations_12012015.csv"  #"GeneAnnotations.csv"
  prb.annotation.id.column <- "probe.ID" #"Gene.Name"
  prb.set.column <- "prb.sets.a" #"Immune.Response.Category"
  min.prb.set.size <- 3
  sample.annotation.id.column <- "smp.ID"  
  sampleannot.variabletypes = c("categorical","continuous","categorical","categorical")
  sampleannot.variables = c("smp.names", "t","trt","cell.type")
  sampleannot.referencelevels = c("Jurkat 100K_50K-1",NA,"no","Jurkat") 
  
  exclude.probes = NULL    #<-------------- This could be ERCC by default
  exclude.samples = NULL   #<-------------- This could be NULL by default
  draw.color.legend = T
  min.samples = 1
  plottypearg = "pdf"
  
  
  
  out <- construct.data.and.annotations(path.data,
                                        path.sampleannot,
                                        path.probeannot,
                                        prb.annotation.id.column, # <------------ This needs to become static as opposed to a func input
                                        prb.set.column,
                                        min.prb.set.size,
                                        sample.annotation.id.column,   
                                        sampleannot.variables,
                                        sampleannot.variabletypes,
                                        sampleannot.referencelevels,
                                        exclude.probes = NULL,    #<-------------- This could be ERCC by default
                                        exclude.samples = NULL,   #<-------------- This could be NULL by default
                                        draw.color.legend,
                                        min.samples,
                                        plottypearg)
  return(out)
  
}

#test.results <-test.construct.data.and.annotations()
#cat(test.results$warnings.paragraph)


# test.sample.annotation.processing <- function(){
#   #-----------------------------------------
#   drawplot = function(filename,type = c("png","pdf","tiff","jpg","bmp"),width = 1, height = 1,heatmapres=FALSE)
#   {
#     # modify filename to remove illegal characters:
#     illegalcharacters = c("<",">",":","\"","\\?","\\|","\\*")  #
#     # only work on the later part of the filename after the "C://":
#     filename1 = substr(filename,1,4)
#     filename2 = substr(filename,5,length(strsplit(filename,split="")[[1]]))
#     for(i in 1:length(illegalcharacters))
#     {
#       filename2 = gsub(illegalcharacters[i],"_",filename2);  #print(illegalcharacters[i]); print(filename2)
#     }
#     filename = paste(filename1,filename2,sep="")
#     # modify resolution for heatmaps:
#     res = NA; pointsize = 12
#     if(heatmapres){res = 200;pointsize = 7}
#     
#     # run plotting function:
#     if(type=="png")
#     {
#       png(filename=paste(filename,type,sep="."),width=480*width,height=480*height,res=res,pointsize=pointsize)
#     }
#     if(type=="jpg")
#     {
#       jpeg(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
#     }
#     if(type=="bmp")
#     {
#       bmp(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
#     }
#     if(type=="pdf")
#     {
#       pdf(file=paste(filename,type,sep="."),width=7*width,height=7*height)
#     }
#     if(type=="tiff")
#     {
#       tiff(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
#     }
#     return(filename) 
#   }
#   #-----------------------------------------
#   
#   sample.annotation.id.column = "File.Name"
#   smp.annot <- read.csv("S:/AMashadi-Hossein/Take2/2015_11_02/nsolverDev/SampleAnnotations.csv",row.names=sample.annotation.id.column,colClasses="character") 
#   smp.names <- rownames(smp.annot)
#   smp.annot <- smp.annot[-1,]
#   sampleannot.variabletypes = c("categorical", "categorical","continuous","categorical","continuous","continuous","categorical")
#   sampleannot.variables = c("Subtype.Call", "BMI.Cat","BMI", "constant.cat","constant.continuous","warn.continuous","cat.ref.1p2.replaced_w_1p3")
#   sampleannot.referencelevels = c("LuminalB", "1",NA,"const",NA,NA,"1.2") 
#   
#   tmp.annot <- construct.sample.annotation(smp.annot,sampleannot.variables,sampleannot.variabletypes,sampleannot.referencelevels)
#   tmp.aes <- annot.to.aes(annot = tmp.annot$smp.annot)
#   render.annot.aes.legends(annotcols = tmp.aes$annotcols,annotcols2 = tmp.aes$annotcols2,plottypearg = "pdf",path.results = "S:/AMashadi-Hossein/Take2/2015_11_02/nsolverDev/")
# }
# 
# 
# test.data.cleanup <-function(){
#   dat <- read.csv("S:/AMashadi-Hossein/Take2/2015_11_02/nsolverDev/RawData.csv",row.names = 1);dat<-dat[,-ncol(dat)]
# }
# 
