# TFS checked in - 1/25/2016 jbucci 

codecols = c("chartreuse3","darkgoldenrod2","deepskyblue2","firebrick2")
codecols2 = c("chartreuse3","darkgoldenrod2","deepskyblue2","firebrick2","darkorchid1","tan","slateblue3","forestgreen","coral","gray48",
              "aquamarine","bisque2","blue3","blueviolet","brown2","burlywood3","darkgoldenrod4","cyan3","cornflowerblue",
              "chocolate2","chartreuse1","darkorange","darkorchid4","darkred","darkslateblue","darkturquoise","deeppink1",
              "deeppink4","gray7","gold1","dodgerblue1")
codecols2 = c(codecols2,colors())


####### function to harmonize reference levels with actual columns of annot:
# if a reference level isn't present, re-assign it to the most similar level:
harmonize.reference.levels = function(sampleannot.variables,sampleannot.variabletypes,sampleannot.referencelevels,annot)
{
  names(sampleannot.variabletypes) = names(sampleannot.referencelevels) = sampleannot.variables
  
  # for each non-NA ref level, is it present?
  for(i in 1:ncol(annot))
  {
    tempvarname = colnames(annot)[i]
    # if it's categorical:
    if(sampleannot.variabletypes[tempvarname]=="categorical")
    {
      # and if the ref level isn't a match:
      if(!is.element(sampleannot.referencelevels[tempvarname],unique(annot[,tempvarname])))
      {
        ref0 = sampleannot.referencelevels[tempvarname]
        # if it looks like a numeric ref level:
        if(!is.na(as.numeric(ref0)))
        {
          # look for the same number amongst the levels of the variable:
          levels0 = as.numeric(as.character(unique(annot[,tempvarname])))
          closestlevel = order(abs(as.numeric(ref0) - levels0))[1]
          replacelevel = levels0[closestlevel]
          sampleannot.referencelevels[tempvarname] = replacelevel
          warning(paste("Assigned reference level",ref0,"had no match in the data.  It is being reassigned to the most similar observed level,",replacelevel))
        }
        # and if it's not a numeric reference level:
        if(is.na(as.numeric(ref0)))
        {
          stop(paste("Assigned reference level",ref0,"had no match in the data, and we were unable to guess which level should be used in its place"))
        }
      }
    }
  }
  return(sampleannot.referencelevels)
}



#### function to turn a named vector with semicolon-separated entries into a matrix of 1s and 0s:
annotcolumntomatrix = function(x)
{
  # remove spaces after semicolons:
  for(i in 1:5){x = gsub("; ",";",x)}
  # split the vector into a list:
  vals = list()
  uniquevals = c()
  for(i in 1:length(x))
  {
    vals[[i]] = unlist(strsplit(x[i],";"))
    uniquevals = union(uniquevals,vals[[i]])
  }
  uniquevals = uniquevals[order(uniquevals)]
  uniquevals = setdiff(uniquevals,NA)
  # create the matrix:
  mat = matrix(0,length(x),length(uniquevals))
  dimnames(mat) = list(names(x),uniquevals)
  for(i in 1:length(x))
  {
    if(length(intersect(colnames(mat),vals[[i]]))>0){mat[i,intersect(colnames(mat),vals[[i]])] = 1}
  }
  return(mat)
}


##############################################
# functions for data library management

downloadpackages <- function(sourceLocation, libraryLocation)
{
  #windows or Mac?
  osName = Sys.info()['sysname']
  
  #We'll set the mirror URLs if we need them...
  cranURL <- ""  
  biocURL <- ""
  
  #checks if package is installed; if not, function downloads the library from the web
  NSL   <- cbind(read.table(paste(sourceLocation,'//pckg_v_list.csv',sep=''), header = FALSE, sep = ','), Installed = 0, packageVersionInstalled = "r")
  existing.packages <- installed.packages(libraryLocation)
  if(nrow(existing.packages)>0)
    userL <- subset(existing.packages, select = c(Package, Version))
 
  NSL[,sapply(NSL,class)=="factor"] <- as.data.frame(sapply(NSL[,sapply(NSL,class)=="factor"],function(x) as.character(x) ),stringsAsFactors = F)

  for (i in seq(1,length(NSL[,1]))){
    if(nrow(existing.packages)>0){
      for (j in seq(1,length(userL[,1]))){
        if (NSL[i,1]==userL[j,1]){
          NSL[i,5] = as.character(packageVersion(as.character(NSL[i,1])))
          
          #if the version matches OR it's a downloaded package - it's installed
          #for our LIBS packages - version must be an exact match
          if (NSL[i,2]==userL[j,2] || NSL[i,3]!="LIBS") {
            NSL[i,4]=1
            cat(paste(NSL[i,1]," is already installed.",sep=''),file=log,sep='\n',append=TRUE)
          } else {
            cat(paste(NSL[i,1]," is already installed, but a newer version will be installed.",sep=''),file=log,sep='\n',append=TRUE)
          }
          
          break
        }
      }
    }

    if (NSL[i,4]==0){
      if (cranURL == "") {
        cranURL = pickcranmirror()
        
        r <- getOption("repos")
        r["CRAN"] <- cranURL
        options(repos = r)        
      }
      
      if (biocURL == "") {
        biocURL = pickbiocmirror()
      }
      
      # If cranURL is still empty, we were unable to connect to any of our preferred mirrors
      if(cranURL == "" || biocURL == ""){
        cat("document.write('<p><b>- The analysis was stopped -</b></p>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        cat("document.write('<hr/>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        cat("document.write('<p><b>nSolver was unable to download the required R libraries. There may be a problem with your internet connection.</b></p>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        cat("document.write('<p>In some cases, running the analysis again may succeed. You can manually download the required libraries by clicking the link below:</p>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        
        if (osName == 'Windows') {
          cat("document.write('<p><a href=\"http://cdn.nanostring.com/Delivery/SFT/SFT-10057_win.zip\">Download R Libraries for Windows</a></p>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        }
        else if (osName == 'Darwin') {
          cat("document.write('<p><a href=\"https://www.nanostring.com/download_file/787/7179\">Download R Libraries for Mac</a></p>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        }
        
        cat("document.write('<hr/>');", file=paste(path.inc,"//status.js",sep=""), append=TRUE)
        
        stop("Packages cannot be downloaded without internet connection - analysis stopped")
      }
      suppressMessages(source(paste(biocURL, "biocLite.R", sep="")))
      suppressMessages(biocLite(lib = libraryLocation, suppressUpdates=TRUE, suppressAutoUpdate=TRUE))
      if (NSL[i,3]=="BC"){
        suppressMessages(suppressWarnings(biocLite(paste(NSL[i,1],sep=''), lib = libraryLocation, suppressUpdates=TRUE, suppressAutoUpdate=TRUE)))
        cat(paste(NSL[i,1]," is now installed.",sep=''),file=log,sep='\n',append=TRUE)
      }
      if (NSL[i,3]=="CRAN"){
        suppressMessages(install.packages(paste(NSL[i,1],sep=''), repos = cranURL, quiet = TRUE, lib = libraryLocation, dependencies = T, type = 'binary'))
        cat(paste(NSL[i,1]," is now installed.",sep=''),file=log,sep='\n',append=TRUE)
      }
      if (NSL[i,3]=="LIBS"){
        if (osName == 'Windows') {
          suppressMessages(install.packages(path.expand(file.path(sourceLocation, paste(NSL[i,1], '_', NSL[i,2], '.zip', sep=''))), lib = libraryLocation))
        }
        else if (osName == "Darwin") {
          suppressMessages(install.packages(path.expand(file.path(sourceLocation, paste(NSL[i,1], '_', NSL[i,2], '.tgz', sep=''))), repos=NULL, lib = libraryLocation))
        }
        cat(paste(NSL[i,1]," is now installed.",sep=''),file=log,sep='\n',append=TRUE)
      }
    }
    if(i%%5==0||i%%length(NSL[,1])==0){cat(paste("document.write('<p>Loaded ",i,"/",length(NSL[,1])," libraries</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)}
  }
  for (i in seq(1,length(NSL[,1]))){
    if(nrow(existing.packages)>0){
      for (j in seq(1,length(userL[,1]))){
        if (NSL[i,1]==userL[j,1]){
          NSL[i,4]=1
          NSL[i,5] = as.character(packageVersion(as.character(NSL[i,1]), lib.loc = libraryLocation))
          cat(paste(NSL[i,1]," is already installed.",sep=''),file=log,sep='\n',append=TRUE)
          break
        }
      }
    }
    
  }
  write.csv(NSL, paste(sourceLocation,'//pckg_v_list_comparison_',getRversion(),'.csv',sep=''))
}


requirelibraries = function(sourceLocation)
{
#  suppressMessages(require(NormqPCR))
#  suppressMessages(require(gplots))
#  suppressMessages(require(pathview))
#  suppressMessages(require(pathifier))
#  suppressMessages(require(xtable))  
#  suppressMessages(require(deming, lib.loc = .libPaths()))
#  suppressMessages(require(mclust, lib.loc = .libPaths()))

  NSL <- read.table(paste(sourceLocation,'//pckg_v_list.csv',sep=''), header = FALSE, sep = ',')
  
  #Don't attach pathview related packages and rClr
  #-----------------------------------------------
  libs.no.attach <- c("package:pathview","package:org.Hs.eg.db",  "package:RSQLite","package:DBI", "package:AnnotationDbi", "package:GenomeInfoDb",
                        "package:IRanges","package:S4Vectors","package:Biobase","package:BiocGenerics","package:parallel","package:stats4","package:KEGGgraph",
                        "package:graph", "package:XML", "package:rClr")
  NSL <- NSL[!NSL$V1 %in% gsub("package:",replacement = "",libs.no.attach),]
  
  for(i in 1:dim(NSL)[1]){
    suppressMessages(require(noquote(NSL[i,1]), lib.loc = .libPaths(), character.only = T))
  }
}


runloadlibraries = function(){
  print("Loading R libraries")  
  cat("LOG:Loading R libraries",file=log,sep='\n\n',append=TRUE)
  cat("document.write('<p>Loading R libraries</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  
  #' Detach all non basic packages. This is to avoid re-execution conflict 
  #' which is atypical in usual workflow and only happens when script is
  #' executed in the dev environment
  #---------------------------------------------------------------------
  detachAllPackages()
  
  ### install packages and load libraries
  path.source.libs = paste(path.pipeline.base, "libs", sep="/")
  
  osName <- Sys.info()['sysname']
  path.libs <- ""
  
  if(osName == "Windows"){
    path.libs = normalizePath(paste(Sys.getenv("ProgramData"), "nSolverAA_Rlibs", sep="/"))
  }
  else if (osName == "Darwin") {
    path.libs = "~/Library/nSolverAA_Rlibs";
  }
  
  if (!dir.exists(path.libs)){
    dir.create(path.libs, showWarnings = F)
  }
  
  ## Since the 2.0 version of the nSolverAdvanced analysis is using an updated version of R, we
  ## are going to start versioning the libraries. This should likely be updated in future versions.
  path.libs = paste(path.libs, "2.0", sep="/")
  
  if (!dir.exists(path.libs)){
    dir.create(path.libs, showWarnings = F)
  }
  
  ## Set lib path onto .libPaths - don't keep existing path so that other versions of the libraries that may
  ## be installed won't be found.
  .libPaths(c(path.libs) )
    
  cat("installing libraries",file=log,sep='\n',append=TRUE)
  downloadpackages(sourceLocation=path.source.libs,  libraryLocation=path.libs)
  
  cat("loading libraries",file=log,sep='\n',append=TRUE)
  requirelibraries(sourceLocation = path.source.libs)
}




#############################################################
### function to draw a plot using any desired output type
drawplot = function(filename,type = c("png","pdf","tiff","jpg","bmp"),width = 1, height = 1,heatmapres=FALSE)
{
  # modify filename to remove illegal characters:
  illegalcharacters = c("<",">",":","\"","\\?","\\|","\\*")  #
  # only work on the later part of the filename after the "C://":
  filename1 = substr(filename,1,4)
  filename2 = substr(filename,5,length(strsplit(filename,split="")[[1]]))
  for(i in 1:length(illegalcharacters))
  {
    filename2 = gsub(illegalcharacters[i],"_",filename2);  #print(illegalcharacters[i]); print(filename2)
  }
  filename = paste(filename1,filename2,sep="")
  # modify resolution for heatmaps:
  res = NA; pointsize = 12
  if(heatmapres){res = 200;pointsize = 7}
  
  # run plotting function:
  if(type=="png")
  {
    png(filename=paste(filename,type,sep="."),width=480*width,height=480*height,res=res,pointsize=pointsize)
  }
  if(type=="jpg")
  {
    jpeg(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
  }
  if(type=="bmp")
  {
    bmp(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
  }
  if(type=="pdf")
  {
    pdf(file=paste(filename,type,sep="."),width=7*width,height=7*height)
  }
  if(type=="tiff")
  {
    tiff(filename=paste(filename,type,sep="."),width=480*width,height=480*height)
  }
  return(filename) 
}
##### function to correct an illegal filename:
correct.filename = function(filename)
{
  # modify filename to remove illegal characters:
  illegalcharacters = c("<",">",":","\"","\\?","\\|","\\*")  
  # only work on the later part of the filename after the "C://":
  filename1 = substr(filename,1,4)
  filename2 = substr(filename,5,length(strsplit(filename,split="")[[1]]))
  for(i in 1:length(illegalcharacters))
  {
    filename2 = gsub(illegalcharacters[i],"_",filename2);  #print(illegalcharacters[i]); print(filename2)
  }
  filename = paste(filename1,filename2,sep="")
  return(filename)
}





############# function to reconcile the arguments in other functions with the annot DF defined by load.and.clean.data:
reconcile.variables.with.annot = function(annot,variables,variabletypes,referencelevels)
{
  ### make variable names legal:
  variables = make.names(variables)
  
  ### remove variables that aren't in annot:
  in.annot = is.element(variables,names(annot))
  if(length(variabletypes)==length(variables)){variabletypes = variabletypes[in.annot]}
  if(length(referencelevels)==length(variables)){referencelevels = referencelevels[in.annot]}
  variables = variables[in.annot]
  
  return(list(variables=variables,variabletypes=variabletypes,referencelevels=referencelevels))  
}



#########
# function to combine a bunch of scores in a trend plot:

trendplot = function(scores,covariate,covariate.name,covariate.type,center=FALSE,scale=FALSE,ylab,lty=NULL,lwd=1)
{
  if(length(lty)==0){lty = rep(1,ncol(scores))}
  # scale scores as appropriate:
  scores = scale(scores,center=center,scale=scale)
  # remove missing data:
  scores = scores[!is.na(covariate),,drop=FALSE]
  covariate = covariate[!is.na(covariate)]
  # make axis label clear:
  if(center&scale){ylab = paste(ylab,"(centered and scaled)")}
  if(!center&scale){ylab = paste(ylab,"(scaled)")}
  if(center&!scale){ylab = paste(ylab,"(centered)")}
  if(covariate.type=="continuous")
  {
    #par(xpd=FALSE)
    layout(cbind(1,2), widths=c(4,2))  # put legend on bottom 1/8th of the chart
    
    plot(0,col=0,xlim=range(covariate),ylim=range(scores),xlab=covariate.name,ylab=ylab)#,bty="L")
    for(i in 1:dim(scores)[2])
    {
      lines(lowess(covariate,scores[,i]),col=codecols2[i],lty=lty[i],lwd=lwd)
    }
    plot.new()
    #par(xpd=TRUE)
    legend("center",lty=lty,lwd=lwd,legend=dimnames(scores)[[2]],col=codecols2[1:dim(scores)[2]], xpd=NA)
  }
  if(covariate.type=="categorical")
  {
    means = matrix(NA,length(levels(covariate)),dim(scores)[2])
    for(i in 1:dim(means)[1])
    {
      means[i,] = apply(scores[covariate==levels(covariate)[i],,drop=FALSE],2,mean)
    }
    layout(cbind(1,2), widths=c(4,2))  # put legend on bottom 1/8th of the chart
    bp = boxplot(scores~covariate,col=0,border=0,xlab="",ylab=ylab,ylim=range(means),las=2)#,bty="L")
    for(i in 1:dim(scores)[2])
    {
      lines(1:length(levels(covariate)),means[,i],col=codecols2[i],lty=lty[i],lwd=lwd)
      points(1:length(levels(covariate)),means[,i])
    }
    plot.new()
    #par(xpd=TRUE)
    legend("center",lty=lty,lwd=lwd,legend=dimnames(scores)[[2]],col=codecols2[1:dim(scores)[2]], xpd=NA)
  }
}



# function to test internet connection for pathview:
testCon <- function(url = "http://www.google.com") {
  
  # test the http capabilities of the current R build
  http <- as.logical(capabilities(what = "http/ftp"))
  if (!http) return(FALSE)
  
  # test connection by trying to read first line at url
  test <- try(suppressWarnings(readLines(url, n = 1)), silent = TRUE)  # silent errors
  
  # return FALSE if test is class 'try-error'
  ifelse(inherits(test, "try-error"), FALSE, TRUE)
}


pickcranmirror <- function()
{
  dest = tempfile()
  connected <- FALSE
  
  ## List of potential CRAN mirrors
  ##
  ##  - Amazon CloudFront
  ##  - University of California, Berkeley, CA, US
  ##  - University of Mnster, Germany
  ##  - University of Science and Technology of China
  ##  - The Institute of Statistical Mathematics, Tokyo
  ##  - TENET, Johannesburg
  ##  - GIS-Lab.info, Russia
  URL <- c("https://cloud.r-project.org/",
              "https://cran.cnr.berkeley.edu/",
              "https://cran.uni-muenster.de/",
              "https://mirrors.ustc.edu.cn/CRAN/",
              "https://cran.ism.ac.jp/",
              "http://cran.mirror.ac.za/",
              "https://cran.gis-lab.info/")
  
  elapsed = c(NA, NA, NA, NA, NA, NA, NA)
  
  mirrorList <- data.frame(URL, elapsed)
  
  nms = dim(mirrorList)[1]
  for(i in 1:nms){ 
    m = mirrorList[i,] 
    url = paste(m$URL,"/src/base/NEWS",sep="") 
    t = try(system.time(download.file(url,dest),gcFirst=TRUE)) 
    if(file.exists(dest) && file.info(dest)$size > 0){ 
      connected = TRUE
      file.remove(dest) 
      mirrorList$elapsed[i]=t['elapsed'] 
      if (i==1) break
    }else{ 
      mirrorList$elapsed[i]=NA 
    } 
  }
  
  selected <- ""
  
  if (connected){
    selected = (as.character(mirrorList[which.min(mirrorList$elapsed),1]))
  }
  
  return(selected)
}

pickbiocmirror <- function()
{
  dest = tempfile()
  connected <- FALSE
  
  ## List of potential bioconductor mirrors
  ##
  URL <- c("http://bioconductor.org/",
           "https://bioconductor.statistik.tu-dortmund.de/",
           "https://mirrors.ebi.ac.uk/bioconductor/",
           "https://bioc.ism.ac.jp/",
           "https://mirrors.ustc.edu.cn/bioc/",
           "https://mirror.aarnet.edu.au/pub/bioconductor/")
  
  elapsed = c(NA, NA, NA, NA, NA, NA)
  
  mirrorList <- data.frame(URL, elapsed)
  
  nms = dim(mirrorList)[1]
  for(i in 1:nms){ 
    m = mirrorList[i,] 
    url = paste(m$URL,"",sep="") 
    t = try(system.time(download.file(url,dest),gcFirst=TRUE)) 
    if(file.exists(dest) && file.info(dest)$size > 0){
      connected = TRUE
      file.remove(dest) 
      mirrorList$elapsed[i]=t['elapsed']
      if (i==1) break
    }else{ 
      mirrorList$elapsed[i]=NA 
    } 
  } 
  
  selected <- ""
  
  if (connected){
    selected = (as.character(mirrorList[which.min(mirrorList$elapsed),1]))
  }
  
  return(selected)
  
}

heatmap.plus = function (x, Rowv = NULL, Colv = if (symm) "Rowv" else NULL, 
                         distfun = dist, hclustfun = hclust, reorderfun = function(d, 
                                                                                   w) reorder(d, w), add.expr, symm = FALSE, revC = identical(Colv, 
                                                                                                                                              "Rowv"), scale = c("row", "column", "none"), na.rm = TRUE, 
                         margins = c(5, 5), ColSideColors, RowSideColors, cexRow = 0.2 + 
                           1/log10(nr), cexCol = 0.2 + 1/log10(nc), labRow = NULL, 
                         labCol = NULL, main = NULL, xlab = NULL, ylab = NULL, keep.dendro = FALSE, 
                         verbose = getOption("verbose"), ...) 
{
  scale <- if (symm && missing(scale)) 
    "none"
  else match.arg(scale)
  if (length(di <- dim(x)) != 2 || !is.numeric(x)) 
    stop("'x' must be a numeric matrix")
  nr <- di[1]
  nc <- di[2]
  #if (nr <= 1 || nc <= 1)                                #<------- new 3-4
  #  stop("'x' must have at least 2 rows and 2 columns")  #<------- new 3-4
  #  if(length(RowSideColors)>0)
  #  {
  #    if(ncol(RowSideColors)==1)
  #    {
  #      RowSideColors = cbind(RowSideColors,RowSideColors)
  #      colnames(RowSideColors)[2] = " "
  #    }
  #  }
  #  if(length(ColSideColors)>0)
  #  {
  #    if(ncol(ColSideColors)==1)
  #    {
  #      ColSideColors = cbind(ColSideColors,ColSideColors)
  #      colnames(ColSideColors)[2] = " "
  #    }
  #  }
  if (!is.numeric(margins) || length(margins) != 2) 
    stop("'margins' must be a numeric vector of length 2")
  doRdend <- !identical(Rowv, NA)
  doCdend <- !identical(Colv, NA)
  if (is.null(Rowv)) 
    Rowv <- rowMeans(x, na.rm = na.rm)
  if (is.null(Colv)) 
    Colv <- colMeans(x, na.rm = na.rm)
  if (doRdend) {
    if (inherits(Rowv, "dendrogram")) 
      ddr <- Rowv
    else {
      hcr <- hclustfun(distfun(x))
      ddr <- as.dendrogram(hcr)
      if (!is.logical(Rowv) || Rowv) 
        ddr <- reorderfun(ddr, Rowv)
    }
    if (nr != length(rowInd <- order.dendrogram(ddr))) 
      stop("row dendrogram ordering gave index of wrong length")
  }
  else rowInd <- 1:nr
  if (doCdend) {
    if (inherits(Colv, "dendrogram")) 
      ddc <- Colv
    else if (identical(Colv, "Rowv")) {
      if (nr != nc) 
        stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
      ddc <- ddr
    }
    else {
      hcc <- hclustfun(distfun(if (symm) 
        x
        else t(x)))
      ddc <- as.dendrogram(hcc)
      if (!is.logical(Colv) || Colv) 
        ddc <- reorderfun(ddc, Colv)
    }
    if (nc != length(colInd <- order.dendrogram(ddc))) 
      stop("column dendrogram ordering gave index of wrong length")
  }
  else colInd <- 1:nc
  x <- x[rowInd, colInd]
  labRow <- if (is.null(labRow)) 
    if (is.null(rownames(x))) 
      (1:nr)[rowInd]
  else rownames(x)
  else labRow[rowInd]
  labCol <- if (is.null(labCol)) 
    if (is.null(colnames(x))) 
      (1:nc)[colInd]
  else colnames(x)
  else labCol[colInd]
  if (scale == "row") {
    x <- sweep(x, 1, rowMeans(x, na.rm = na.rm))
    sx <- apply(x, 1, sd, na.rm = na.rm)
    x <- sweep(x, 1, sx, "/")
  }
  else if (scale == "column") {
    x <- sweep(x, 2, colMeans(x, na.rm = na.rm))
    sx <- apply(x, 2, sd, na.rm = na.rm)
    x <- sweep(x, 2, sx, "/")
  }
  lmat <- rbind(c(NA, 3), 2:1)
  lwid <- c(if (doRdend) 1 else 0.05, 4)
  lhei <- c((if (doCdend) 1 else 0.05) + if (!is.null(main)) 0.2 else 0, 4)
  if (!missing(ColSideColors)) {
    if (!is.matrix(ColSideColors)) 
      stop("'ColSideColors' must be a matrix")
    if (!is.character(ColSideColors) || dim(ColSideColors)[1] != 
          nc) 
      stop("'ColSideColors' dim()[2] must be of length ncol(x)")
    lmat <- rbind(lmat[1, ] + 1, c(NA, 1), lmat[2, ] + 1)
    #lhei <- c(lhei[1], 0.2, lhei[2])
    # new code to widen the color bars:
    lhei=c(lhei[1], 0.1*dim(ColSideColors)[2], lhei[2])
    
    
  }
  if (!missing(RowSideColors)) {
    if(ncol(RowSideColors)==1)   #<--------- new 5-8
    {
      RowSideColors = cbind(RowSideColors,RowSideColors)
      colnames(RowSideColors)[2] = " "
    }
    if (!is.matrix(RowSideColors)) 
      stop("'RowSideColors' must be a matrix")
    if (!is.character(RowSideColors) || dim(RowSideColors)[1] != 
          nr) 
      stop("'RowSideColors' must be a character vector of length nrow(x)")
    lmat <- cbind(lmat[, 1] + 1, c(rep(NA, nrow(lmat) - 1), 
                                   1), lmat[, 2] + 1)
    #lwid <- c(lwid[1], 0.2, lwid[2])
    # new code to widen the color bars:
    lwid=c(lwid[1], 0.1*dim(RowSideColors)[2], lwid[2])
    
  }
  lmat[is.na(lmat)] <- 0
  if (verbose) {
    cat("layout: widths = ", lwid, ", heights = ", lhei, 
        "; lmat=\n")
    cat(lmat)
  }
  op <- par(no.readonly = TRUE)
  on.exit(par(op))
  layout(lmat, widths = lwid, heights = lhei, respect = FALSE)
  if (!missing(RowSideColors)) {
    par(mar = c(margins[1], 0, 0, 0.5))
    rsc = RowSideColors[rowInd, ,drop=FALSE]  #<----- changed 3-4
    rsc.colors = matrix()
    rsc.names = names(table(rsc))
    rsc.i = 1
    for (rsc.name in rsc.names) {
      rsc.colors[rsc.i] = rsc.name
      rsc[rsc == rsc.name] = rsc.i
      rsc.i = rsc.i + 1
    }
    rsc = matrix(as.numeric(rsc), nrow = dim(rsc)[1])
    image(t(rsc), col = as.vector(rsc.colors), axes = FALSE)
    if (length(colnames(RowSideColors)) > 0) {
      #axis(1, 0:(dim(rsc)[2] - 1)/(dim(rsc)[2] - 1), colnames(RowSideColors),   
      axis(1, at=0:max(dim(rsc)[2] - 1,0)/max((dim(rsc)[2] - 1),.1), colnames(RowSideColors), #<--- changed 4-16          
           las = 2, tick = FALSE)
    }
  }
  if (!missing(ColSideColors)) {
    if(ncol(ColSideColors)==1) #<--------- new 5-8
    {
      ColSideColors = cbind(ColSideColors,ColSideColors)
      colnames(ColSideColors)[2] = " "
    }
    par(mar = c(0.5, 0, 0, margins[2]))
    csc = ColSideColors[colInd, ,drop=FALSE]  #<----- changed 3-4
    csc.colors = matrix()
    csc.names = names(table(csc))
    csc.i = 1
    for (csc.name in csc.names) {
      csc.colors[csc.i] = csc.name
      csc[csc == csc.name] = csc.i
      csc.i = csc.i + 1
    }
    csc = matrix(as.numeric(csc), nrow = dim(csc)[1])
    image(csc, col = as.vector(csc.colors), axes = FALSE)
    if (length(colnames(ColSideColors)) > 0) {
      #axis(2, 0:(dim(csc)[2] - 1)/(dim(csc)[2] - 1), colnames(ColSideColors), 
      axis(2, at=0:max(dim(csc)[2] - 1,1)/max(dim(csc)[2] - 1,1), colnames(ColSideColors), #<--- changed 4-16
           las = 2, tick = FALSE)
    }
  }
  par(mar = c(margins[1], 0, 0, margins[2]))
  if (!symm || scale != "none") {
    x <- t(x)
  }
  if (revC) {
    iy <- nr:1
    ddr <- rev(ddr)
    x <- x[, iy]
  }
  else iy <- 1:nr
  image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 + 
          c(0, nr), axes = FALSE, xlab = "", ylab = "", ...)
  axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0, 
       cex.axis = cexCol)
  if (!is.null(xlab)) 
    mtext(xlab, side = 1, line = margins[1] - 1.25)
  axis(4, iy, labels = labRow, las = 2, line = -0.5, tick = 0, 
       cex.axis = cexRow)
  if (!is.null(ylab)) 
    mtext(ylab, side = 4, line = margins[2] - 1.25)
  if (!missing(add.expr)) 
    eval(substitute(add.expr))
  par(mar = c(margins[1], 0, 0, 0))
  if (doRdend) 
    plot(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
  else frame()
  par(mar = c(0, 0, if (!is.null(main)) 1 else 0, margins[2]))
  if (doCdend) 
    plot(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
  else if (!is.null(main)) 
    frame()
  if (!is.null(main)) 
    title(main, cex.main = 1.5 * op[["cex.main"]])
  invisible(list(rowInd = rowInd, colInd = colInd, Rowv = if (keep.dendro && 
                                                                doRdend) ddr, Colv = if (keep.dendro && doCdend) ddc))  
}


####### new pairs() function: simple pairs() plot, but cex varies with number of points:
pairs.adaptive.cex = function(dat,...)
{
  if(nrow(dat)>50){adaptive.cex=1}
  if(nrow(dat)<=50){adaptive.cex=2}
  par(las = 1)
  pairs(dat,cex=adaptive.cex,...)
}


####### new heatmap function: heatmap.plus, w my earlier modifications, plus the option to have a list of input matrices of diff analytes
# the fn will draw a series of column-aligned heatmaps in which each analyte's data matrix has clustered rows
heatmap.plus.multipanel = function(Xlist,rowcolorlist,...)
{
  
  elipsis.parm <- list(...)
  
  # step 0: pre-processing:
  # if only a matrix is entered, turn it into a list of length 1. 
  if(is.matrix(Xlist)){Xlist = list(Xlist)}
  
  # #drop NA columns
  # Xlist <- sapply(Xlist,function(x) t(t(x)[complete.cases(t(x)),]))
  #drop any element which has any dimension zero
  Xlist[sapply(Xlist,function(x)any(dim(x)==0))] <- NULL
  if(is.matrix(rowcolorlist)&!missing(rowcolorlist)){rowcolorlist = list(rowcolorlist)}
  
  # create a single matrix from the column-aligned component matrices:
  bigmat = c()
  for(i in 1:length(Xlist))
  {
    bigmat = rbind(bigmat,Xlist[[i]])
  }
  if(!missing(rowcolorlist))
  {
    bigrowcolors = c()
    for(i in 1:length(rowcolorlist))
    {
      bigrowcolors = rbind(bigrowcolors,rowcolorlist[[i]])
    }  
  }
  
  # step 1: get the dendogram of each matrix in the list, and combine into a reordering of the big matrix (including line breaks)
  reorder.row = c(); 
  for(i in 1:length(Xlist))
  {
    tempmat = Xlist[[i]]
    if(nrow(Xlist[[i]])>1){tempclust = hclust(dist(tempmat))$order}
    if(nrow(Xlist[[i]])==1){tempclust = 1}
    #dends[[i]] = hclust(dist(tempmat))$order
    reorder.row = c(reorder.row,tempclust+length(reorder.row)-i+1)
    # add a separator between the diff matrices:
    if(i<length(Xlist)){reorder.row = c(reorder.row,NA)}
  }
  # step 2: get the dendrogram of the combined data:
  #reorder.column = hclust(dist(t(bigmat)))$order
  reorder.column = 1:ncol(bigmat)
  # matrix to enter into heatmap.plus:
  x = bigmat[reorder.row,reorder.column]
  keep.not.all.na.smps <- !apply(x,2,function(y) all(is.na(y)))# This is to drop columns with all NA consider instead of all sum>1
  x <- x[,keep.not.all.na.smps,drop=F] 
  elipsis.parm$ColSideColors <- elipsis.parm$ColSideColors[keep.not.all.na.smps,,drop=F]
  x = replace(x,is.na(x),-Inf)
  # flip it:
  x = x[nrow(x):1,,drop=FALSE]
  if(!missing(rowcolorlist))
  {
    RowSideColors = bigrowcolors[reorder.row,,drop=FALSE]
    RowSideColors = replace(RowSideColors,is.na(RowSideColors),"white")
    RowSideColors = RowSideColors[nrow(RowSideColors):1,,drop=FALSE]
  }
  # now plot it:
  
  do.call(heatmap.plus.modified, c(list(x=x,Rowv=NA,RowSideColors=RowSideColors),elipsis.parm))
  # heatmap.plus.modified(x=x,Rowv=NA,RowSideColors=RowSideColors,...)
}  

### below: heatmap.plus modified to work right when symm=TRUE and to accept only 1 row/column of colors
heatmap.plus.modified = function (x, Rowv = NULL, Colv = if (symm) "Rowv" else NULL, 
                         distfun = dist, hclustfun = hclust, reorderfun = function(d, w) reorder(d, w), 
                         add.expr, symm = FALSE, revC = identical(Colv, "Rowv"), scale = c("row", "column", "none"), na.rm = TRUE, 
                         margins = c(5, 5), ColSideColors, RowSideColors, cexRow = 0.2 + 1/log10(nr), cexCol = 0.2 + 1/log10(nc), 
                         labRow = NULL, labCol = NULL, main = NULL, xlab = NULL, ylab = NULL, keep.dendro = FALSE, 
                         verbose = getOption("verbose"), rowsep=NULL,sepwidth=NULL,sepcolor="white",...) 
{
  # a Threshold above which labels are hard to distinguish and won't be added.
  max.col.label.allowed <- max.row.label.allowed <- 100
  
  scale <- if (symm && missing(scale)) 
    "none"
  else match.arg(scale)
  if (length(di <- dim(x)) != 2 || !is.numeric(x)) 
    stop("'x' must be a numeric matrix")
  nr <- di[1]
  nc <- di[2]

  if (!is.numeric(margins) || length(margins) != 2) 
    stop("'margins' must be a numeric vector of length 2")
  doRdend <- !identical(Rowv, NA)
  doCdend <- !identical(Colv, NA)
  if (is.null(Rowv)) 
    Rowv <- rowMeans(x, na.rm = na.rm)
  if (is.null(Colv)) 
    Colv <- colMeans(x, na.rm = na.rm)
  if (doRdend) {
    if (inherits(Rowv, "dendrogram")) 
      ddr <- Rowv
    else {
      hcr <- hclustfun(distfun(x))
      ddr <- as.dendrogram(hcr)
      if (!is.logical(Rowv) || Rowv) 
        ddr <- reorderfun(ddr, Rowv)
    }
    if (nr != length(rowInd <- order.dendrogram(ddr))) 
      stop("row dendrogram ordering gave index of wrong length")
  }
  else rowInd <- 1:nr
  if (doCdend) {
    if (inherits(Colv, "dendrogram")) 
      ddc <- Colv
    else if (identical(Colv, "Rowv")) {
      if (nr != nc) 
        stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
      ddc <- ddr
    }
    else {
      hcc <- hclustfun(distfun(if (symm) 
        x
        else t(x)))
      ddc <- as.dendrogram(hcc)
      if (!is.logical(Colv) || Colv) 
        ddc <- reorderfun(ddc, Colv)
    }
    if (nc != length(colInd <- order.dendrogram(ddc))) 
      stop("column dendrogram ordering gave index of wrong length")
  }
  else colInd <- 1:nc
  x <- x[rowInd, colInd]
  labRow <- if (is.null(labRow)) 
    if (is.null(rownames(x))) 
      (1:nr)[rowInd]
  else rownames(x)
  else labRow[rowInd]
  labCol <- if (is.null(labCol)) 
    if (is.null(colnames(x))) 
      (1:nc)[colInd]
  else colnames(x)
  else labCol[colInd]
  if (scale == "row") {
    x <- sweep(x, 1, rowMeans(x, na.rm = na.rm))
    sx <- apply(x, 1, sd, na.rm = na.rm)
    x <- sweep(x, 1, sx, "/")
  }
  else if (scale == "column") {
    x <- sweep(x, 2, colMeans(x, na.rm = na.rm))
    sx <- apply(x, 2, sd, na.rm = na.rm)
    x <- sweep(x, 2, sx, "/")
  }
  lmat <- rbind(c(NA, 3), 2:1)
  lwid <- c(if (doRdend) 1 else 0.05, 4)
  lhei <- c((if (doCdend) 1 else 0.05) + if (!is.null(main)) 0.2 else 0, 4)
  if (!missing(ColSideColors)) {
    if (!is.matrix(ColSideColors)) 
      stop("'ColSideColors' must be a matrix")
    if (!is.character(ColSideColors) || dim(ColSideColors)[1] != 
          nc) 
      stop("'ColSideColors' dim()[2] must be of length ncol(x)")
    lmat <- rbind(lmat[1, ] + 1, c(NA, 1), lmat[2, ] + 1)
    #lhei <- c(lhei[1], 0.2, lhei[2])
    # new code to widen the color bars:
    lhei=c(lhei[1], 0.1*dim(ColSideColors)[2], lhei[2])
    
    
  }
  if (!missing(RowSideColors)) {
    if(ncol(RowSideColors)==1)   #<--------- new 5-8
    {
      RowSideColors = cbind(RowSideColors,RowSideColors)
      colnames(RowSideColors)[2] = " "
    }
    if (!is.matrix(RowSideColors)) 
      stop("'RowSideColors' must be a matrix")
    if (!is.character(RowSideColors) || dim(RowSideColors)[1] != 
          nr) 
      stop("'RowSideColors' must be a character vector of length nrow(x)")
    lmat <- cbind(lmat[, 1] + 1, c(rep(NA, nrow(lmat) - 1), 
                                   1), lmat[, 2] + 1)
    #lwid <- c(lwid[1], 0.2, lwid[2])
    # new code to widen the color bars:
    lwid=c(lwid[1], 0.1*dim(RowSideColors)[2], lwid[2])
    
  }
  lmat[is.na(lmat)] <- 0
  if (verbose) {
    cat("layout: widths = ", lwid, ", heights = ", lhei, 
        "; lmat=\n")
    cat(lmat)
  }
  op <- par(no.readonly = TRUE)
  on.exit(par(op))
  layout(lmat, widths = lwid, heights = lhei, respect = FALSE)
  if (!missing(RowSideColors)) {
    par(mar = c(margins[1], 0, 0, 0.5))
    rsc = RowSideColors[rowInd, ,drop=FALSE]  #<----- changed 3-4
    rsc.colors = matrix()
    rsc.names = names(table(rsc))
    rsc.i = 1
    for (rsc.name in rsc.names) {
      rsc.colors[rsc.i] = rsc.name
      rsc[rsc == rsc.name] = rsc.i
      rsc.i = rsc.i + 1
    }
    rsc = matrix(as.numeric(rsc), nrow = dim(rsc)[1])
    image(t(rsc), col = as.vector(rsc.colors), axes = FALSE)
    if (length(colnames(RowSideColors)) > 0) {
      #axis(1, 0:(dim(rsc)[2] - 1)/(dim(rsc)[2] - 1), colnames(RowSideColors),   
      axis(1, at=0:max(dim(rsc)[2] - 1,0)/max((dim(rsc)[2] - 1),.1), colnames(RowSideColors), #<--- changed 4-16          
           las = 2, tick = FALSE)
    }
  }
  if (!missing(ColSideColors)) {
    if(ncol(ColSideColors)==1) #<--------- new 5-8
    {
      ColSideColors = cbind(ColSideColors,ColSideColors)
      colnames(ColSideColors)[2] = " "
    }
    par(mar = c(0.5, 0, 0, margins[2]))
    csc = ColSideColors[colInd, ,drop=FALSE]  #<----- changed 3-4
    csc.colors = matrix()
    csc.names = names(table(csc))
    csc.i = 1
    for (csc.name in csc.names) {
      csc.colors[csc.i] = csc.name
      csc[csc == csc.name] = csc.i
      csc.i = csc.i + 1
    }
    csc = matrix(as.numeric(csc), nrow = dim(csc)[1])
    image(csc, col = as.vector(csc.colors), axes = FALSE)
    if (length(colnames(ColSideColors)) > 0) {
      #axis(2, 0:(dim(csc)[2] - 1)/(dim(csc)[2] - 1), colnames(ColSideColors), 
      axis(2, at=0:max(dim(csc)[2] - 1,1)/max(dim(csc)[2] - 1,1), colnames(ColSideColors), #<--- changed 4-16
           las = 2, tick = FALSE)
    }
  }
  
  
  par(mar = c(margins[1], 0, 0, margins[2]))
  if (!symm || scale != "none") {
    x <- t(x)
  }
  if (revC) {
    iy <- nr:1
    ddr <- rev(ddr)
    x <- x[, iy]
  }
  else iy <- 1:nr
  image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 + 
          c(0, nr), axes = FALSE, xlab = "", ylab = "", ...)
  
  if(length(labCol) < max.col.label.allowed)
    axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0, cex.axis = cexCol)
  
  if (!is.null(xlab)) 
    mtext(xlab, side = 1, line = margins[1] - 1.25)
  
  if(length(labRow) < max.row.label.allowed)
    axis(4, iy, labels = labRow, las = 2, line = -0.5, tick = 0, cex.axis = cexRow)
  
  if (!is.null(ylab)) 
    mtext(ylab, side = 4, line = margins[2] - 1.25)
  if (!missing(add.expr)) 
    eval(substitute(add.expr))
  par(mar = c(margins[1], 0, 0, 0))
  if (doRdend) 
    plot(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
  else frame()
  par(mar = c(0, 0, if (!is.null(main)) 1 else 0, margins[2]))
  if (doCdend) 
    plot(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
  else if (!is.null(main)) 
    frame()
  if (!is.null(main)) 
    title(main, cex.main = 1.5 * op[["cex.main"]])
  invisible(list(rowInd = rowInd, colInd = colInd, Rowv = if (keep.dendro && 
                                                                doRdend) ddr, Colv = if (keep.dendro && doCdend) ddc))  
  
}



#### final heatmap function:
heatmap.3 <- function(x,
                      Rowv = TRUE, Colv = if (symm) "Rowv" else TRUE,
                      distfun = dist,
                      hclustfun = hclust,
                      dendrogram = c("both","row", "column", "none"),
                      symm = FALSE,
                      scale = c("none","row", "column"),
                      na.rm = TRUE,
                      revC = identical(Colv,"Rowv"),
                      add.expr,
                      breaks,
                      symbreaks = max(x < 0, na.rm = TRUE) || scale != "none",
                      col = "heat.colors",
                      colsep,
                      rowsep,
                      sepcolor = "white",
                      sepwidth = c(0.05, 0.05),
                      cellnote,
                      notecex = 1,
                      notecol = "cyan",
                      na.color = par("bg"),
                      trace = c("none", "column","row", "both"),
                      tracecol = "cyan",
                      hline = median(breaks),
                      vline = median(breaks),
                      linecol = tracecol,
                      margins = c(5,5),
                      ColSideColors,
                      RowSideColors,
                      side.height.fraction=0.3,
                      cexRow = 0.2 + 1/log10(nr),
                      cexCol = 0.2 + 1/log10(nc),
                      labRow = NULL,
                      labCol = NULL,
                      key = TRUE,
                      keysize = 1.5,
                      density.info = c("none", "histogram", "density"),
                      denscol = tracecol,
                      symkey = max(x < 0, na.rm = TRUE) || symbreaks,
                      densadj = 0.25,
                      main = NULL,
                      xlab = NULL,
                      ylab = NULL,
                      lmat = NULL,
                      lhei = NULL,
                      lwid = NULL,
                      NumColSideColors = 1,
                      NumRowSideColors = 1,
                      KeyValueName="Value",...){
  
  invalid <- function (x) {
    if (missing(x) || is.null(x) || length(x) == 0)
      return(TRUE)
    if (is.list(x))
      return(all(sapply(x, invalid)))
    else if (is.vector(x))
      return(all(is.na(x)))
    else return(FALSE)
  }
  
  x <- as.matrix(x)
  scale01 <- function(x, low = min(x), high = max(x)) {
    x <- (x - low)/(high - low)
    x
  }
  retval <- list()
  scale <- if (symm && missing(scale))
    "none"
  else match.arg(scale)
  dendrogram <- match.arg(dendrogram)
  trace <- match.arg(trace)
  density.info <- match.arg(density.info)
  if (length(col) == 1 && is.character(col))
    col <- get(col, mode = "function")
  if (!missing(breaks) && (scale != "none"))
    warning("Using scale=\"row\" or scale=\"column\" when breaks are",
            "specified can produce unpredictable results.", "Please consider using only one or the other.")
  if (is.null(Rowv) || is.na(Rowv))
    Rowv <- FALSE
  if (is.null(Colv) || is.na(Colv))
    Colv <- FALSE
  else if (Colv == "Rowv" && !isTRUE(Rowv))
    Colv <- FALSE
  if (length(di <- dim(x)) != 2 || !is.numeric(x))
    stop("`x' must be a numeric matrix")
  nr <- di[1]
  nc <- di[2]
  if (nr <= 1 || nc <= 1)
    stop("`x' must have at least 2 rows and 2 columns")
  if (!is.numeric(margins) || length(margins) != 2)
    stop("`margins' must be a numeric vector of length 2")
  if (missing(cellnote))
    cellnote <- matrix("", ncol = ncol(x), nrow = nrow(x))
  if (!inherits(Rowv, "dendrogram")) {
    if (((!isTRUE(Rowv)) || (is.null(Rowv))) && (dendrogram %in%
                                                   c("both", "row"))) {
      if (is.logical(Colv) && (Colv))
        dendrogram <- "column"
      else dedrogram <- "none"
      warning("Discrepancy: Rowv is FALSE, while dendrogram is `",
              dendrogram, "'. Omitting row dendogram.")
    }
  }
  if (!inherits(Colv, "dendrogram")) {
    if (((!isTRUE(Colv)) || (is.null(Colv))) && (dendrogram %in%
                                                   c("both", "column"))) {
      if (is.logical(Rowv) && (Rowv))
        dendrogram <- "row"
      else dendrogram <- "none"
      warning("Discrepancy: Colv is FALSE, while dendrogram is `",
              dendrogram, "'. Omitting column dendogram.")
    }
  }
  if (inherits(Rowv, "dendrogram")) {
    ddr <- Rowv
    rowInd <- order.dendrogram(ddr)
  }
  else if (is.integer(Rowv)) {
    hcr <- hclustfun(distfun(x))
    ddr <- as.dendrogram(hcr)
    ddr <- reorder(ddr, Rowv)
    rowInd <- order.dendrogram(ddr)
    if (nr != length(rowInd))
      stop("row dendrogram ordering gave index of wrong length")
  }
  else if (isTRUE(Rowv)) {
    Rowv <- rowMeans(x, na.rm = na.rm)
    hcr <- hclustfun(distfun(x))
    ddr <- as.dendrogram(hcr)
    ddr <- reorder(ddr, Rowv)
    rowInd <- order.dendrogram(ddr)
    if (nr != length(rowInd))
      stop("row dendrogram ordering gave index of wrong length")
  }
  else {
    rowInd <- nr:1
  }
  if (inherits(Colv, "dendrogram")) {
    ddc <- Colv
    colInd <- order.dendrogram(ddc)
  }
  else if (identical(Colv, "Rowv")) {
    if (nr != nc)
      stop("Colv = \"Rowv\" but nrow(x) != ncol(x)")
    if (exists("ddr")) {
      ddc <- ddr
      colInd <- order.dendrogram(ddc)
    }
    else colInd <- rowInd
  }
  else if (is.integer(Colv)) {
    hcc <- hclustfun(distfun(if (symm)
      x
      else t(x)))
    ddc <- as.dendrogram(hcc)
    ddc <- reorder(ddc, Colv)
    colInd <- order.dendrogram(ddc)
    if (nc != length(colInd))
      stop("column dendrogram ordering gave index of wrong length")
  }
  else if (isTRUE(Colv)) {
    Colv <- colMeans(x, na.rm = na.rm)
    hcc <- hclustfun(distfun(if (symm)
      x
      else t(x)))
    ddc <- as.dendrogram(hcc)
    ddc <- reorder(ddc, Colv)
    colInd <- order.dendrogram(ddc)
    if (nc != length(colInd))
      stop("column dendrogram ordering gave index of wrong length")
  }
  else {
    colInd <- 1:nc
  }
  retval$rowInd <- rowInd
  retval$colInd <- colInd
  retval$call <- match.call()
  x <- x[rowInd, colInd]
  x.unscaled <- x
  cellnote <- cellnote[rowInd, colInd]
  if (is.null(labRow))
    labRow <- if (is.null(rownames(x)))
      (1:nr)[rowInd]
  else rownames(x)
  else labRow <- labRow[rowInd]
  if (is.null(labCol))
    labCol <- if (is.null(colnames(x)))
      (1:nc)[colInd]
  else colnames(x)
  else labCol <- labCol[colInd]
  if (scale == "row") {
    retval$rowMeans <- rm <- rowMeans(x, na.rm = na.rm)
    x <- sweep(x, 1, rm)
    retval$rowSDs <- sx <- apply(x, 1, sd, na.rm = na.rm)
    x <- sweep(x, 1, sx, "/")
  }
  else if (scale == "column") {
    retval$colMeans <- rm <- colMeans(x, na.rm = na.rm)
    x <- sweep(x, 2, rm)
    retval$colSDs <- sx <- apply(x, 2, sd, na.rm = na.rm)
    x <- sweep(x, 2, sx, "/")
  }
  if (missing(breaks) || is.null(breaks) || length(breaks) < 1) {
    if (missing(col) || is.function(col))
      breaks <- 16
    else breaks <- length(col) + 1
  }
  if (length(breaks) == 1) {
    if (!symbreaks)
      breaks <- seq(min(x, na.rm = na.rm), max(x, na.rm = na.rm),
                    length = breaks)
    else {
      extreme <- max(abs(x), na.rm = TRUE)
      breaks <- seq(-extreme, extreme, length = breaks)
    }
  }
  nbr <- length(breaks)
  ncol <- length(breaks) - 1
  if (class(col) == "function")
    col <- col(ncol)
  min.breaks <- min(breaks)
  max.breaks <- max(breaks)
  x[x < min.breaks] <- min.breaks
  x[x > max.breaks] <- max.breaks
  if (missing(lhei) || is.null(lhei))
    lhei <- c(keysize, 4)
  if (missing(lwid) || is.null(lwid))
    lwid <- c(keysize, 4)
  if (missing(lmat) || is.null(lmat)) {
    lmat <- rbind(4:3, 2:1)
    
    if (!missing(ColSideColors)) {
      #if (!is.matrix(ColSideColors))
      #stop("'ColSideColors' must be a matrix")
      if (!is.character(ColSideColors) || nrow(ColSideColors) != nc)
        stop("'ColSideColors' must be a matrix of nrow(x) rows")
      lmat <- rbind(lmat[1, ] + 1, c(NA, 1), lmat[2, ] + 1)
      #lhei <- c(lhei[1], 0.2, lhei[2])
      lhei=c(lhei[1], side.height.fraction*NumColSideColors, lhei[2])
    }
    
    if (!missing(RowSideColors)) 
    {
      if(nrow(RowSideColors)==1)   #<--------- new 5-8
      {
        RowSideColors = rbind(RowSideColors,RowSideColors)
        rownames(RowSideColors)[2] = " "
      }
      #if (!is.matrix(RowSideColors))
      #stop("'RowSideColors' must be a matrix")
      if (!is.character(RowSideColors) || ncol(RowSideColors) != nr)
        stop("'RowSideColors' must be a matrix of ncol(x) columns")
      lmat <- cbind(lmat[, 1] + 1, c(rep(NA, nrow(lmat) - 1), 1), lmat[,2] + 1)
      #lwid <- c(lwid[1], 0.2, lwid[2])
      lwid <- c(lwid[1], side.height.fraction*NumRowSideColors, lwid[2])
    }
    lmat[is.na(lmat)] <- 0
  }
  
  if (length(lhei) != nrow(lmat))
    stop("lhei must have length = nrow(lmat) = ", nrow(lmat))
  if (length(lwid) != ncol(lmat))
    stop("lwid must have length = ncol(lmat) =", ncol(lmat))
  op <- par(no.readonly = TRUE)
  on.exit(par(op))
  
  layout(lmat, widths = lwid, heights = lhei, respect = FALSE)
  
  if (!missing(RowSideColors)) {
    if (!is.matrix(RowSideColors)){
      par(mar = c(margins[1], 0, 0, 0.5))
      image(rbind(1:nr), col = RowSideColors[rowInd], axes = FALSE)
    } else {
      par(mar = c(margins[1], 0, 0, 0.5))
      rsc = t(RowSideColors[,rowInd, drop=F])
      rsc.colors = matrix()
      rsc.names = names(table(rsc))
      rsc.i = 1
      for (rsc.name in rsc.names) {
        rsc.colors[rsc.i] = rsc.name
        rsc[rsc == rsc.name] = rsc.i
        rsc.i = rsc.i + 1
      }
      rsc = matrix(as.numeric(rsc), nrow = dim(rsc)[1])
      image(t(rsc), col = as.vector(rsc.colors), axes = FALSE)
      if (length(rownames(RowSideColors)) > 0) {
        axis(1, 0:(dim(rsc)[2] - 1)/(dim(rsc)[2] - 1), rownames(RowSideColors), las = 2, tick = FALSE)
      }
    }
  }
  
  if (!missing(ColSideColors)) 
  {
    if(ncol(ColSideColors)==1) #<--------- new 5-8
    {
      ColSideColors = cbind(ColSideColors,ColSideColors)
      colnames(ColSideColors)[2] = " "
    }
    
    if (!is.matrix(ColSideColors)){
      par(mar = c(0.5, 0, 0, margins[2]))
      image(cbind(1:nc), col = ColSideColors[colInd], axes = FALSE)
    } else {
      par(mar = c(0.5, 0, 0, margins[2]))
      csc = ColSideColors[colInd, , drop=F]
      csc.colors = matrix()
      csc.names = names(table(csc))
      csc.i = 1
      for (csc.name in csc.names) {
        csc.colors[csc.i] = csc.name
        csc[csc == csc.name] = csc.i
        csc.i = csc.i + 1
      }
      csc = matrix(as.numeric(csc), nrow = dim(csc)[1])
      image(csc, col = as.vector(csc.colors), axes = FALSE)
      if (length(colnames(ColSideColors)) > 0) {
        axis(2, 0:(dim(csc)[2] - 1)/max(1,(dim(csc)[2] - 1)), colnames(ColSideColors), las = 2, tick = FALSE)
      }
    }
  }
  
  par(mar = c(margins[1], 0, 0, margins[2]))
  x <- t(x)
  cellnote <- t(cellnote)
  if (revC) {
    iy <- nr:1
    if (exists("ddr"))
      ddr <- rev(ddr)
    x <- x[, iy]
    cellnote <- cellnote[, iy]
  }
  else iy <- 1:nr
  image(1:nc, 1:nr, x, xlim = 0.5 + c(0, nc), ylim = 0.5 + c(0, nr), axes = FALSE, xlab = "", ylab = "", col = col, breaks = breaks, ...)
  retval$carpet <- x
  if (exists("ddr"))
    retval$rowDendrogram <- ddr
  if (exists("ddc"))
    retval$colDendrogram <- ddc
  retval$breaks <- breaks
  retval$col <- col
  if (!invalid(na.color) & any(is.na(x))) { # load library(gplots)
    mmat <- ifelse(is.na(x), 1, NA)
    image(1:nc, 1:nr, mmat, axes = FALSE, xlab = "", ylab = "",
          col = na.color, add = TRUE)
  }
  axis(1, 1:nc, labels = labCol, las = 2, line = -0.5, tick = 0,
       cex.axis = cexCol)
  if (!is.null(xlab))
    mtext(xlab, side = 1, line = margins[1] - 1.25)
  axis(4, iy, labels = labRow, las = 2, line = -0.5, tick = 0,
       cex.axis = cexRow)
  if (!is.null(ylab))
    mtext(ylab, side = 4, line = margins[2] - 1.25)
  if (!missing(add.expr))
    eval(substitute(add.expr))
  if (!missing(colsep))
    for (csep in colsep) rect(xleft = csep + 0.5, ybottom = rep(0, length(csep)), xright = csep + 0.5 + sepwidth[1], ytop = rep(ncol(x) + 1, csep), lty = 1, lwd = 1, col = sepcolor, border = sepcolor)
  if (!missing(rowsep))
    for (rsep in rowsep) rect(xleft = 0, ybottom = (ncol(x) + 1 - rsep) - 0.5, xright = nrow(x) + 1, ytop = (ncol(x) + 1 - rsep) - 0.5 - sepwidth[2], lty = 1, lwd = 1, col = sepcolor, border = sepcolor)
  min.scale <- min(breaks)
  max.scale <- max(breaks)
  x.scaled <- scale01(t(x), min.scale, max.scale)
  if (trace %in% c("both", "column")) {
    retval$vline <- vline
    vline.vals <- scale01(vline, min.scale, max.scale)
    for (i in colInd) {
      if (!is.null(vline)) {
        abline(v = i - 0.5 + vline.vals, col = linecol,
               lty = 2)
      }
      xv <- rep(i, nrow(x.scaled)) + x.scaled[, i] - 0.5
      xv <- c(xv[1], xv)
      yv <- 1:length(xv) - 0.5
      lines(x = xv, y = yv, lwd = 1, col = tracecol, type = "s")
    }
  }
  if (trace %in% c("both", "row")) {
    retval$hline <- hline
    hline.vals <- scale01(hline, min.scale, max.scale)
    for (i in rowInd) {
      if (!is.null(hline)) {
        abline(h = i + hline, col = linecol, lty = 2)
      }
      yv <- rep(i, ncol(x.scaled)) + x.scaled[i, ] - 0.5
      yv <- rev(c(yv[1], yv))
      xv <- length(yv):1 - 0.5
      lines(x = xv, y = yv, lwd = 1, col = tracecol, type = "s")
    }
  }
  if (!missing(cellnote))
    text(x = c(row(cellnote)), y = c(col(cellnote)), labels = c(cellnote),
         col = notecol, cex = notecex)
  par(mar = c(margins[1], 0, 0, 0))
  if (dendrogram %in% c("both", "row")) {
    plot(ddr, horiz = TRUE, axes = FALSE, yaxs = "i", leaflab = "none")
  }
  else plot.new()
  par(mar = c(0, 0, if (!is.null(main)) 5 else 0, margins[2]))
  if (dendrogram %in% c("both", "column")) {
    plot(ddc, axes = FALSE, xaxs = "i", leaflab = "none")
  }
  else plot.new()
  if (!is.null(main))
    title(main, cex.main = 1.5 * op[["cex.main"]])
  if (key) {
    par(mar = c(5, 4, 2, 1), cex = 0.75)
    tmpbreaks <- breaks
    if (symkey) {
      max.raw <- max(abs(c(x, breaks)), na.rm = TRUE)
      min.raw <- -max.raw
      tmpbreaks[1] <- -max(abs(x), na.rm = TRUE)
      tmpbreaks[length(tmpbreaks)] <- max(abs(x), na.rm = TRUE)
    }
    else {
      min.raw <- min(x, na.rm = TRUE)
      max.raw <- max(x, na.rm = TRUE)
    }
    
    z <- seq(min.raw, max.raw, length = length(col))
    image(z = matrix(z, ncol = 1), col = col, breaks = tmpbreaks,
          xaxt = "n", yaxt = "n")
    par(usr = c(0, 1, 0, 1))
    lv <- pretty(breaks)
    xv <- scale01(as.numeric(lv), min.raw, max.raw)
    axis(1, at = xv, labels = lv)
    if (scale == "row")
      mtext(side = 1, "Row Z-Score", line = 2)
    else if (scale == "column")
      mtext(side = 1, "Column Z-Score", line = 2)
    else mtext(side = 1, KeyValueName, line = 2)
    if (density.info == "density") {
      dens <- density(x, adjust = densadj, na.rm = TRUE)
      omit <- dens$x < min(breaks) | dens$x > max(breaks)
      dens$x <- dens$x[-omit]
      dens$y <- dens$y[-omit]
      dens$x <- scale01(dens$x, min.raw, max.raw)
      lines(dens$x, dens$y/max(dens$y) * 0.95, col = denscol,
            lwd = 1)
      axis(2, at = pretty(dens$y)/max(dens$y) * 0.95, pretty(dens$y))
      title("Color Key\nand Density Plot")
      par(cex = 0.5)
      mtext(side = 2, "Density", line = 2)
    }
    else if (density.info == "histogram") {
      h <- hist(x, plot = FALSE, breaks = breaks)
      hx <- scale01(breaks, min.raw, max.raw)
      hy <- c(h$counts, h$counts[length(h$counts)])
      lines(hx, hy/max(hy) * 0.95, lwd = 1, type = "s",
            col = denscol)
      axis(2, at = pretty(hy)/max(hy) * 0.95, pretty(hy))
      title("Color Key\nand Histogram")
      par(cex = 0.5)
      mtext(side = 2, "Count", line = 2)
    }
    else title("Color Key")
  }
  else plot.new()
  retval$colorTable <- data.frame(low = retval$breaks[-length(retval$breaks)],
                                  high = retval$breaks[-1], color = retval$col)
  invisible(retval)
}

#'This function visualizes a matrix or data frame
#'@param x a matrix or dataframe
#'@param show.val logical. If TRUE it prints the value of each cell
#'@param frmt.val if show.val is TRUE this determins the formatting of the text
#'@param keep.order logical. If TRUE, the order of columns and rows remain intact
#'@param low.col a color as chracter string. The low end of the color heatmap eg. "green"
#'@param high.col a color as character string. The high endo the color gradient eg. "red"
#'@param xlabel a character string. When specified it is labels the colomn names (not the column names themselves)
#'@param ylabel a character string. When specified it is labels the row names (not the row names themselves)

visualize.matrix <- function(x,show.val = FALSE ,frmt.val = "%1.2f",keep.order = T, low.col = NULL, high.col = NULL,xlabel = NULL, ylabel = NULL){
  if(is.data.frame(x))
    x <- as.matrix(x)
  
  require(reshape)
  dat <- melt(t(as.matrix(as.data.frame.matrix(x))))
  dat$label <- sprintf(fmt = eval(frmt.val),dat$value)
  
  if(keep.order){
    dat$X1 <- factor(dat$X1,levels = colnames(x))
    dat$X2 <- factor(dat$X2,levels = rownames(x))
  }
  
  require(ggplot2)
  
  p <- ggplot(data =  dat, aes(x = X1, y = X2)) +
    geom_tile(aes(fill = value), colour = "white") + xlab(xlabel) + ylab(ylabel) + 
    theme(axis.text.x = element_text(angle = 90, hjust = 1))
  
  if(show.val)
    p <- p + geom_text(aes(label = label), vjust = 1) 
  
  if(!is.null(low.col) & !is.null(high.col))
    p <- p + scale_fill_gradient(low = low.col, high = high.col)
  
    
  return(p)
}

# add custom legend overlay to plots
add.legend.fn <- function(...)
{
  opar <- par(fig=c(0, 1, 0, 1), oma=c(0, 0, 0, 0), mar=c(0, 0, 0, 0), new=TRUE)
  on.exit(par(opar))
  plot(0, 0, type='n', bty='n', xaxt='n', yaxt='n')
  legend(...)
}



detachAllPackages <- function() {
  
  basic.packages <- paste("package",sessionInfo()$basePkgs,sep=":") #c("package:stats","package:graphics","package:grDevices","package:utils","package:datasets","package:methods","package:base")
  
  package.list <- search()[ifelse(unlist(gregexpr("package:",search()))==1,TRUE,FALSE)]
  
  package.list <- setdiff(package.list,basic.packages)
  
  if (length(package.list)>0)  for (package in package.list) detach(package, character.only=TRUE)
  
}


#' This function receives annotation data.frame and snv object and append snv calls to the annot and returns the appended 
#' data.frame
#' @param snv output of the snv routine
#' @param annot sample annotation
#' @return annot.appended a data.frame which in addition to the original annotations has the the mutation status at probes where
#' there is at least one sample with mutation at that probe.

append.svn.call.to.annot <- function(snv, annot){
  call.rule <- snv$call.rule$categories
  tmp.call <- snv$snv.f2[,,"snv.call"]
  snv$snv.dat %>% distinct(smp.grouping,.keep_all = T) %>% select(smp.label,smp.grouping)-> names.map
  rownames(names.map) <- names.map$smp.grouping
  
  rownames(tmp.call) <- names.map[rownames(tmp.call),"smp.label"]
  
  annot.appended <- tmp.call[,sapply(as.data.frame(tmp.call,stringsAsFactors = F,check.names = F),function(x) any(x!=0))] %>%
    data.frame(stringsAsFactors = F, check.names = F) %>% cbind(smps = rownames(.),.,stringsAsFactors = F) %>%
    gather(.,key = loci, value = snv.call, 2:ncol(.)) %>%
    mutate(.,snv.call = factor(call.rule$call.type[1+(.$snv.call)],levels = call.rule$call.type))%>% 
    spread(.,key = loci,value = snv.call) %>% dplyr::rename(tmp.smp.ttl.to.mrg = smps) %>%
    left_join(x = annot %>% mutate(tmp.smp.ttl.to.mrg = rownames(.)),
              y = .,
              by = "tmp.smp.ttl.to.mrg") %>%
    `rownames<-` (.$tmp.smp.ttl.to.mrg) %>% select(-tmp.smp.ttl.to.mrg)
  
  return(annot.appended)
}
