#=====================================================
#     Self contained utility functions
#=====================================================

# This function deals with incompatibilities between how 
# the argument file is written as a text file with .R extension 
# and what R expects. For 
# example it casts "NULL" as NULL etc.
# test: "list(mRNA="Dynamic",protein="Dynamic")"

source.arguments <- function(){
  
  #Re-write lists in correct R format
  #-----------------------------------
  txt.arg.input <- readLines("./arguments.r")
  if(!file.exists("arguments_raw.r"))
    writeLines(text = noquote(txt.arg.input), con = "arguments_raw.r")
  list.indices <- grep(pattern ="list([(]{1})",x = txt.arg.input)
  for(i in list.indices){
    var.name <- unlist(strsplit(txt.arg.input[i],split = "\\\""))[1]
    if(length(grep(pattern ="list([(]{1})",x = var.name))!=0)
      next
    
    var.val <- paste(unlist(strsplit(txt.arg.input[i],split = "="))[-1],collapse = "=")
    if(gsub(" ","",var.val) == "\"list()\""){
      txt.arg.input[i] <- paste0(var.name,"list()")
    }else{
      n.components <- sum(unlist(strsplit(var.val,split = ""))=="=")
      component.vals <- lapply(unlist(strsplit(var.val,"=|,|)"))[(1:n.components)*2],function(x)x)
      component.vals[grep("[0-9]",unlist(component.vals))] <- as.numeric(component.vals[grep("[0-9]",unlist(component.vals))])
      component.names <- unlist(strsplit(var.val,"=|\\(|,"))[(1:n.components)*2]
      txt.arg.input[i] <-paste(var.name,"list(",paste(paste(component.names,component.vals,sep="="),collapse = ","),")",sep="")
    }
  }
  
  #Replace "NUL" with NULL 
  txt.arg.input <- gsub(pattern = "(=\"NULL\")|(= \"NULL\")",replacement = noquote("= NULL"),x = txt.arg.input)
  writeLines(text = noquote(txt.arg.input), con = "arguments.r")
  
  
  
  tmp.env <- new.env()
  source("./arguments.r",local = tmp.env)
  
  
  # Add the hardcoded variables:
  #-----------------------------
  if(!exists("probe.annotation.KEGG.id.column",envir = tmp.env))
    assign("probe.annotation.KEGG.id.column" ,value = "KEGG.Pathways",envir = tmp.env)
  if(!exists("probe.annotation.KEGG.Gene.Name",envir = tmp.env))
    assign("probe.annotation.KEGG.Gene.Name" ,value = "Official.Gene.Name",envir = tmp.env)
  if(!exists("min.KEGG.set.size",envir = tmp.env))
    assign("min.KEGG.set.size" ,value = 1,envir = tmp.env)
  if(!exists("celltypes.arg.filePath",envir = tmp.env))
    assign("celltypes.arg.filePath" ,value = "./cell type contrasts matrix.csv",envir = tmp.env)
  
  
  # Make new convenience variables based on argument
  #-------------------------------------------------
  assign("normmodule.arg.run" ,value = !is.null(tmp.env$normmodule.analytes.run),envir = tmp.env)
  
  
  # Change varnames
  #----------------
  if(exists("probeAnnotation",envir = tmp.env)){
    assign("path.probe.annot" ,value = get("probeAnnotation",envir = tmp.env),envir = tmp.env)
    rm(list = c("path.geneannot","probeAnnotation"),envir = tmp.env)
  }
  if(exists("gene.annotation.id.column",envir = tmp.env)){
    assign("probe.annotation.id.column" ,value = "ProbeID",envir = tmp.env)
    rm(list = "gene.annotation.id.column",envir = tmp.env)
  }else{# TEMPORARY HARDCOING OF PROBE ID
    assign("probe.annotation.id.column" ,value = "ProbeID",envir = tmp.env)
  }
  
  if(exists("min.gene.covariate",envir =  tmp.env)){
    assign("min.prb.set.size" ,value = get("min.gene.covariate",envir = tmp.env),envir = tmp.env)
    rm(list = "min.gene.covariate",envir = tmp.env)
  }else{# TEMPORARY HARDCODING OF MIN NUMBER OF PROBES IN A SET. DEFAULT IS NEEDED FROM UI
    assign("min.prb.set.size" ,value = 5, envir = tmp.env)
  }
  
  if(exists("deregmodule.arg.run",envir =  tmp.env)){
    assign("pathscores.arg.run" ,value = get("deregmodule.arg.run",envir = tmp.env),envir = tmp.env)
    rm(list = "deregmodule.arg.run",envir = tmp.env)
  }
  
  if(exists("deregmodule.arg.method",envir =  tmp.env)){
    assign("pathscores.arg.method" ,value = get("deregmodule.arg.method",envir = tmp.env),envir = tmp.env)
    rm(list = "deregmodule.arg.method",envir = tmp.env)
  }
  
  if(exists("covariates.for.pathscoring",envir =  tmp.env)){
    assign("pathscores.arg.covariates" ,value = get("covariates.for.pathscoring",envir = tmp.env),envir = tmp.env)
    rm(list = "covariates.for.pathscoring",envir = tmp.env)
  }
  
  
  if(exists("deregmodule.arg.adjust.for",envir =  tmp.env)){
    assign("pathscores.arg.adjust.for" ,value = get("deregmodule.arg.adjust.for",envir = tmp.env),envir = tmp.env)
    rm(list = "deregmodule.arg.adjust.for",envir = tmp.env)
  }
  
  if(exists("celltypes.arg.geneselection",envir =  tmp.env)){
    assign("celltypes.arg.probeselection" ,value = get("celltypes.arg.geneselection",envir = tmp.env),envir = tmp.env)
    rm(list = "celltypes.arg.geneselection",envir = tmp.env)
  }
  
  
  if(exists("celltypes.arg.geneselection",envir =  tmp.env)){
    assign("celltypes.arg.probeselection" ,value = get("celltypes.arg.geneselection",envir = tmp.env),envir = tmp.env)
    rm(list = "celltypes.arg.geneselection",envir = tmp.env)
  }
  
  if(exists("celltypes.arg.filePath",envir =  tmp.env)){
    if(file.exists(get("celltypes.arg.filePath",envir = tmp.env))){
      assign("celltypes.arg.contrastsfile" ,value = get("celltypes.arg.filePath",envir = tmp.env),envir = tmp.env)
      rm(list = "celltypes.arg.filePath",envir = tmp.env)
    }
  }
  
  # Convert .png, .pdf etc. to png, pdf etc
  #----------------------------------------
  if(exists("plottypearg",envir =  tmp.env)){
    if(!is.null(tmp.env$plottypearg))
      tmp.env$plottypearg <- gsub(pattern = "\\.",replacement = "",tmp.env$plottypearg)
    tmp.env$plottypearg = unique(c("png",tmp.env$plottypearg))
  }
  
  
  
  # Extract SNV reference info
  #---------------------------
  tmp.env$snvarg.run <- FALSE
  if(exists("snvarg.ref.ID.covariate",envir =  tmp.env) & exists("snvarg.ref.ID",envir =  tmp.env)){
    if(!is.null(tmp.env$snvarg.ref.ID) | !is.null(tmp.env$snvarg.ref.ID.covariate))
      tmp.env$snvarg.run  <- TRUE
    if(is.null(tmp.env$snvarg.ref.ID) & !is.null(tmp.env$snvarg.ref.ID.covariate)){
      
      tmp <- 
        tryCatch(expr = {
          annot <- read.csv(tmp.env$path.sampleannot,colClasses="character",check.names = F)
          annot[tolower(as.character(annot[,tmp.env$snvarg.ref.ID.covariate[1]])) == tolower(as.character(tmp.env$snvarg.ref.ID.covariate[2])), tmp.env$sample.annotation.id.column]
        },   
        
        error = function(er) return(er),
        finally ={})
      
      if( length(grep("error", tolower(class(tmp))))==0 ){
        tmp.env$snvarg.ref.ID <- tmp
      }else{
        tmp.env$snvarg.run  <- FALSE
        warnings("could not identify the SNV reference samples from sample annotations")
      }
     
    }
  }
    
  
  
  #======================================================
  #           TEMPORARY TURNING OFF of MODULES
  #======================================================
  #tmp.env$pathscores.arg.run<- FALSE
  #tmp.env$pathviewmodule.arg.run <- FALSE
  
  
  # Load tmp.env to global.env and cast NULL
  # Note: the casting of NULL may be redundant
  #       as we gsub above
  #-----------------------------------------
  arg.names <- ls(envir = tmp.env)
  
  for(arg.i in arg.names){
    
    tmp <- get(x = arg.i,envir = tmp.env)
    assign(x = arg.i,value = tmp,envir = globalenv())
    if(length(tmp)==1){
      
      if(is.character(tmp)){
        if(tmp == "NULL"){
          assign(x = arg.i,value = NULL,envir = globalenv())
          assign(x = arg.i,value = NULL,envir = tmp.env)
        }
        
        if(tmp == "NA"){
          assign(x = arg.i,value = NA,envir = globalenv())
          assign(x = arg.i,value = NA,envir = tmp.env)
        }
        
      } 
    } 
  }
  arg.env = tmp.env
  rm(tmp.env)
  return(arg.env)
  
}








#======================================================
#           Analysis Initialization
#======================================================
analysis.path <- getwd()
path.results <-  path.results0 <- paste(getwd(),"/results",sep="")
dir.create(path.results)
log <- paste(analysis.path,"//log_file.txt",sep="")

# Assign path names
#------------------
path.rsource = paste(getwd(), "//R_source", sep="")
path.resources = paste(getwd(),"//resources",sep="")
path.js = paste(path.resources,"//js",sep="")
path.css = paste(path.resources,"//css",sep="")
path.img = paste(path.resources,"//img",sep="")
path.inc = paste(path.resources,"//inc",sep="")

path.to.normalization.results = paste(path.results,"//Normalization",sep="")
path.to.csvs = paste(path.results,"//csvs",sep="")
path.to.SNV.results = paste(path.results,"//SNV",sep="")
path.to.fusion.results = paste(path.results,"//fusion",sep="")
path.to.DE.results = paste(path.results,"//DE",sep="")
path.to.SPD.results = paste(path.results,"//SPD",sep="") 
path.to.RPD.results = paste(path.results,"//RPD",sep="")
path.to.QC.results = paste(path.results,"//QC",sep="")
path.to.pathview.results = paste(path.results,"//Pathview",sep="")
path.to.GSA.results = paste(path.results,"//Gene set analysis",sep="")
path.to.celltype.results = paste(path.results,"//cell types",sep="")
path.to.pathscore.results = paste(path.results,"//pathway scoring",sep="")


#Create the paths
#---------------
dir.create(path.rsource)
dir.create(path.resources)
dir.create(path.js)
dir.create(path.css)
dir.create(path.img)
dir.create(path.inc)

dir.create(path.to.normalization.results)
dir.create(path.to.csvs)
dir.create(path.to.SNV.results)
dir.create(path.to.fusion.results)
dir.create(path.to.DE.results)
dir.create(path.to.SPD.results)
dir.create(path.to.RPD.results)
dir.create(path.to.QC.results)
dir.create(path.to.pathview.results)
dir.create(path.to.GSA.results)
dir.create(path.to.celltype.results)
dir.create(path.to.pathscore.results)

# Read in the argument file
#--------------------------
cat("loading arguments file",file=log,sep='\n')
arg.env <- source.arguments() # NOTE: arg.env is for convenience it is an environment with all the values of argument files and those modified



# Copy the other R files into their folder
#-----------------------------------------
flistr <- list.files(paste(path.pipeline.base,"//R_source",sep=""),"^.+[.][rR]$",full.names=TRUE)
invisible(file.copy(flistr,path.rsource))


# Load the source files
#----------------------
source("R_source/source - utils.r")
source("R_source/source - run_analysis.R")
# source("R_source/NSDE.r")
# source("R_source/DE_func.r")
source("R_source/source - DE.R")
source("R_source/source - normalization.r") 
source("R_source/source - SPD.R")
source("R_source/source - RPD.R")
source("R_source/source - load_n_clean.R")     # <- add it to util?
source("R_source/species_specific.r")
source("R_source/source - pathview.r")
source("R_source/source - GSA.R")
source("R_source/source - QC.r")
source("R_source/source - signatures.r")
source("R_source/source - cell type profiling.R")
source("R_source/source - SNV_util.R")
source("R_source/source - SNV_eval.R")
source("R_source/source - SNV_plot.R")
source("R_source/Fusion_analysis.r")
source("R_source/advanced_analysis_report.r")
source("R_source/advanced_analysis_report_extended2.r")
source("R_source/write_analysis_report.R")


cat("loaded source",file=log,sep='\n\n',append=TRUE)


#copy static files
#-----------------
flistcss <- list.files(paste(path.pipeline.base,"//css",sep=""),"^.+[.]css$",full.names=TRUE)
invisible(file.copy(flistcss,path.css))
flistjs <- list.files(paste(path.pipeline.base,"//js",sep=""),"^.+[.]js$",full.names=TRUE)
invisible(file.copy(flistjs,path.js))
flistimg <- list.files(paste(path.pipeline.base,"//img",sep=""),"^.+[.].+$",full.names=TRUE)
invisible(file.copy(flistimg,path.img))

#copy the wizard data json, if it was updated
#--------------------------------------------
if (exists("wizardData")) {
  if (file.exists(wizardData)) {
    file.copy(wizardData, paste(path.pipeline.base, "Data", sep="/"), overwrite = TRUE)
  }
}

#======================================================
#                   Run the analyses
#======================================================

result <- tryCatch(expr = run.analysis(),
                   error = function(er){
                     ertmp <- paste("Analysis aborted entirely due to critical failures (see the logs)\n",er)
                     class(ertmp) <- "error"
                     return(ertmp)
                   },
                   finally = {})
warnings()
if(class(result) == "error"){
  print("Analysis could not be completed")
  cat(result)
  cat(paste("LOG: Analysis could not be completed",result),file=log,sep='\n',append=TRUE)
  cat(result,file=log,sep='\n',append=TRUE)
  cat("document.write('<p>Analysis could not be completed</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  tmp <- unlist(strsplit(result,split = "\\n"))
  tmp <- tmp[tmp!="" & tmp!=" "]
  for(err.i in tmp){
    cat(paste("document.write('<p>",err.i,"</p>');"), file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  }
  
  
}else{
  #======================================================
  #                 Generate HTML report
  #======================================================
  # initialize the reporting api
  #------------------------------
  # This function call adds the objects 
  # 1- advAnalysisReport 
  # 2- paramReport
  # to the environment from which it was called from (here: globalenv)
  
  initAdvAnalysisReport(path.pipeline.base) 
  
  reportresult = try(write.analysis.report(result = result,advAnalysisReport = advAnalysisReport, prb.annots = result$prb.annots, arg.env=result$arg.env))
  invisible(file.copy(from = "AdvAnalysisReport.html", to = result$results.basedir))
  
  warnings()
  if(class(reportresult) == "try-error"){
    print("Failed to write analysis report")
    cat("LOG: Failed to write analysis report",file=log,sep='\n',append=TRUE)
    cat(reportresult,file=log,sep='\n',append=TRUE)
    cat("document.write('<p>Analysis could not be completed</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
  }else {
    print("Analysis is complete")
    cat("LOG:Analysis is complete",file=log,sep='\n',append=TRUE)
    cat("document.write('<p>Analysis is complete</p>');", file=paste(path.inc,"//status.js",sep=""),append=TRUE)
    cat("window.location = \"AdvAnalysisReport.html\";", file=paste(path.inc,"//redirect.js",sep=""),append=TRUE)  
  }
  
}
setwd(analysis.path)



